/*  mini86 interpretativni emulator zasticenog flat rezima i386 
 *  Zasnovano na kodu DosBox emulatora,  2002-2009  The DOSBox Team
 */
#include <stdio.h>
#include <stdlib.h>
#include "cpu.h"
#include "flags.h"
#include "table_ea.h"
#include "helpers.h"
#include <string.h>

Bitu DasmI386(char* buffer, HostPt pc, Bitu cur_ip, bool bit32);

static const int PREFIX_NONE		=	0x000;
static const int OPCODE_0F			= 0x100;
static const int PREFIX_SIZE			= 0x200;

static const int PREFIX_ADDR			= 0x1;

static const Bit32u AddrMaskTable[2]={0x0000ffff,0xffffffff};

void print_trace()
{
    FillFlags();
    printf("CF=%1d PF=%1d AF=%1d\tEAX=%08x\tEBX=%08x\tECX=%08x\tEDX=%08x\n",((reg_flags & FLAG_CF) ? true : false ),((reg_flags & FLAG_PF) ? true : false ),((reg_flags & FLAG_AF) ? true : false ),reg_eax,reg_ebx,reg_ecx,reg_edx);
    printf("ZF=%1d SF=%1d OF=%1d\tESP=%08x\tEBP=%08x\tESI=%08x\tEDI=%08x\n",((reg_flags & FLAG_ZF) ? true : false ),((reg_flags & FLAG_SF) ? true : false ),((reg_flags & FLAG_OF) ? true : false ),reg_esp,reg_ebp,reg_esi,reg_edi);
}

void CPU_Core_Simple_Run(void) {
    bool running=true;
    bool trap=true;
    while ( running ) {
        if ( trap )
        {
            char da_line[200];
            DasmI386(da_line, MemBase+reg_eip, reg_eip, true);
            printf("\n%08x: %s\n", reg_eip, da_line);
        }
        core.cseip=(MemBase+reg_eip);
        core.prefix=PREFIX_NONE;
        core.addr32=1;
        core.ea_table=&EATable[256];
restart_opcode:
        switch (core.prefix+Fetchb()) {

        case (PREFIX_NONE+0x01):						/* ADD r/m32, r32 */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    lf_var1d=*eard;
                    lf_var2d=*rmrd;
                    lf_resd=lf_var1d+lf_var2d;
                    *eard=lf_resd;
                    lflags.type=t_ADDd;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1d=host_readd(eaa);
                    lf_var2d=*rmrd;
                    lf_resd=lf_var1d+lf_var2d;
                    host_writed(eaa,lf_resd);
                    lflags.type=t_ADDd;
                }
            }
            break;	
        case (PREFIX_NONE+0x03):						/* ADD r32, r/m32 */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    lf_var1d=*rmrd;
                    lf_var2d=*eard;
                    lf_resd=lf_var1d+lf_var2d;
                    *rmrd=lf_resd;
                    lflags.type=t_ADDd;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1d=*rmrd;
                    lf_var2d=host_readd(eaa);
                    lf_resd=lf_var1d+lf_var2d;
                    *rmrd=lf_resd;
                    lflags.type=t_ADDd;
                }
            }
            break;
        case (PREFIX_NONE+0x05):									/* ADD EAX, imm32 */
            {
                lf_var1d=reg_eax;
                lf_var2d=Fetchd();
                lf_resd=lf_var1d+lf_var2d;
                reg_eax=lf_resd;
                lflags.type=t_ADDd;
            }
            break;
        case (PREFIX_NONE+0x06):									/* PUSH ES */		
            goto illegal_opcode; // not implemented 
        case (PREFIX_NONE+0x07):									/* POP ES */
            goto illegal_opcode; // not implemented 
        case (PREFIX_NONE+0x09):									/* OR r/m32, r32 */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    lf_var1d=*eard;
                    lf_var2d=*rmrd;
                    lf_resd=lf_var1d | lf_var2d;
                    *eard=lf_resd;
                    lflags.type=t_ORd;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1d=host_readd(eaa);
                    lf_var2d=*rmrd;
                    lf_resd=lf_var1d | lf_var2d;
                    host_writed(eaa,lf_resd);
                    lflags.type=t_ORd;
                }
            }
            break;
        case (PREFIX_NONE+0x0b):									/* OR r32, r/m32 */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    lf_var1d=*rmrd;
                    lf_var2d=*eard;
                    lf_resd=lf_var1d | lf_var2d;
                    *rmrd=lf_resd;
                    lflags.type=t_ORd;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1d=*rmrd;
                    lf_var2d=host_readd(eaa);
                    lf_resd=lf_var1d | lf_var2d;
                    *rmrd=lf_resd;
                    lflags.type=t_ORd;
                }
            }
            break;
        case (PREFIX_NONE+0x0d):									/* OR EAX, imm32 */
            {
                lf_var1d=reg_eax;
                lf_var2d=Fetchd();
                lf_resd=lf_var1d | lf_var2d;
                reg_eax=lf_resd;
                lflags.type=t_ORd;
            }
            break;
        case (PREFIX_NONE+0x0e):									/* PUSH CS */		
            goto illegal_opcode; // not implemented 
        case (PREFIX_NONE+0x11):									/* ADC r/m32, r32 */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    lflags.oldcf=get_CF()!=0;
                    lf_var1d=*eard;
                    lf_var2d=*rmrd;
                    lf_resd=lf_var1d+lf_var2d+lflags.oldcf;
                    *eard=lf_resd;
                    lflags.type=t_ADCd;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lflags.oldcf=get_CF()!=0;
                    lf_var1d=host_readd(eaa);
                    lf_var2d=*rmrd;
                    lf_resd=lf_var1d+lf_var2d+lflags.oldcf;
                    host_writed(eaa,lf_resd);
                    lflags.type=t_ADCd;
                }
            }
            break;	
        case (PREFIX_NONE+0x13):									/* ADC r32, r/m32 */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    lflags.oldcf=get_CF()!=0;
                    lf_var1d=*rmrd;
                    lf_var2d=*eard;
                    lf_resd=lf_var1d+lf_var2d+lflags.oldcf;
                    *rmrd=lf_resd;
                    lflags.type=t_ADCd;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lflags.oldcf=get_CF()!=0;
                    lf_var1d=*rmrd;
                    lf_var2d=host_readd(eaa);
                    lf_resd=lf_var1d+lf_var2d+lflags.oldcf;
                    *rmrd=lf_resd;
                    lflags.type=t_ADCd;
                }
            }
            break;
        case (PREFIX_NONE+0x15):									/* ADC EAX, imm32 */
            {
                lflags.oldcf=get_CF()!=0;
                lf_var1d=reg_eax;
                lf_var2d=Fetchd();
                lf_resd=lf_var1d+lf_var2d+lflags.oldcf;
                reg_eax=lf_resd;
                lflags.type=t_ADCd;
            }
            break;
        case (PREFIX_NONE+0x16):									/* PUSH SS */
            goto illegal_opcode; // not implemented 
        case (PREFIX_NONE+0x17):									/* POP SS */
            goto illegal_opcode; // not implemented 
        case (PREFIX_NONE+0x19):									/* SBB r/m32, r32 */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    lflags.oldcf=get_CF()!=0;
                    lf_var1d=*eard;
                    lf_var2d=*rmrd;
                    lf_resd=lf_var1d-(lf_var2d+lflags.oldcf);
                    *eard=lf_resd;
                    lflags.type=t_SBBd;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lflags.oldcf=get_CF()!=0;
                    lf_var1d=host_readd(eaa);
                    lf_var2d=*rmrd;
                    lf_resd=lf_var1d-(lf_var2d+lflags.oldcf);
                    host_writed(eaa,lf_resd);
                    lflags.type=t_SBBd;
                }
            }
            break;
        case (PREFIX_NONE+0x1b):									/* SBB r32, r/m32 */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    lflags.oldcf=get_CF()!=0;
                    lf_var1d=*rmrd;
                    lf_var2d=*eard;
                    lf_resd=lf_var1d-(lf_var2d+lflags.oldcf);
                    *rmrd=lf_resd;
                    lflags.type=t_SBBd;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lflags.oldcf=get_CF()!=0;
                    lf_var1d=*rmrd;
                    lf_var2d=host_readd(eaa);
                    lf_resd=lf_var1d-(lf_var2d+lflags.oldcf);
                    *rmrd=lf_resd;
                    lflags.type=t_SBBd;
                }
            }
            break;
        case (PREFIX_NONE+0x1d):									/* SBB EAX, imm32 */
            {
                lflags.oldcf=get_CF()!=0;
                lf_var1d=reg_eax;
                lf_var2d=Fetchd();
                lf_resd=lf_var1d-(lf_var2d+lflags.oldcf);
                reg_eax=lf_resd;
                lflags.type=t_SBBd;
            }
            break;
        case (PREFIX_NONE+0x1e):									/* PUSH DS */		
            goto illegal_opcode; // not implemented 
        case (PREFIX_NONE+0x1f):									/* POP DS */
            goto illegal_opcode; // not implemented 
        case (PREFIX_NONE+0x21):									/* AND r/m32, r32 */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    lf_var1d=*eard;
                    lf_var2d=*rmrd;
                    lf_resd=lf_var1d & lf_var2d;
                    *eard=lf_resd;
                    lflags.type=t_ANDd;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1d=host_readd(eaa);
                    lf_var2d=*rmrd;
                    lf_resd=lf_var1d & lf_var2d;
                    host_writed(eaa,lf_resd);
                    lflags.type=t_ANDd;
                }
            }
            break;	
        case (PREFIX_NONE+0x23):									/* AND r32, r/m32 */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    lf_var1d=*rmrd;
                    lf_var2d=*eard;
                    lf_resd=lf_var1d & lf_var2d;
                    *rmrd=lf_resd;
                    lflags.type=t_ANDd;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1d=*rmrd;
                    lf_var2d=host_readd(eaa);
                    lf_resd=lf_var1d & lf_var2d;
                    *rmrd=lf_resd;
                    lflags.type=t_ANDd;
                }
            }
            break;
        case (PREFIX_NONE+0x25):									/* AND EAX, imm32 */
            {
                lf_var1d=reg_eax;
                lf_var2d=Fetchd();
                lf_resd=lf_var1d & lf_var2d;
                reg_eax=lf_resd;
                lflags.type=t_ANDd;
            }
            break;
        case (PREFIX_NONE+0x29):									/* SUB r/m32, r32 */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    lf_var1d=*eard;
                    lf_var2d=*rmrd;
                    lf_resd=lf_var1d-lf_var2d;
                    *eard=lf_resd;
                    lflags.type=t_SUBd;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1d=host_readd(eaa);
                    lf_var2d=*rmrd;
                    lf_resd=lf_var1d-lf_var2d;
                    host_writed(eaa,lf_resd);
                    lflags.type=t_SUBd;
                }
            }
            break;
        case (PREFIX_NONE+0x2b):									/* SUB r32, r/m32 */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    lf_var1d=*rmrd;
                    lf_var2d=*eard;
                    lf_resd=lf_var1d-lf_var2d;
                    *rmrd=lf_resd;
                    lflags.type=t_SUBd;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1d=*rmrd;
                    lf_var2d=host_readd(eaa);
                    lf_resd=lf_var1d-lf_var2d;
                    *rmrd=lf_resd;
                    lflags.type=t_SUBd;
                }
            }
            break;
        case (PREFIX_NONE+0x2d):									/* SUB EAX, imm32 */
            {
                lf_var1d=reg_eax;
                lf_var2d=Fetchd();
                lf_resd=lf_var1d-lf_var2d;
                reg_eax=lf_resd;
                lflags.type=t_SUBd;
            }
            break;
        case (PREFIX_NONE+0x31):									/* XOR r/m32, r32 */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    lf_var1d=*eard;
                    lf_var2d=*rmrd;
                    lf_resd=lf_var1d ^ lf_var2d;
                    *eard=lf_resd;
                    lflags.type=t_XORd;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1d=host_readd(eaa);
                    lf_var2d=*rmrd;
                    lf_resd=lf_var1d ^ lf_var2d;
                    host_writed(eaa,lf_resd);
                    lflags.type=t_XORd;
                }
            }
            break;	
        case (PREFIX_NONE+0x33):									/* XOR r32, r/m32 */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    lf_var1d=*rmrd;
                    lf_var2d=*eard;
                    lf_resd=lf_var1d ^ lf_var2d;
                    *rmrd=lf_resd;
                    lflags.type=t_XORd;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1d=*rmrd;
                    lf_var2d=host_readd(eaa);
                    lf_resd=lf_var1d ^ lf_var2d;
                    *rmrd=lf_resd;
                    lflags.type=t_XORd;
                }
            }
            break;
        case (PREFIX_NONE+0x35):									/* XOR EAX, imm32 */
            {
                lf_var1d=reg_eax;
                lf_var2d=Fetchd();
                lf_resd=lf_var1d ^ lf_var2d;
                reg_eax=lf_resd;
                lflags.type=t_XORd;
            }
            break;
        case (PREFIX_NONE+0x39):									/* CMP r/m32, r32 */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    lf_var1d=*eard;
                    lf_var2d=*rmrd;
                    lf_resd=lf_var1d-lf_var2d;
                    lflags.type=t_CMPd;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1d=host_readd(eaa);
                    lf_var2d=*rmrd;
                    lf_resd=lf_var1d-lf_var2d;
                    lflags.type=t_CMPd;
                }
            }
            break;
        case (PREFIX_NONE+0x3b):									/* CMP r32, r/m32 */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    lf_var1d=*rmrd;
                    lf_var2d=*eard;
                    lf_resd=lf_var1d-lf_var2d;
                    lflags.type=t_CMPd;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1d=*rmrd;
                    lf_var2d=host_readd(eaa);
                    lf_resd=lf_var1d-lf_var2d;
                    lflags.type=t_CMPd;
                }
            }
            break;
        case (PREFIX_NONE+0x3d):									/* CMP EAX, imm32 */
            {
                lf_var1d=reg_eax;
                lf_var2d=Fetchd();
                lf_resd=lf_var1d-lf_var2d;
                lflags.type=t_CMPd;
            }
            break;
        case (PREFIX_NONE+0x40):									/* INC EAX */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1d=reg_eax;
            lf_resd=lf_var1d+1;
            reg_eax=lf_resd;
            lflags.type=t_INCd;
            break;
        case (PREFIX_NONE+0x41):									/* INC ECX */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1d=reg_ecx;
            lf_resd=lf_var1d+1;
            reg_ecx=lf_resd;
            lflags.type=t_INCd;
            break;
        case (PREFIX_NONE+0x42):									/* INC EDX */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1d=reg_edx;
            lf_resd=lf_var1d+1;
            reg_edx=lf_resd;
            lflags.type=t_INCd;
            break;
        case (PREFIX_NONE+0x43):									/* INC EBX */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1d=reg_ebx;
            lf_resd=lf_var1d+1;
            reg_ebx=lf_resd;
            lflags.type=t_INCd;
            break;
        case (PREFIX_NONE+0x44):									/* INC ESP */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1d=reg_esp;
            lf_resd=lf_var1d+1;
            reg_esp=lf_resd;
            lflags.type=t_INCd;
            break;
        case (PREFIX_NONE+0x45):									/* INC EBP */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1d=reg_ebp;
            lf_resd=lf_var1d+1;
            reg_ebp=lf_resd;
            lflags.type=t_INCd;
            break;
        case (PREFIX_NONE+0x46):									/* INC ESI */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1d=reg_esi;
            lf_resd=lf_var1d+1;
            reg_esi=lf_resd;
            lflags.type=t_INCd;
            break;
        case (PREFIX_NONE+0x47):									/* INC EDI */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1d=reg_edi;
            lf_resd=lf_var1d+1;
            reg_edi=lf_resd;
            lflags.type=t_INCd;
            break;
        case (PREFIX_NONE+0x48):									/* DEC EAX */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1d=reg_eax;
            lf_resd=lf_var1d-1;
            reg_eax=lf_resd;
            lflags.type=t_DECd;
            break;
        case (PREFIX_NONE+0x49):									/* DEC ECX */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1d=reg_ecx;
            lf_resd=lf_var1d-1;
            reg_ecx=lf_resd;
            lflags.type=t_DECd;
            break;
        case (PREFIX_NONE+0x4a):									/* DEC EDX */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1d=reg_edx;
            lf_resd=lf_var1d-1;
            reg_edx=lf_resd;
            lflags.type=t_DECd;
            break;
        case (PREFIX_NONE+0x4b):									/* DEC EBX */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1d=reg_ebx;
            lf_resd=lf_var1d-1;
            reg_ebx=lf_resd;
            lflags.type=t_DECd;
            break;
        case (PREFIX_NONE+0x4c):									/* DEC ESP */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1d=reg_esp;
            lf_resd=lf_var1d-1;
            reg_esp=lf_resd;
            lflags.type=t_DECd;
            break;
        case (PREFIX_NONE+0x4d):									/* DEC EBP */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1d=reg_ebp;
            lf_resd=lf_var1d-1;
            reg_ebp=lf_resd;
            lflags.type=t_DECd;
            break;
        case (PREFIX_NONE+0x4e):									/* DEC ESI */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1d=reg_esi;
            lf_resd=lf_var1d-1;
            reg_esi=lf_resd;
            lflags.type=t_DECd;
            break;
        case (PREFIX_NONE+0x4f):									/* DEC EDI */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1d=reg_edi;
            lf_resd=lf_var1d-1;
            reg_edi=lf_resd;
            lflags.type=t_DECd;
            break;
        case (PREFIX_NONE+0x50):									/* PUSH EAX */
            CPU_Push32(reg_eax);
            break;
        case (PREFIX_NONE+0x51):									/* PUSH ECX */
            CPU_Push32(reg_ecx);
            break;
        case (PREFIX_NONE+0x52):									/* PUSH EDX */
            CPU_Push32(reg_edx);
            break;
        case (PREFIX_NONE+0x53):									/* PUSH EBX */
            CPU_Push32(reg_ebx);
            break;
        case (PREFIX_NONE+0x54):									/* PUSH ESP */
            CPU_Push32(reg_esp);
            break;
        case (PREFIX_NONE+0x55):									/* PUSH EBP */
            CPU_Push32(reg_ebp);
            break;
        case (PREFIX_NONE+0x56):									/* PUSH ESI */
            CPU_Push32(reg_esi);
            break;
        case (PREFIX_NONE+0x57):									/* PUSH EDI */
            CPU_Push32(reg_edi);
            break;
        case (PREFIX_NONE+0x58):									/* POP EAX */
            reg_eax=CPU_Pop32();
            break;
        case (PREFIX_NONE+0x59):									/* POP ECX */
            reg_ecx=CPU_Pop32();
            break;
        case (PREFIX_NONE+0x5a):									/* POP EDX */
            reg_edx=CPU_Pop32();
            break;
        case (PREFIX_NONE+0x5b):									/* POP EBX */
            reg_ebx=CPU_Pop32();
            break;
        case (PREFIX_NONE+0x5c):									/* POP ESP */
            reg_esp=CPU_Pop32();
            break;
        case (PREFIX_NONE+0x5d):									/* POP EBP */
            reg_ebp=CPU_Pop32();
            break;
        case (PREFIX_NONE+0x5e):									/* POP ESI */
            reg_esi=CPU_Pop32();
            break;
        case (PREFIX_NONE+0x5f):									/* POP EDI */
            reg_edi=CPU_Pop32();
            break;
        case (PREFIX_NONE+0x60):									/* PUSHAD */
            {
                Bitu tmpesp = reg_esp;
                CPU_Push32(reg_eax);CPU_Push32(reg_ecx);CPU_Push32(reg_edx);CPU_Push32(reg_ebx);
                CPU_Push32(tmpesp);CPU_Push32(reg_ebp);CPU_Push32(reg_esi);CPU_Push32(reg_edi);
            }
            break;
        case (PREFIX_NONE+0x61):									/* POPAD */
            reg_edi=CPU_Pop32();
            reg_esi=CPU_Pop32();
            reg_ebp=CPU_Pop32();CPU_Pop32();
            reg_ebx=CPU_Pop32();
            reg_edx=CPU_Pop32();
            reg_ecx=CPU_Pop32();
            reg_eax=CPU_Pop32();
            break;
        case (PREFIX_NONE+0x62):									/* BOUND r/m32 */
            goto illegal_opcode; // not implemented 
        case (PREFIX_NONE+0x63):									/* ARPL r/m32, r32 */
            goto illegal_opcode; // not implemented 
        case (PREFIX_NONE+0x68):									/* PUSH imm32 */
            CPU_Push32(Fetchd());
            break;
        case (PREFIX_NONE+0x69):									/* IMUL r32, r/m32, imm32 */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];{
                        Bit64s res=((Bit64s)((Bit32s)*eard))*((Bit64s)((Bit32s)Fetchd()));
                        *rmrd=(Bit32s)res; FillFlagsNoCFOF();
                        if ((res>-((Bit64s)(2147483647)+1)) && (res<(Bit64s)2147483647)) {
                            reg_flags&=~FLAG_CF;reg_flags&=~FLAG_OF;
                        } else {
                            reg_flags|=FLAG_CF;reg_flags|=FLAG_OF;
                        }
                    }
                } else {
                    HostPt eaa=core.ea_table[rm]();{
                        Bit64s res=((Bit64s)((Bit32s)host_readd(eaa)))*((Bit64s)((Bit32s)Fetchd()));
                        *rmrd=(Bit32s)res; FillFlagsNoCFOF();
                        if ((res>-((Bit64s)(2147483647)+1)) && (res<(Bit64s)2147483647)) {
                            reg_flags&=~FLAG_CF;reg_flags&=~FLAG_OF;
                        } else {
                            reg_flags|=FLAG_CF;reg_flags|=FLAG_OF;
                        }
                    }
                }
            }
            break;
        case (PREFIX_NONE+0x6a):									/* PUSH imm8 */
            CPU_Push32(Fetchb());
            break;
        case (PREFIX_NONE+0x6b):									/* IMUL r32, r/m32, imm8 */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];{
                        Bit64s res=((Bit64s)((Bit32s)*eard))*((Bit64s)((Bit32s)Fetchb()));
                        *rmrd=(Bit32s)res; FillFlagsNoCFOF();
                        if ((res>-((Bit64s)(2147483647)+1)) && (res<(Bit64s)2147483647)) {
                            reg_flags&=~FLAG_CF;reg_flags&=~FLAG_OF;
                        } else {
                            reg_flags|=FLAG_CF;reg_flags|=FLAG_OF;
                        }
                    }
                } else {
                    HostPt eaa=core.ea_table[rm]();{
                        Bit64s res=((Bit64s)((Bit32s)host_readd(eaa)))*((Bit64s)((Bit32s)Fetchb()));
                        *rmrd=(Bit32s)res; FillFlagsNoCFOF();
                        if ((res>-((Bit64s)(2147483647)+1)) && (res<(Bit64s)2147483647)) {
                            reg_flags&=~FLAG_CF;reg_flags&=~FLAG_OF;
                        } else {
                            reg_flags|=FLAG_CF;reg_flags|=FLAG_OF;
                        }
                    }
                }
            }
            break;
        case (PREFIX_NONE+0x6d):									/* INSD */
            goto illegal_opcode; // not implemented 
        case (PREFIX_NONE+0x6f):									/* OUTSD */
            goto illegal_opcode; // not implemented 
        case (PREFIX_NONE+0x70):									/* JO */
            {
                reg_eip=(core.cseip-MemBase);
                if ((get_OF())) reg_eip+=Fetchb();
                reg_eip+=1; continue;
            }
            break;
        case (PREFIX_NONE+0x71):									/* JNO */
            {
                reg_eip=(core.cseip-MemBase);
                if ((!get_OF())) reg_eip+=Fetchb();
                reg_eip+=1; continue;
            }
            break;
        case (PREFIX_NONE+0x72):									/* JB */
            {
                reg_eip=(core.cseip-MemBase);
                if ((get_CF())) reg_eip+=Fetchb();
                reg_eip+=1; continue;
            }
            break;
        case (PREFIX_NONE+0x73):									/* JNB */
            {
                reg_eip=(core.cseip-MemBase);
                if ((!get_CF())) reg_eip+=Fetchb();
                reg_eip+=1; continue;
            }
            break;
        case (PREFIX_NONE+0x74):									/* JZ */
            {
                reg_eip=(core.cseip-MemBase);
                if ((get_ZF())) reg_eip+=Fetchb();
                reg_eip+=1; continue;
            }
            break;
        case (PREFIX_NONE+0x75):									/* JNZ */
            {
                reg_eip=(core.cseip-MemBase);
                if ((!get_ZF())) reg_eip+=Fetchb();
                reg_eip+=1; continue;
            }
            break;
        case (PREFIX_NONE+0x76):									/* JBE */
            {
                reg_eip=(core.cseip-MemBase);
                if ((get_CF() || get_ZF())) reg_eip+=Fetchb();
                reg_eip+=1; continue;
            }
            break;
        case (PREFIX_NONE+0x77):									/* JNBE */
            {
                reg_eip=(core.cseip-MemBase);
                if ((!get_CF() && !get_ZF())) reg_eip+=Fetchb();
                reg_eip+=1; continue;
            }
            break;
        case (PREFIX_NONE+0x78):									/* JS */
            {
                reg_eip=(core.cseip-MemBase);
                if ((get_SF())) reg_eip+=Fetchb();
                reg_eip+=1; continue;
            }
            break;
        case (PREFIX_NONE+0x79):									/* JNS */
            {
                reg_eip=(core.cseip-MemBase);
                if ((!get_SF())) reg_eip+=Fetchb();
                reg_eip+=1; continue;
            }
            break;
        case (PREFIX_NONE+0x7a):									/* JP */
            {
                reg_eip=(core.cseip-MemBase);
                if ((get_PF())) reg_eip+=Fetchb();
                reg_eip+=1; continue;
            }
            break;
        case (PREFIX_NONE+0x7b):									/* JNP */
            {
                reg_eip=(core.cseip-MemBase);
                if ((!get_PF())) reg_eip+=Fetchb();
                reg_eip+=1; continue;
            }
            break;
        case (PREFIX_NONE+0x7c):									/* JL */
            {
                reg_eip=(core.cseip-MemBase);
                if (((get_SF()!=0) != (get_OF()!=0))) reg_eip+=Fetchb();
                reg_eip+=1; continue;
            }
            break;
        case (PREFIX_NONE+0x7d):									/* JNL */
            {
                reg_eip=(core.cseip-MemBase);
                if (((get_SF()!=0) == (get_OF()!=0))) reg_eip+=Fetchb();
                reg_eip+=1; continue;
            }
            break;
        case (PREFIX_NONE+0x7e):									/* JLE */
            {
                reg_eip=(core.cseip-MemBase);
                if ((get_ZF() || ((get_SF()!=0) != (get_OF()!=0)))) reg_eip+=Fetchb();
                reg_eip+=1; continue;
            }
            break;
        case (PREFIX_NONE+0x7f):									/* JNLE */
            {
                reg_eip=(core.cseip-MemBase);
                if ((!get_ZF() && ((get_SF()!=0) == (get_OF()!=0)))) reg_eip+=Fetchb();
                reg_eip+=1; continue;
            }
            break;
        case (PREFIX_NONE+0x81):									/* Grpl r/m32, imm32 */
            {
                Bit8u rm=Fetchb();
                Bitu which=(rm>>3)&7;
                if (rm >= 0xc0) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    Bit32u id=Fetchd();
                    switch (which) {
                    case 0x00:
                        lf_var1d=*eard;
                        lf_var2d=id;
                        lf_resd=lf_var1d+lf_var2d;
                        *eard=lf_resd;
                        lflags.type=t_ADDd;
                        break;
                    case 0x01: lf_var1d=*eard;
                        lf_var2d=id;
                        lf_resd=lf_var1d | lf_var2d;
                        *eard=lf_resd;
                        lflags.type=t_ORd;
                        break;
                    case 0x02:lflags.oldcf=get_CF()!=0;
                        lf_var1d=*eard;
                        lf_var2d=id;
                        lf_resd=lf_var1d+lf_var2d+lflags.oldcf;
                        *eard=lf_resd;
                        lflags.type=t_ADCd;
                        break;
                    case 0x03:lflags.oldcf=get_CF()!=0;
                        lf_var1d=*eard;
                        lf_var2d=id;
                        lf_resd=lf_var1d-(lf_var2d+lflags.oldcf);
                        *eard=lf_resd;
                        lflags.type=t_SBBd;
                        break;
                    case 0x04:
                        lf_var1d=*eard;
                        lf_var2d=id;
                        lf_resd=lf_var1d & lf_var2d;
                        *eard=lf_resd;
                        lflags.type=t_ANDd;
                        break;
                    case 0x05:
                        lf_var1d=*eard;
                        lf_var2d=id;
                        lf_resd=lf_var1d-lf_var2d;
                        *eard=lf_resd;
                        lflags.type=t_SUBd;
                        break;
                    case 0x06:
                        lf_var1d=*eard;
                        lf_var2d=id;
                        lf_resd=lf_var1d ^ lf_var2d;
                        *eard=lf_resd;
                        lflags.type=t_XORd;
                        break;
                    case 0x07:
                        lf_var1d=*eard;
                        lf_var2d=id;
                        lf_resd=lf_var1d-lf_var2d;
                        lflags.type=t_CMPd;
                        break;
                    }
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    Bit32u id=Fetchd();
                    switch (which) {
                    case 0x00:
                        lf_var1d=host_readd(eaa);
                        lf_var2d=id;
                        lf_resd=lf_var1d+lf_var2d;
                        host_writed(eaa,lf_resd);
                        lflags.type=t_ADDd;
                        break;
                    case 0x01: 
                        lf_var1d=host_readd(eaa);
                        lf_var2d=id;
                        lf_resd=lf_var1d | lf_var2d;
                        host_writed(eaa,lf_resd);
                        lflags.type=t_ORd;
                        break;
                    case 0x02:
                        lflags.oldcf=get_CF()!=0;
                        lf_var1d=host_readd(eaa);
                        lf_var2d=id;
                        lf_resd=lf_var1d+lf_var2d+lflags.oldcf;
                        host_writed(eaa,lf_resd);
                        lflags.type=t_ADCd;
                        break;
                    case 0x03:
                        lflags.oldcf=get_CF()!=0;
                        lf_var1d=host_readd(eaa);
                        lf_var2d=id;
                        lf_resd=lf_var1d-(lf_var2d+lflags.oldcf);
                        host_writed(eaa,lf_resd);
                        lflags.type=t_SBBd;
                        break;
                    case 0x04:
                        lf_var1d=host_readd(eaa);
                        lf_var2d=id;
                        lf_resd=lf_var1d & lf_var2d;
                        host_writed(eaa,lf_resd);
                        lflags.type=t_ANDd;
                        break;
                    case 0x05:
                        lf_var1d=host_readd(eaa);
                        lf_var2d=id;
                        lf_resd=lf_var1d-lf_var2d;
                        host_writed(eaa,lf_resd);
                        lflags.type=t_SUBd;
                        break;
                    case 0x06:
                        lf_var1d=host_readd(eaa);
                        lf_var2d=id;
                        lf_resd=lf_var1d ^ lf_var2d;
                        host_writed(eaa,lf_resd);
                        lflags.type=t_XORd;
                        break;
                    case 0x07:
                        lf_var1d=host_readd(eaa);
                        lf_var2d=id;
                        lf_resd=lf_var1d-lf_var2d;
                        lflags.type=t_CMPd;
                        break;
                    }
                }
            }
            break;
        case (PREFIX_NONE+0x83):									/* Grpl r/m32, imm8 */
            {
                Bit8u rm=Fetchb();
                Bitu which=(rm>>3)&7;
                if (rm >= 0xc0) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    Bit32u id=(Bit32s)Fetchb();
                    switch (which) {
                    case 0x00:
                        lf_var1d=*eard;
                        lf_var2d=id;
                        lf_resd=lf_var1d+lf_var2d;
                        *eard=lf_resd;
                        lflags.type=t_ADDd;
                        break;
                    case 0x01: 
                        lf_var1d=*eard;
                        lf_var2d=id;
                        lf_resd=lf_var1d | lf_var2d;
                        *eard=lf_resd;
                        lflags.type=t_ORd;
                        break;
                    case 0x02:
                        lflags.oldcf=get_CF()!=0;
                        lf_var1d=*eard;
                        lf_var2d=id;
                        lf_resd=lf_var1d+lf_var2d+lflags.oldcf;
                        *eard=lf_resd;
                        lflags.type=t_ADCd;
                        break;
                    case 0x03:
                        lflags.oldcf=get_CF()!=0;
                        lf_var1d=*eard;
                        lf_var2d=id;
                        lf_resd=lf_var1d-(lf_var2d+lflags.oldcf);
                        *eard=lf_resd;
                        lflags.type=t_SBBd;
                        break;
                    case 0x04:
                        lf_var1d=*eard;
                        lf_var2d=id;
                        lf_resd=lf_var1d & lf_var2d;
                        *eard=lf_resd;
                        lflags.type=t_ANDd;
                        break;
                    case 0x05:
                        lf_var1d=*eard;
                        lf_var2d=id;
                        lf_resd=lf_var1d-lf_var2d;
                        *eard=lf_resd;
                        lflags.type=t_SUBd;
                        break;
                    case 0x06:
                        lf_var1d=*eard;
                        lf_var2d=id;
                        lf_resd=lf_var1d ^ lf_var2d;
                        *eard=lf_resd;
                        lflags.type=t_XORd;
                        break;
                    case 0x07:
                        lf_var1d=*eard;
                        lf_var2d=id;
                        lf_resd=lf_var1d-lf_var2d;
                        lflags.type=t_CMPd;
                        break;
                    }
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    Bit32u id=(Bit32s)Fetchb();
                    switch (which) {
                    case 0x00:
                        lf_var1d=host_readd(eaa);
                        lf_var2d=id;
                        lf_resd=lf_var1d+lf_var2d;
                        host_writed(eaa,lf_resd);
                        lflags.type=t_ADDd;
                        break;
                    case 0x01: lf_var1d=host_readd(eaa);
                        lf_var2d=id;
                        lf_resd=lf_var1d | lf_var2d;
                        host_writed(eaa,lf_resd);
                        lflags.type=t_ORd;
                        break;
                    case 0x02:lflags.oldcf=get_CF()!=0;
                        lf_var1d=host_readd(eaa);
                        lf_var2d=id;
                        lf_resd=lf_var1d+lf_var2d+lflags.oldcf;
                        host_writed(eaa,lf_resd);
                        lflags.type=t_ADCd;
                        break;
                    case 0x03:lflags.oldcf=get_CF()!=0;
                        lf_var1d=host_readd(eaa);
                        lf_var2d=id;
                        lf_resd=lf_var1d-(lf_var2d+lflags.oldcf);
                        host_writed(eaa,lf_resd);
                        lflags.type=t_SBBd;
                        break;
                    case 0x04:
                        lf_var1d=host_readd(eaa);
                        lf_var2d=id;
                        lf_resd=lf_var1d & lf_var2d;
                        host_writed(eaa,lf_resd);
                        lflags.type=t_ANDd;
                        break;
                    case 0x05:
                        lf_var1d=host_readd(eaa);
                        lf_var2d=id;
                        lf_resd=lf_var1d-lf_var2d;
                        host_writed(eaa,lf_resd);
                        lflags.type=t_SUBd;
                        break;
                    case 0x06:
                        lf_var1d=host_readd(eaa);
                        lf_var2d=id;
                        lf_resd=lf_var1d ^ lf_var2d;
                        host_writed(eaa,lf_resd);
                        lflags.type=t_XORd;
                        break;
                    case 0x07:
                        lf_var1d=host_readd(eaa);
                        lf_var2d=id;
                        lf_resd=lf_var1d-lf_var2d;
                        lflags.type=t_CMPd;
                        break;
                    }
                }
            }
            break;
        case (PREFIX_NONE+0x85):									/* TEST r/m32, r32 */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    lf_var1d=*eard;
                    lf_var2d=*rmrd;
                    lf_resd=lf_var1d & lf_var2d;
                    lflags.type=t_TESTd;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1d=host_readd(eaa);
                    lf_var2d=*rmrd;
                    lf_resd=lf_var1d & lf_var2d;
                    lflags.type=t_TESTd;
                }
            }
            break;
        case (PREFIX_NONE+0x87):									/* XCHG r/m32, r32 */
            {	
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                Bit32u oldrmrd=*rmrd;
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    *rmrd=*eard;
                    *eard=oldrmrd;
                }
                else {
                    HostPt eaa=core.ea_table[rm]();
                    *rmrd=host_readd(eaa);
                    host_writed(eaa,oldrmrd);
                }
                break;
            }
        case (PREFIX_NONE+0x89):									/* MOV r/m32, r32 */
            {	
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    *eard=*rmrd;
                }
                else {
                    HostPt eaa=core.ea_table[rm]();
                    host_writed(eaa,*rmrd);
                }
                break;
            }
        case (PREFIX_NONE+0x8b):									/* MOV r32, r/m32 */
            {	
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    *rmrd=*eard;
                }
                else {
                    HostPt eaa=core.ea_table[rm]();
                    *rmrd=host_readd(eaa);
                }
                break;
            }
        case (PREFIX_NONE+0x8c):									/* Mov r/m16, Sreg */
            {
                Bit8u rm=Fetchb();
                Bit16u val;
                Bitu which=(rm>>3)&7;
                switch (which) {
                case 0x00:		/* MOV r/m16,ES */
                    goto illegal_opcode; // not implemented 
                case 0x01:		/* MOV r/m16,CS */
                    goto illegal_opcode; // not implemented 
                case 0x02:		/* MOV r/m16,SS */
                    goto illegal_opcode; // not implemented 
                case 0x03:		/* MOV r/m16,DS */
                    goto illegal_opcode; // not implemented 
                case 0x04:		/* MOV r/m16,FS */
                    goto illegal_opcode; // not implemented 
                case 0x05:		/* MOV r/m16,GS */
                    goto illegal_opcode; // not implemented 
                default:
                    goto illegal_opcode; // not implemented
                }
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    *eard=val;
                }
                else {
                    HostPt eaa=core.ea_table[rm]();
                    host_writew(eaa,val);
                }
                break;
            }	
        case (PREFIX_NONE+0x8d):									/* LEA r32 */
            {

                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                core.base_ds=core.base_ss=0;
                if (core.addr32 & PREFIX_ADDR) {
                    *rmrd=(Bit32u)(*EATable[256+rm])();
                } else {
                    *rmrd=(Bit32u)(*EATable[rm])();
                }
                break;
            }
        case (PREFIX_NONE+0x8f):									/* POP r/m32 */
            {
                Bit32u val=CPU_Pop32();
                Bit8u rm=Fetchb();
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    *eard=val;
                }
                else {
                    HostPt eaa=core.ea_table[rm]();
                    host_writed(eaa,val);
                }
                break;
            }
        case (PREFIX_NONE+0x91):									/* XCHG ECX,EAX */
            {
                Bit32u temp=reg_eax;
                reg_eax=reg_ecx;
                reg_ecx=temp;
                break;
            }
        case (PREFIX_NONE+0x92):									/* XCHG EDX,EAX */
            {
                Bit32u temp=reg_eax;
                reg_eax=reg_edx;
                reg_edx=temp;
                break;
            }
            break;
        case (PREFIX_NONE+0x93):									/* XCHG EBX,EAX */
            {
                Bit32u temp=reg_eax;
                reg_eax=reg_ebx;
                reg_ebx=temp;
                break;
            }
            break;
        case (PREFIX_NONE+0x94):									/* XCHG ESP,EAX */
            {
                Bit32u temp=reg_eax;
                reg_eax=reg_esp;
                reg_esp=temp;
                break;
            }
            break;
        case (PREFIX_NONE+0x95):									/* XCHG EBP,EAX */
            {
                Bit32u temp=reg_eax;
                reg_eax=reg_ebp;
                reg_ebp=temp;
                break;
            }
            break;
        case (PREFIX_NONE+0x96):									/* XCHG ESI,EAX */
            {
                Bit32u temp=reg_eax;
                reg_eax=reg_esi;
                reg_esi=temp;
                break;
            }
            break;
        case (PREFIX_NONE+0x97):									/* XCHG EDI,EAX */
            {
                Bit32u temp=reg_eax;
                reg_eax=reg_edi;
                reg_edi=temp;
                break;
            }
            break;
        case (PREFIX_NONE+0x98):									/* CWDE */
            reg_eax=(Bit16s)reg_ax;
            break;
        case (PREFIX_NONE+0x99):									/* CDQ */
            if (reg_eax & 0x80000000) reg_edx=0xffffffff;
            else reg_edx=0;
            break;
        case (PREFIX_NONE+0x9a):									/* CALL ptr16:32 */
            goto illegal_opcode; // not implemented 
        case (PREFIX_NONE+0x9c):									/* PUSHFD */
            CPU_PUSHF(true);
            break;
        case (PREFIX_NONE+0x9d):									/* POPFD */
            CPU_POPF(true);
            break;
        case (PREFIX_NONE+0xa1):									/* MOV EAX, moffs32 */
            {
                HostPt eaa;
                if (core.addr32 & PREFIX_ADDR) {
                    eaa=core.base_ds+Fetchd();
                } else {
                    eaa=core.base_ds+Fetchw();
                }
                reg_eax=host_readd(eaa);
            }
            break;
        case (PREFIX_NONE+0xa3):									/* MOV moffs32,EAX */
            {
                HostPt eaa;
                if (core.addr32 & PREFIX_ADDR) {
                    eaa=core.base_ds+Fetchd();
                } else {
                    eaa=core.base_ds+Fetchw();
                }
                host_writed(eaa,reg_eax);
            }
            break;
        case (PREFIX_NONE+0xa5):									/* MOVSD */
            goto illegal_opcode; // not implemented 
        case (PREFIX_NONE+0xa7):									/* CMPSD */
            goto illegal_opcode; // not implemented 
        case (PREFIX_NONE+0xa9):									/* TEST EAX, imm32 */
            {
                lf_var1d=reg_eax;
                lf_var2d=Fetchd();
                lf_resd=lf_var1d & lf_var2d;
                lflags.type=t_TESTd;
            }
            break;
        case (PREFIX_NONE+0xab):									/* STOSD */
            goto illegal_opcode; // not implemented 
        case (PREFIX_NONE+0xad):									/* LODSD */
            goto illegal_opcode; // not implemented 
        case (PREFIX_NONE+0xaf):									/* SCASD */
            goto illegal_opcode; // not implemented 
        case (PREFIX_NONE+0xb8):									/* MOV EAX, imm32 */
            reg_eax=Fetchd();
            break;
        case (PREFIX_NONE+0xb9):									/* MOV ECX, imm32 */
            reg_ecx=Fetchd();
            break;
        case (PREFIX_NONE+0xba):									/* MOV EDX, imm16 */
            reg_edx=Fetchd();
            break;
        case (PREFIX_NONE+0xbb):									/* MOV EBX, imm32 */
            reg_ebx=Fetchd();
            break;
        case (PREFIX_NONE+0xbc):									/* MOV ESP, imm32 */
            reg_esp=Fetchd();
            break;
        case (PREFIX_NONE+0xbd):									/* MOV EBP, imm32 */
            reg_ebp=Fetchd();
            break;
        case (PREFIX_NONE+0xbe):									/* MOV ESI, imm32 */
            reg_esi=Fetchd();
            break;
        case (PREFIX_NONE+0xbf):									/* MOV EDI, imm32 */
            reg_edi=Fetchd();
            break;
        case (PREFIX_NONE+0xc1):									/* GRP2 r/m32, imm8 */
            {
                Bit8u rm=Fetchb();
                Bitu which=(rm>>3)&7;
                if (rm >= 0xc0) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    Bit8u val=Fetchb() & 0x1f; 
                    switch (which) {
                    case 0x00:
                        if (!val) break; FillFlagsNoCFOF();
                        lf_var1d=*eard;
                        lf_var2b=val;
                        lf_resd=(lf_var1d << lf_var2b) | (lf_var1d >> (32-lf_var2b));
                        *eard=lf_resd;
                        if (lf_resd & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resd & 1) ^ (lf_resd >> 31)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x01:
                        if (!val) break; FillFlagsNoCFOF();
                        lf_var1d=*eard;
                        lf_var2b=val;
                        lf_resd=(lf_var1d >> lf_var2b) | (lf_var1d << (32-lf_var2b));
                        *eard=lf_resd;
                        if (lf_resd & 0x80000000) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resd ^ (lf_resd<<1)) & 0x80000000) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x02:
                        if (!val) break; {
                            Bit32u cf=(Bit32u)FillFlags()&0x1;
                            lf_var1d=*eard;
                            lf_var2b=val;
                            if (lf_var2b==1) {
                                lf_resd=(lf_var1d << 1) | cf;
                            } else {
                                lf_resd=(lf_var1d << lf_var2b) | (cf << (lf_var2b-1)) | (lf_var1d >> (33-lf_var2b));
                            } 
                            *eard=lf_resd;
                            if (((lf_var1d >> (32-lf_var2b)) & 1)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((reg_flags & 1) ^ (lf_resd >> 31)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x03:
                        if (val) {
                            Bit32u cf=(Bit32u)FillFlags()&0x1;
                            lf_var1d=*eard;
                            lf_var2b=val;
                            if (lf_var2b==1) {
                                lf_resd=lf_var1d >> 1 | cf << 31;
                            } else {
                                lf_resd=(lf_var1d >> lf_var2b) | (cf << (32-lf_var2b)) | (lf_var1d << (33-lf_var2b));
                            } 
                            *eard=lf_resd;
                            if ((lf_var1d >> (lf_var2b - 1)) & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((lf_resd ^ (lf_resd<<1)) & 0x80000000) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x04:
                    case 0x06:
                        if (!val) break;
                        lf_var1d=*eard;
                        lf_var2b=val;
                        lf_resd=lf_var1d << lf_var2b;
                        *eard=lf_resd;
                        lflags.type=t_SHLd;
                        break; 
                    case 0x05:
                        if (!val) break;
                        lf_var1d=*eard;
                        lf_var2b=val;
                        lf_resd=lf_var1d >> lf_var2b;
                        *eard=lf_resd;
                        lflags.type=t_SHRd;
                        break; 
                    case 0x07:
                        if (!val) break;
                        lf_var2b=val;
                        lf_var1d=*eard;
                        if (lf_var1d & 0x80000000) {
                            lf_resd=(lf_var1d >> lf_var2b)| (0xffffffff << (32 - lf_var2b));
                        } else {
                            lf_resd=lf_var1d >> lf_var2b;
                        } 
                        *eard=lf_resd;
                        lflags.type=t_SARd;
                        break;
                    }
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    Bit8u val=Fetchb() & 0x1f; 
                    switch (which) {
                    case 0x00:
                        if (!val) break; FillFlagsNoCFOF();
                        lf_var1d=host_readd(eaa);
                        lf_var2b=val;
                        lf_resd=(lf_var1d << lf_var2b) | (lf_var1d >> (32-lf_var2b));
                        host_writed(eaa,lf_resd);
                        if (lf_resd & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resd & 1) ^ (lf_resd >> 31)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x01:
                        if (!val) break; FillFlagsNoCFOF();
                        lf_var1d=host_readd(eaa);
                        lf_var2b=val;
                        lf_resd=(lf_var1d >> lf_var2b) | (lf_var1d << (32-lf_var2b));
                        host_writed(eaa,lf_resd);
                        if (lf_resd & 0x80000000) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resd ^ (lf_resd<<1)) & 0x80000000) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x02:
                        if (!val) break; {
                            Bit32u cf=(Bit32u)FillFlags()&0x1;
                            lf_var1d=host_readd(eaa);
                            lf_var2b=val;
                            if (lf_var2b==1) {
                                lf_resd=(lf_var1d << 1) | cf;
                            } else {
                                lf_resd=(lf_var1d << lf_var2b) | (cf << (lf_var2b-1)) | (lf_var1d >> (33-lf_var2b));
                            } 
                            host_writed(eaa,lf_resd);
                            if (((lf_var1d >> (32-lf_var2b)) & 1)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((reg_flags & 1) ^ (lf_resd >> 31)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x03:
                        if (val) {
                            Bit32u cf=(Bit32u)FillFlags()&0x1;
                            lf_var1d=host_readd(eaa);
                            lf_var2b=val;
                            if (lf_var2b==1) {
                                lf_resd=lf_var1d >> 1 | cf << 31;
                            } else {
                                lf_resd=(lf_var1d >> lf_var2b) | (cf << (32-lf_var2b)) | (lf_var1d << (33-lf_var2b));
                            } 
                            host_writed(eaa,lf_resd);
                            if ((lf_var1d >> (lf_var2b - 1)) & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((lf_resd ^ (lf_resd<<1)) & 0x80000000) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x04:
                    case 0x06:
                        if (!val) break;
                        lf_var1d=host_readd(eaa);
                        lf_var2b=val;
                        lf_resd=lf_var1d << lf_var2b;
                        host_writed(eaa,lf_resd);
                        lflags.type=t_SHLd;
                        break; 
                    case 0x05:
                        if (!val) break;
                        lf_var1d=host_readd(eaa);
                        lf_var2b=val;
                        lf_resd=lf_var1d >> lf_var2b;
                        host_writed(eaa,lf_resd);
                        lflags.type=t_SHRd;
                        break; 
                    case 0x07:
                        if (!val) break;
                        lf_var2b=val;
                        lf_var1d=host_readd(eaa);
                        if (lf_var1d & 0x80000000) {
                            lf_resd=(lf_var1d >> lf_var2b)| (0xffffffff << (32 - lf_var2b));
                        } else {
                            lf_resd=lf_var1d >> lf_var2b;
                        } 
                        host_writed(eaa,lf_resd);
                        lflags.type=t_SARd;
                        break;
                    }
                } 
            }
            break;
        case (PREFIX_NONE+0xc2):									/* RETN imm16 */
            reg_eip=CPU_Pop32();
            reg_esp+=Fetchw();
            continue;
        case (PREFIX_NONE+0xc3):									/* RETN */
            reg_eip=CPU_Pop32();
            continue;
        case (PREFIX_NONE+0xc4):									/* LES */
            goto illegal_opcode; // not implemented 
        case (PREFIX_NONE+0xc5):									/* LDS */
            goto illegal_opcode; // not implemented 
        case (PREFIX_NONE+0xc7):									/* MOV r/m32, imm32 */
            {
                Bit8u rm=Fetchb();
                if (rm >= 0xc0) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    *eard=Fetchd();
                }
                else {
                    HostPt eaa=core.ea_table[rm]();
                    host_writed(eaa,Fetchd());
                }
                break;
            }
        case (PREFIX_NONE+0xc8):									/* ENTER imm16, imm8 */
            {
                Bitu bytes=Fetchw();
                Bitu level=Fetchb();
                CPU_ENTER(bytes,level);
            }
            break;
        case (PREFIX_NONE+0xc9):									/* LEAVE */
            reg_esp=reg_ebp;
            reg_ebp=CPU_Pop32();
            break;
        case (PREFIX_NONE+0xca):									/* RETF imm16 */
            goto illegal_opcode; // not implemented 
        case (PREFIX_NONE+0xcb):									/* RETF */			
            goto illegal_opcode; // not implemented 
        case (PREFIX_NONE+0xcf):									/* IRET */
            {
                goto illegal_opcode; // not implemented 
            }
        case (PREFIX_NONE+0xd1):									/* GRP2 r/m32, 1 */
            {
                Bit8u rm=Fetchb();
                Bitu which=(rm>>3)&7;
                if (rm >= 0xc0) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    Bit8u val=1 & 0x1f; 
                    switch (which) {
                    case 0x00:
                        if (!val) break; FillFlagsNoCFOF();
                        lf_var1d=*eard;
                        lf_var2b=val;
                        lf_resd=(lf_var1d << lf_var2b) | (lf_var1d >> (32-lf_var2b));
                        *eard=lf_resd;
                        if (lf_resd & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resd & 1) ^ (lf_resd >> 31)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x01:
                        if (!val) break; FillFlagsNoCFOF();
                        lf_var1d=*eard;
                        lf_var2b=val;
                        lf_resd=(lf_var1d >> lf_var2b) | (lf_var1d << (32-lf_var2b));
                        *eard=lf_resd;
                        if (lf_resd & 0x80000000) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resd ^ (lf_resd<<1)) & 0x80000000) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x02:
                        if (!val) break; {
                            Bit32u cf=(Bit32u)FillFlags()&0x1;
                            lf_var1d=*eard;
                            lf_var2b=val;
                            if (lf_var2b==1) {
                                lf_resd=(lf_var1d << 1) | cf;
                            } else {
                                lf_resd=(lf_var1d << lf_var2b) | (cf << (lf_var2b-1)) | (lf_var1d >> (33-lf_var2b));
                            } 
                            *eard=lf_resd;
                            if (((lf_var1d >> (32-lf_var2b)) & 1)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((reg_flags & 1) ^ (lf_resd >> 31)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x03:
                        if (val) {
                            Bit32u cf=(Bit32u)FillFlags()&0x1;
                            lf_var1d=*eard;
                            lf_var2b=val;
                            if (lf_var2b==1) {
                                lf_resd=lf_var1d >> 1 | cf << 31;
                            } else {
                                lf_resd=(lf_var1d >> lf_var2b) | (cf << (32-lf_var2b)) | (lf_var1d << (33-lf_var2b));
                            } 
                            *eard=lf_resd;
                            if ((lf_var1d >> (lf_var2b - 1)) & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((lf_resd ^ (lf_resd<<1)) & 0x80000000) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x04:
                    case 0x06:
                        if (!val) break;
                        lf_var1d=*eard;
                        lf_var2b=val;
                        lf_resd=lf_var1d << lf_var2b;
                        *eard=lf_resd;
                        lflags.type=t_SHLd;
                        break; 
                    case 0x05:
                        if (!val) break;
                        lf_var1d=*eard;
                        lf_var2b=val;
                        lf_resd=lf_var1d >> lf_var2b;
                        *eard=lf_resd;
                        lflags.type=t_SHRd;
                        break; 
                    case 0x07:
                        if (!val) break;
                        lf_var2b=val;
                        lf_var1d=*eard;
                        if (lf_var1d & 0x80000000) {
                            lf_resd=(lf_var1d >> lf_var2b)| (0xffffffff << (32 - lf_var2b));
                        } else {
                            lf_resd=lf_var1d >> lf_var2b;
                        } 
                        *eard=lf_resd;
                        lflags.type=t_SARd;
                        break;
                    }
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    Bit8u val=1 & 0x1f; 
                    switch (which) {
                    case 0x00:
                        if (!val) break; FillFlagsNoCFOF();
                        lf_var1d=host_readd(eaa);
                        lf_var2b=val;
                        lf_resd=(lf_var1d << lf_var2b) | (lf_var1d >> (32-lf_var2b));
                        host_writed(eaa,lf_resd);
                        if (lf_resd & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resd & 1) ^ (lf_resd >> 31)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x01:
                        if (!val) break; FillFlagsNoCFOF();
                        lf_var1d=host_readd(eaa);
                        lf_var2b=val;
                        lf_resd=(lf_var1d >> lf_var2b) | (lf_var1d << (32-lf_var2b));
                        host_writed(eaa,lf_resd);
                        if (lf_resd & 0x80000000) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resd ^ (lf_resd<<1)) & 0x80000000) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x02:
                        if (!val) break; {
                            Bit32u cf=(Bit32u)FillFlags()&0x1;
                            lf_var1d=host_readd(eaa);
                            lf_var2b=val;
                            if (lf_var2b==1) {
                                lf_resd=(lf_var1d << 1) | cf;
                            } else {
                                lf_resd=(lf_var1d << lf_var2b) | (cf << (lf_var2b-1)) | (lf_var1d >> (33-lf_var2b));
                            } 
                            host_writed(eaa,lf_resd);
                            if (((lf_var1d >> (32-lf_var2b)) & 1)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((reg_flags & 1) ^ (lf_resd >> 31)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x03:
                        if (val) {
                            Bit32u cf=(Bit32u)FillFlags()&0x1;
                            lf_var1d=host_readd(eaa);
                            lf_var2b=val;
                            if (lf_var2b==1) {
                                lf_resd=lf_var1d >> 1 | cf << 31;
                            } else {
                                lf_resd=(lf_var1d >> lf_var2b) | (cf << (32-lf_var2b)) | (lf_var1d << (33-lf_var2b));
                            } 
                            host_writed(eaa,lf_resd);
                            if ((lf_var1d >> (lf_var2b - 1)) & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((lf_resd ^ (lf_resd<<1)) & 0x80000000) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x04:
                    case 0x06:
                        if (!val) break;
                        lf_var1d=host_readd(eaa);
                        lf_var2b=val;
                        lf_resd=lf_var1d << lf_var2b;
                        host_writed(eaa,lf_resd);
                        lflags.type=t_SHLd;
                        break; 
                    case 0x05:
                        if (!val) break;
                        lf_var1d=host_readd(eaa);
                        lf_var2b=val;
                        lf_resd=lf_var1d >> lf_var2b;
                        host_writed(eaa,lf_resd);
                        lflags.type=t_SHRd;
                        break; 
                    case 0x07:
                        if (!val) break;
                        lf_var2b=val;
                        lf_var1d=host_readd(eaa);
                        if (lf_var1d & 0x80000000) {
                            lf_resd=(lf_var1d >> lf_var2b)| (0xffffffff << (32 - lf_var2b));
                        } else {
                            lf_resd=lf_var1d >> lf_var2b;
                        } 
                        host_writed(eaa,lf_resd);
                        lflags.type=t_SARd;
                        break;
                    }
                } 
            }
            break;
        case (PREFIX_NONE+0xd3):									/* GRP2 r/m32, CL */
            {
                Bit8u rm=Fetchb();
                Bitu which=(rm>>3)&7;
                if (rm >= 0xc0) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    Bit8u val=reg_cl & 0x1f; 
                    switch (which) {
                    case 0x00:
                        if (!val) break; FillFlagsNoCFOF();
                        lf_var1d=*eard;
                        lf_var2b=val;
                        lf_resd=(lf_var1d << lf_var2b) | (lf_var1d >> (32-lf_var2b));
                        *eard=lf_resd;
                        if (lf_resd & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resd & 1) ^ (lf_resd >> 31)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x01:
                        if (!val) break; FillFlagsNoCFOF();
                        lf_var1d=*eard;
                        lf_var2b=val;
                        lf_resd=(lf_var1d >> lf_var2b) | (lf_var1d << (32-lf_var2b));
                        *eard=lf_resd;
                        if (lf_resd & 0x80000000) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resd ^ (lf_resd<<1)) & 0x80000000) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x02:
                        if (!val) break; {
                            Bit32u cf=(Bit32u)FillFlags()&0x1;
                            lf_var1d=*eard;
                            lf_var2b=val;
                            if (lf_var2b==1) {
                                lf_resd=(lf_var1d << 1) | cf;
                            } else {
                                lf_resd=(lf_var1d << lf_var2b) | (cf << (lf_var2b-1)) | (lf_var1d >> (33-lf_var2b));
                            } 
                            *eard=lf_resd;
                            if (((lf_var1d >> (32-lf_var2b)) & 1)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((reg_flags & 1) ^ (lf_resd >> 31)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x03:
                        if (val) {
                            Bit32u cf=(Bit32u)FillFlags()&0x1;
                            lf_var1d=*eard;
                            lf_var2b=val;
                            if (lf_var2b==1) {
                                lf_resd=lf_var1d >> 1 | cf << 31;
                            } else {
                                lf_resd=(lf_var1d >> lf_var2b) | (cf << (32-lf_var2b)) | (lf_var1d << (33-lf_var2b));
                            } 
                            *eard=lf_resd;
                            if ((lf_var1d >> (lf_var2b - 1)) & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((lf_resd ^ (lf_resd<<1)) & 0x80000000) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x04:
                    case 0x06:
                        if (!val) break;
                        lf_var1d=*eard;
                        lf_var2b=val;
                        lf_resd=lf_var1d << lf_var2b;
                        *eard=lf_resd;
                        lflags.type=t_SHLd;
                        break; 
                    case 0x05:
                        if (!val) break;
                        lf_var1d=*eard;
                        lf_var2b=val;
                        lf_resd=lf_var1d >> lf_var2b;
                        *eard=lf_resd;
                        lflags.type=t_SHRd;
                        break; 
                    case 0x07:
                        if (!val) break;
                        lf_var2b=val;
                        lf_var1d=*eard;
                        if (lf_var1d & 0x80000000) {
                            lf_resd=(lf_var1d >> lf_var2b)| (0xffffffff << (32 - lf_var2b));
                        } else {
                            lf_resd=lf_var1d >> lf_var2b;
                        } 
                        *eard=lf_resd;
                        lflags.type=t_SARd;
                        break;
                    }
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    Bit8u val=reg_cl & 0x1f; 
                    switch (which) {
                    case 0x00:
                        if (!val) break; FillFlagsNoCFOF();
                        lf_var1d=host_readd(eaa);
                        lf_var2b=val;
                        lf_resd=(lf_var1d << lf_var2b) | (lf_var1d >> (32-lf_var2b));
                        host_writed(eaa,lf_resd);
                        if (lf_resd & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resd & 1) ^ (lf_resd >> 31)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x01:
                        if (!val) break; FillFlagsNoCFOF();
                        lf_var1d=host_readd(eaa);
                        lf_var2b=val;
                        lf_resd=(lf_var1d >> lf_var2b) | (lf_var1d << (32-lf_var2b));
                        host_writed(eaa,lf_resd);
                        if (lf_resd & 0x80000000) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resd ^ (lf_resd<<1)) & 0x80000000) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x02:
                        if (!val) break; {
                            Bit32u cf=(Bit32u)FillFlags()&0x1;
                            lf_var1d=host_readd(eaa);
                            lf_var2b=val;
                            if (lf_var2b==1) {
                                lf_resd=(lf_var1d << 1) | cf;
                            } else {
                                lf_resd=(lf_var1d << lf_var2b) | (cf << (lf_var2b-1)) | (lf_var1d >> (33-lf_var2b));
                            } 
                            host_writed(eaa,lf_resd);
                            if (((lf_var1d >> (32-lf_var2b)) & 1)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((reg_flags & 1) ^ (lf_resd >> 31)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x03:
                        if (val) {
                            Bit32u cf=(Bit32u)FillFlags()&0x1;
                            lf_var1d=host_readd(eaa);
                            lf_var2b=val;
                            if (lf_var2b==1) {
                                lf_resd=lf_var1d >> 1 | cf << 31;
                            } else {
                                lf_resd=(lf_var1d >> lf_var2b) | (cf << (32-lf_var2b)) | (lf_var1d << (33-lf_var2b));
                            } 
                            host_writed(eaa,lf_resd);
                            if ((lf_var1d >> (lf_var2b - 1)) & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((lf_resd ^ (lf_resd<<1)) & 0x80000000) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x04:
                    case 0x06:
                        if (!val) break;
                        lf_var1d=host_readd(eaa);
                        lf_var2b=val;
                        lf_resd=lf_var1d << lf_var2b;
                        host_writed(eaa,lf_resd);
                        lflags.type=t_SHLd;
                        break; 
                    case 0x05:
                        if (!val) break;
                        lf_var1d=host_readd(eaa);
                        lf_var2b=val;
                        lf_resd=lf_var1d >> lf_var2b;
                        host_writed(eaa,lf_resd);
                        lflags.type=t_SHRd;
                        break; 
                    case 0x07:
                        if (!val) break;
                        lf_var2b=val;
                        lf_var1d=host_readd(eaa);
                        if (lf_var1d & 0x80000000) {
                            lf_resd=(lf_var1d >> lf_var2b)| (0xffffffff << (32 - lf_var2b));
                        } else {
                            lf_resd=lf_var1d >> lf_var2b;
                        } 
                        host_writed(eaa,lf_resd);
                        lflags.type=t_SARd;
                        break;
                    }
                } 
            }
            break;
        case (PREFIX_NONE+0xe0):									/* LOOPNZ */
            if (core.addr32 & PREFIX_ADDR) {
                {
                    reg_eip=(core.cseip-MemBase);
                    if (--reg_ecx && !get_ZF()) reg_eip+=Fetchb();
                    reg_eip+=1; continue;
                }
            } else {
                {
                    reg_eip=(core.cseip-MemBase);
                    if (--reg_cx && !get_ZF()) reg_eip+=Fetchb();
                    reg_eip+=1; continue;
                }
            }
            break;
        case (PREFIX_NONE+0xe1):									/* LOOPZ */
            if (core.addr32 & PREFIX_ADDR) {
                {
                    reg_eip=(core.cseip-MemBase);
                    if (--reg_ecx && get_ZF()) reg_eip+=Fetchb();
                    reg_eip+=1; continue;
                }
            } else {
                {
                    reg_eip=(core.cseip-MemBase);
                    if (--reg_cx && get_ZF()) reg_eip+=Fetchb();
                    reg_eip+=1; continue;
                }
            }
            break;
        case (PREFIX_NONE+0xe2):									/* LOOP */
            if (core.addr32 & PREFIX_ADDR) {	
                {
                    reg_eip=(core.cseip-MemBase);
                    if (--reg_ecx) reg_eip+=Fetchb();
                    reg_eip+=1; continue;
                }
            } else {
                {
                    reg_eip=(core.cseip-MemBase);
                    if (--reg_cx) reg_eip+=Fetchb();
                    reg_eip+=1; continue;
                }
            }
            break;
        case (PREFIX_NONE+0xe3):									/* JCXZ */
            {
                reg_eip=(core.cseip-MemBase);
                if (!(reg_ecx & AddrMaskTable[core.addr32& PREFIX_ADDR])) reg_eip+=Fetchb();
                reg_eip+=1; continue;
            }
            break;
        case (PREFIX_NONE+0xe5):									/* IN EAX, imm8 */
            {
                Bitu port=Fetchb();
                reg_eax=IO_ReadD(port);
                break;
            }
        case (PREFIX_NONE+0xe7):									/* OUT imm8,EAX */
            {
                Bitu port=Fetchb();
                IO_WriteD(port,reg_eax);
                break;
            }
        case (PREFIX_NONE+0xe8):									/* CALL rel32 */
            { 
                Bit32s addip=Fetchd();
                reg_eip=(core.cseip-MemBase);
                CPU_Push32(reg_eip);
                reg_eip+=addip;
                continue;
            }
        case (PREFIX_NONE+0xe9):									/* JMP rel32 */
            { 
                Bit32s addip=Fetchd();
                reg_eip=(core.cseip-MemBase);
                reg_eip+=addip;
                continue;
            }
        case (PREFIX_NONE+0xea):									/* JMP ptr16:32 */
            goto illegal_opcode; // not implemented 
        case (PREFIX_NONE+0xeb):									/* JMP rel8 */
            { 
                Bit32s addip=Fetchb();
                reg_eip=(core.cseip-MemBase);
                reg_eip+=addip;
                continue;
            }
        case (PREFIX_NONE+0xed):									/* IN EAX,DX */
            reg_eax=IO_ReadD(reg_dx);
            break;
        case (PREFIX_NONE+0xef):									/* OUT DX,EAX */
            IO_WriteD(reg_dx,reg_eax);
            break;
        case (PREFIX_NONE+0xf7):									/* GRP3 r/m32(, imm32) */
            { 
                Bit8u rm=Fetchb();
                Bitu which=(rm>>3)&7;
                switch (which) {
                case 0x00:								/* TEST r/m32, imm32 */
                case 0x01:								/* TEST r/m32, imm32 Undocumented*/
                    {
                        if (rm >= 0xc0 ) {
                            Bit32u * eard=lookupRMEAregd[rm];
                            lf_var1d=*eard;
                            lf_var2d=Fetchd();
                            lf_resd=lf_var1d & lf_var2d;
                            lflags.type=t_TESTd;
                        }
                        else {
                            HostPt eaa=core.ea_table[rm]();
                            lf_var1d=host_readd(eaa);
                            lf_var2d=Fetchd();
                            lf_resd=lf_var1d & lf_var2d;
                            lflags.type=t_TESTd;
                        }
                        break;
                    }
                case 0x02:								/* NOT r/m32 */
                    {
                        if (rm >= 0xc0 ) {
                            Bit32u * eard=lookupRMEAregd[rm];
                            *eard=~*eard;
                        }
                        else {
                            HostPt eaa=core.ea_table[rm]();
                            host_writed(eaa,~host_readd(eaa));
                        }
                        break;
                    }
                case 0x03:								/* NEG r/m32 */
                    {
                        lflags.type=t_NEGd;
                        if (rm >= 0xc0 ) {
                            Bit32u * eard=lookupRMEAregd[rm];
                            lf_var1d=*eard;
                            lf_resd=0-lf_var1d;
                            *eard=lf_resd;
                        } else {
                            HostPt eaa=core.ea_table[rm]();
                            lf_var1d=host_readd(eaa);
                            lf_resd=0-lf_var1d;
                            host_writed(eaa,lf_resd);
                        }
                        break;
                    }
                case 0x04:								/* MUL EAX, r/m32 */
                    {
                        if (rm >= 0xc0 ) {
                            Bit32u * eard=lookupRMEAregd[rm];{
                                Bit64u tempu=(Bit64u)reg_eax*(Bit64u)(*eard);
                                reg_eax=(Bit32u)(tempu);
                                reg_edx=(Bit32u)(tempu >> 32); FillFlagsNoCFOF();
                                if (reg_eax == 0) reg_flags|=FLAG_ZF; else reg_flags&=~FLAG_ZF;
                                if (reg_edx) {
                                    reg_flags|=FLAG_CF;reg_flags|=FLAG_OF;
                                } else {
                                    reg_flags&=~FLAG_CF;reg_flags&=~FLAG_OF;
                                }
                            }
                        } else {
                            HostPt eaa=core.ea_table[rm]();{
                                Bit64u tempu=(Bit64u)reg_eax*(Bit64u)(host_readd(eaa));
                                reg_eax=(Bit32u)(tempu);
                                reg_edx=(Bit32u)(tempu >> 32); FillFlagsNoCFOF();
                                if (reg_eax == 0) reg_flags|=FLAG_ZF; else reg_flags&=~FLAG_ZF;
                                if (reg_edx) {
                                    reg_flags|=FLAG_CF;reg_flags|=FLAG_OF;
                                } else {
                                    reg_flags&=~FLAG_CF;reg_flags&=~FLAG_OF;
                                }
                            }
                        }
                    }
                    break;
                case 0x05:								/* IMUL EAX, r/m32 */
                    {
                        if (rm >= 0xc0 ) {
                            Bit32u * eard=lookupRMEAregd[rm];{
                                Bit64s temps=((Bit64s)((Bit32s)reg_eax))* ((Bit64s)((Bit32s)(*eard)));
                                reg_eax=(Bit32u)(temps);
                                reg_edx=(Bit32u)(temps >> 32); FillFlagsNoCFOF();
                                if ((reg_edx==0xffffffff) && (reg_eax & 0x80000000) ) {
                                    reg_flags&=~FLAG_CF;reg_flags&=~FLAG_OF;
                                } else if ( (reg_edx==0x00000000) && (reg_eax< 0x80000000) ) {
                                    reg_flags&=~FLAG_CF;reg_flags&=~FLAG_OF;
                                } else {
                                    reg_flags|=FLAG_CF;reg_flags|=FLAG_OF;
                                }
                            }
                        } else {
                            HostPt eaa=core.ea_table[rm]();{
                                Bit64s temps=((Bit64s)((Bit32s)reg_eax))* ((Bit64s)((Bit32s)(host_readd(eaa))));
                                reg_eax=(Bit32u)(temps);
                                reg_edx=(Bit32u)(temps >> 32); FillFlagsNoCFOF();
                                if ((reg_edx==0xffffffff) && (reg_eax & 0x80000000) ) {
                                    reg_flags&=~FLAG_CF;reg_flags&=~FLAG_OF;
                                } else if ( (reg_edx==0x00000000) && (reg_eax< 0x80000000) ) {
                                    reg_flags&=~FLAG_CF;reg_flags&=~FLAG_OF;
                                } else {
                                    reg_flags|=FLAG_CF;reg_flags|=FLAG_OF;
                                }
                            }
                        }
                    }
                    break;
                case 0x06:								/* DIV r/m32 */
                    {
                        if (rm >= 0xc0 ) {
                            Bit32u * eard=lookupRMEAregd[rm];{
                                Bitu val=*eard;
                                if (val==0) val=1 ;
                                Bit64u num=(((Bit64u)reg_edx)<<32)|reg_eax;
                                Bit64u quo=num/val;
                                Bit32u rem=(Bit32u)(num % val);
                                Bit32u quo32=(Bit32u)(quo&0xffffffff);
                                reg_edx=rem;
                                reg_eax=quo32;
                            }
                        } else {
                            HostPt eaa=core.ea_table[rm]();{
                                Bitu val=host_readd(eaa);
                                if (val==0) val=1 ;
                                Bit64u num=(((Bit64u)reg_edx)<<32)|reg_eax;
                                Bit64u quo=num/val;
                                Bit32u rem=(Bit32u)(num % val);
                                Bit32u quo32=(Bit32u)(quo&0xffffffff);
                                reg_edx=rem;
                                reg_eax=quo32;
                            }
                        }
                    }
                    break;
                case 0x07:								/* IDIV r/m32 */
                    {
                        if (rm >= 0xc0 ) {
                            Bit32u * eard=lookupRMEAregd[rm];{
                                Bits val=(Bit32s)(*eard);
                                if (val==0) val=1 ;
                                Bit64s num=(((Bit64u)reg_edx)<<32)|reg_eax;
                                Bit64s quo=num/val;
                                Bit32s rem=(Bit32s)(num % val);
                                Bit32s quo32s=(Bit32s)(quo&0xffffffff);
                                reg_edx=rem;
                                reg_eax=quo32s;
                            }
                        } else {
                            HostPt eaa=core.ea_table[rm]();{
                                Bits val=(Bit32s)(host_readd(eaa));
                                if (val==0) val=1 ;
                                Bit64s num=(((Bit64u)reg_edx)<<32)|reg_eax;
                                Bit64s quo=num/val;
                                Bit32s rem=(Bit32s)(num % val);
                                Bit32s quo32s=(Bit32s)(quo&0xffffffff);
                                reg_edx=rem;
                                reg_eax=quo32s;
                            }
                        }
                    }
                    break;
                }
                break;
            }
        case (PREFIX_NONE+0xff):									/* GRP 5 r/m32 */
            {
                Bit8u rm=Fetchb();
                Bitu which=(rm>>3)&7;
                switch (which) {
                case 0x00:								/* INC r/m32 */
                    {
                        if (rm >= 0xc0 ) {
                            Bit32u * eard=lookupRMEAregd[rm];if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            lf_var1d=*eard;
                            lf_resd=lf_var1d+1;
                            *eard=lf_resd;
                            lflags.type=t_INCd;
                        } else {
                            HostPt eaa=core.ea_table[rm]();if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            lf_var1d=host_readd(eaa);
                            lf_resd=lf_var1d+1;
                            host_writed(eaa,lf_resd);
                            lflags.type=t_INCd;
                        }
                    }
                    break;		
                case 0x01:								/* DEC r/m32 */
                    {
                        if (rm >= 0xc0 ) {
                            Bit32u * eard=lookupRMEAregd[rm];if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            lf_var1d=*eard;
                            lf_resd=lf_var1d-1;
                            *eard=lf_resd;
                            lflags.type=t_DECd;
                        } else {
                            HostPt eaa=core.ea_table[rm]();if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            lf_var1d=host_readd(eaa);
                            lf_resd=lf_var1d-1;
                            host_writed(eaa,lf_resd);
                            lflags.type=t_DECd;
                        }
                    }
                    break;
                case 0x02:								/* CALL NEAR r/m32 */
                    if (rm >= 0xc0 ) {
                        Bit32u * eard=lookupRMEAregd[rm];
                        reg_eip=*eard;
                    }
                    else {
                        HostPt eaa=core.ea_table[rm]();
                        reg_eip=host_readd(eaa);
                    }
                    CPU_Push32((core.cseip-MemBase));
                    continue;
                case 0x03:								/* CALL FAR r/m32 */
                    goto illegal_opcode; // not implemented 
                case 0x04:								/* JMP NEAR r/m32 */	
                    if (rm >= 0xc0 ) {
                        Bit32u * eard=lookupRMEAregd[rm];
                        reg_eip=*eard;
                    }
                    else {
                        HostPt eaa=core.ea_table[rm]();
                        reg_eip=host_readd(eaa);
                    }
                    continue;
                case 0x05:								/* JMP FAR r/m32 */	
                    goto illegal_opcode; // not implemented 
                    break;
                case 0x06:								/* Push r/m32 */
                    if (rm >= 0xc0 ) {
                        Bit32u * eard=lookupRMEAregd[rm];CPU_Push32(*eard);
                    }
                    else {
                        HostPt eaa=core.ea_table[rm]();CPU_Push32(host_readd(eaa));
                    }
                    break;
                default:
                    goto illegal_opcode; // not implemented 
                    break;
                }
                break;
            }



        case (PREFIX_SIZE+0x00):
        case (PREFIX_NONE+0x00):									/* ADD r/m8, r8 */
            {
                Bit8u rm=Fetchb();
                Bit8u * rmrb;
                rmrb=lookupRMregb[rm];
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    lf_var1b=*earb;
                    lf_var2b=*rmrb;
                    lf_resb=lf_var1b+lf_var2b;
                    *earb=lf_resb;
                    lflags.type=t_ADDb;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1b=host_readb(eaa);
                    lf_var2b=*rmrb;
                    lf_resb=lf_var1b+lf_var2b;
                    host_writeb(eaa,lf_resb);
                    lflags.type=t_ADDb;
                }
            }
            break;
        case (PREFIX_SIZE+0x01):									/* ADD r/m16, r16 */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    lf_var1w=*earw;
                    lf_var2w=*rmrw;
                    lf_resw=lf_var1w+lf_var2w;
                    *earw=lf_resw;
                    lflags.type=t_ADDw;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1w=host_readw(eaa);
                    lf_var2w=*rmrw;
                    lf_resw=lf_var1w+lf_var2w;
                    host_writew(eaa,lf_resw);
                    lflags.type=t_ADDw;
                }
            }
            break;	
        case (PREFIX_SIZE+0x02):
        case (PREFIX_NONE+0x02):									/* ADD r8, r/m8 */
            {
                Bit8u rm=Fetchb();
                Bit8u * rmrb;
                rmrb=lookupRMregb[rm];
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    lf_var1b=*rmrb;
                    lf_var2b=*earb;
                    lf_resb=lf_var1b+lf_var2b;
                    *rmrb=lf_resb;
                    lflags.type=t_ADDb;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1b=*rmrb;
                    lf_var2b=host_readb(eaa);
                    lf_resb=lf_var1b+lf_var2b;
                    *rmrb=lf_resb;
                    lflags.type=t_ADDb;
                }
            }
            break;
        case (PREFIX_SIZE+0x03):									/* ADD r16, r/m16 */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    lf_var1w=*rmrw;
                    lf_var2w=*earw;
                    lf_resw=lf_var1w+lf_var2w;
                    *rmrw=lf_resw;
                    lflags.type=t_ADDw;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1w=*rmrw;
                    lf_var2w=host_readw(eaa);
                    lf_resw=lf_var1w+lf_var2w;
                    *rmrw=lf_resw;
                    lflags.type=t_ADDw;
                }
            }
            break;
        case (PREFIX_SIZE+0x04):
        case (PREFIX_NONE+0x04):									/* ADD AL, imm8 */
            {
                lf_var1b=reg_al;
                lf_var2b=Fetchb();
                lf_resb=lf_var1b+lf_var2b;
                reg_al=lf_resb;
                lflags.type=t_ADDb;
            }
            break;
        case (PREFIX_SIZE+0x05):									/* ADD AX, imm16 */
            {
                lf_var1w=reg_ax;
                lf_var2w=Fetchw();
                lf_resw=lf_var1w+lf_var2w;
                reg_ax=lf_resw;
                lflags.type=t_ADDw;
            }
            break;
        case (PREFIX_SIZE+0x06):									/* PUSH ES */		
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0x07):									/* POP ES */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0x08):
        case (PREFIX_NONE+0x08):									/* OR r/m8, r8 */
            {
                Bit8u rm=Fetchb();
                Bit8u * rmrb;
                rmrb=lookupRMregb[rm];
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    lf_var1b=*earb;
                    lf_var2b=*rmrb;
                    lf_resb=lf_var1b | lf_var2b;
                    *earb=lf_resb;
                    lflags.type=t_ORb;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1b=host_readb(eaa);
                    lf_var2b=*rmrb;
                    lf_resb=lf_var1b | lf_var2b;
                    host_writeb(eaa,lf_resb);
                    lflags.type=t_ORb;
                }
            }
            break;
        case (PREFIX_SIZE+0x09):									/* OR r/m16, r16 */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    lf_var1w=*earw;
                    lf_var2w=*rmrw;
                    lf_resw=lf_var1w | lf_var2w;
                    *earw=lf_resw;
                    lflags.type=t_ORw;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1w=host_readw(eaa);
                    lf_var2w=*rmrw;
                    lf_resw=lf_var1w | lf_var2w;
                    host_writew(eaa,lf_resw);
                    lflags.type=t_ORw;
                }
            }
            break;
        case (PREFIX_SIZE+0x0a):
        case (PREFIX_NONE+0x0a):									/* OR r8, r/m8 */
            {
                Bit8u rm=Fetchb();
                Bit8u * rmrb;
                rmrb=lookupRMregb[rm];
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    lf_var1b=*rmrb;
                    lf_var2b=*earb;
                    lf_resb=lf_var1b | lf_var2b;
                    *rmrb=lf_resb;
                    lflags.type=t_ORb;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1b=*rmrb;
                    lf_var2b=host_readb(eaa);
                    lf_resb=lf_var1b | lf_var2b;
                    *rmrb=lf_resb;
                    lflags.type=t_ORb;
                }
            }
            break;
        case (PREFIX_SIZE+0x0b):									/* OR r16, r/m16 */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    lf_var1w=*rmrw;
                    lf_var2w=*earw;
                    lf_resw=lf_var1w | lf_var2w;
                    *rmrw=lf_resw;
                    lflags.type=t_ORw;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1w=*rmrw;
                    lf_var2w=host_readw(eaa);
                    lf_resw=lf_var1w | lf_var2w;
                    *rmrw=lf_resw;
                    lflags.type=t_ORw;
                }
            }
            break;
        case (PREFIX_SIZE+0x0c):
        case (PREFIX_NONE+0x0c):									/* OR AL, imm8 */
            {
                lf_var1b=reg_al;
                lf_var2b=Fetchb();
                lf_resb=lf_var1b | lf_var2b;
                reg_al=lf_resb;
                lflags.type=t_ORb;
            }
            break;
        case (PREFIX_SIZE+0x0d):									/* OR AX, imm16 */
            {
                lf_var1w=reg_ax;
                lf_var2w=Fetchw();
                lf_resw=lf_var1w | lf_var2w;
                reg_ax=lf_resw;
                lflags.type=t_ORw;
            }
            break;
        case (PREFIX_SIZE+0x0e):									/* PUSH CS */		
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0x0f):
        case (PREFIX_NONE+0x0f):									/* 2 byte opcodes*/		
            core.prefix|=OPCODE_0F;
            goto restart_opcode;
            break;
        case (PREFIX_SIZE+0x10):
        case (PREFIX_NONE+0x10):									/* ADC r/m8, r8 */
            {
                Bit8u rm=Fetchb();
                Bit8u * rmrb;
                rmrb=lookupRMregb[rm];
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    lflags.oldcf=get_CF()!=0;
                    lf_var1b=*earb;
                    lf_var2b=*rmrb;
                    lf_resb=lf_var1b+lf_var2b+lflags.oldcf;
                    *earb=lf_resb;
                    lflags.type=t_ADCb;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lflags.oldcf=get_CF()!=0;
                    lf_var1b=host_readb(eaa);
                    lf_var2b=*rmrb;
                    lf_resb=lf_var1b+lf_var2b+lflags.oldcf;
                    host_writeb(eaa,lf_resb);
                    lflags.type=t_ADCb;
                }
            }
            break;
        case (PREFIX_SIZE+0x11):									/* ADC r/m16, r16 */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    lflags.oldcf=get_CF()!=0;
                    lf_var1w=*earw;
                    lf_var2w=*rmrw;
                    lf_resw=lf_var1w+lf_var2w+lflags.oldcf;
                    *earw=lf_resw;
                    lflags.type=t_ADCw;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lflags.oldcf=get_CF()!=0;
                    lf_var1w=host_readw(eaa);
                    lf_var2w=*rmrw;
                    lf_resw=lf_var1w+lf_var2w+lflags.oldcf;
                    host_writew(eaa,lf_resw);
                    lflags.type=t_ADCw;
                }
            }
            break;	
        case (PREFIX_SIZE+0x12):
        case (PREFIX_NONE+0x12):									/* ADC r8, r/m8 */
            {
                Bit8u rm=Fetchb();
                Bit8u * rmrb;
                rmrb=lookupRMregb[rm];
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    lflags.oldcf=get_CF()!=0;
                    lf_var1b=*rmrb;
                    lf_var2b=*earb;
                    lf_resb=lf_var1b+lf_var2b+lflags.oldcf;
                    *rmrb=lf_resb;
                    lflags.type=t_ADCb;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lflags.oldcf=get_CF()!=0;
                    lf_var1b=*rmrb;
                    lf_var2b=host_readb(eaa);
                    lf_resb=lf_var1b+lf_var2b+lflags.oldcf;
                    *rmrb=lf_resb;
                    lflags.type=t_ADCb;
                }
            }
            break;
        case (PREFIX_SIZE+0x13):									/* ADC r16, r/m16 */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    lflags.oldcf=get_CF()!=0;
                    lf_var1w=*rmrw;
                    lf_var2w=*earw;
                    lf_resw=lf_var1w+lf_var2w+lflags.oldcf;
                    *rmrw=lf_resw;
                    lflags.type=t_ADCw;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lflags.oldcf=get_CF()!=0;
                    lf_var1w=*rmrw;
                    lf_var2w=host_readw(eaa);
                    lf_resw=lf_var1w+lf_var2w+lflags.oldcf;
                    *rmrw=lf_resw;
                    lflags.type=t_ADCw;
                }
            }
            break;
        case (PREFIX_SIZE+0x14):
        case (PREFIX_NONE+0x14):									/* ADC AL, imm8 */
            {
                lflags.oldcf=get_CF()!=0;
                lf_var1b=reg_al;
                lf_var2b=Fetchb();
                lf_resb=lf_var1b+lf_var2b+lflags.oldcf;
                reg_al=lf_resb;
                lflags.type=t_ADCb;
            }
            break;
        case (PREFIX_SIZE+0x15):									/* ADC AX, imm16 */
            {
                lflags.oldcf=get_CF()!=0;
                lf_var1w=reg_ax;
                lf_var2w=Fetchw();
                lf_resw=lf_var1w+lf_var2w+lflags.oldcf;
                reg_ax=lf_resw;
                lflags.type=t_ADCw;
            }
            break;
        case (PREFIX_SIZE+0x16):									/* PUSH SS */		
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0x17):									/* POP SS */		
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0x18):
        case (PREFIX_NONE+0x18):									/* SBB r/m8, r8 */
            {
                Bit8u rm=Fetchb();
                Bit8u * rmrb;
                rmrb=lookupRMregb[rm];
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    lflags.oldcf=get_CF()!=0;
                    lf_var1b=*earb;
                    lf_var2b=*rmrb;
                    lf_resb=lf_var1b-(lf_var2b+lflags.oldcf);
                    *earb=lf_resb;
                    lflags.type=t_SBBb;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lflags.oldcf=get_CF()!=0;
                    lf_var1b=host_readb(eaa);
                    lf_var2b=*rmrb;
                    lf_resb=lf_var1b-(lf_var2b+lflags.oldcf);
                    host_writeb(eaa,lf_resb);
                    lflags.type=t_SBBb;
                }
            }
            break;
        case (PREFIX_SIZE+0x19):									/* SBB r/m16, r16 */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    lflags.oldcf=get_CF()!=0;
                    lf_var1w=*earw;
                    lf_var2w=*rmrw;
                    lf_resw=lf_var1w-(lf_var2w+lflags.oldcf);
                    *earw=lf_resw;
                    lflags.type=t_SBBw;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lflags.oldcf=get_CF()!=0;
                    lf_var1w=host_readw(eaa);
                    lf_var2w=*rmrw;
                    lf_resw=lf_var1w-(lf_var2w+lflags.oldcf);
                    host_writew(eaa,lf_resw);
                    lflags.type=t_SBBw;
                }
            }
            break;
        case (PREFIX_SIZE+0x1a):
        case (PREFIX_NONE+0x1a):									/* SBB r8, r/m8 */
            {
                Bit8u rm=Fetchb();
                Bit8u * rmrb;
                rmrb=lookupRMregb[rm];
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    lflags.oldcf=get_CF()!=0;
                    lf_var1b=*rmrb;
                    lf_var2b=*earb;
                    lf_resb=lf_var1b-(lf_var2b+lflags.oldcf);
                    *rmrb=lf_resb;
                    lflags.type=t_SBBb;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lflags.oldcf=get_CF()!=0;
                    lf_var1b=*rmrb;
                    lf_var2b=host_readb(eaa);
                    lf_resb=lf_var1b-(lf_var2b+lflags.oldcf);
                    *rmrb=lf_resb;
                    lflags.type=t_SBBb;
                }
            }
            break;
        case (PREFIX_SIZE+0x1b):									/* SBB r16, r/m16 */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    lflags.oldcf=get_CF()!=0;
                    lf_var1w=*rmrw;
                    lf_var2w=*earw;
                    lf_resw=lf_var1w-(lf_var2w+lflags.oldcf);
                    *rmrw=lf_resw;
                    lflags.type=t_SBBw;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lflags.oldcf=get_CF()!=0;
                    lf_var1w=*rmrw;
                    lf_var2w=host_readw(eaa);
                    lf_resw=lf_var1w-(lf_var2w+lflags.oldcf);
                    *rmrw=lf_resw;
                    lflags.type=t_SBBw;
                }
            }
            break;
        case (PREFIX_SIZE+0x1c):
        case (PREFIX_NONE+0x1c):									/* SBB AL, imm8 */
            {
                lflags.oldcf=get_CF()!=0;
                lf_var1b=reg_al;
                lf_var2b=Fetchb();
                lf_resb=lf_var1b-(lf_var2b+lflags.oldcf);
                reg_al=lf_resb;
                lflags.type=t_SBBb;
            }
            break;
        case (PREFIX_SIZE+0x1d):									/* SBB AX, imm16 */
            {
                lflags.oldcf=get_CF()!=0;
                lf_var1w=reg_ax;
                lf_var2w=Fetchw();
                lf_resw=lf_var1w-(lf_var2w+lflags.oldcf);
                reg_ax=lf_resw;
                lflags.type=t_SBBw;
            }
            break;
        case (PREFIX_SIZE+0x1e):									/* PUSH DS */		
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0x1f):									/* POP DS */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0x20):
        case (PREFIX_NONE+0x20):									/* AND r/m8, r8 */
            {
                Bit8u rm=Fetchb();
                Bit8u * rmrb;
                rmrb=lookupRMregb[rm];
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    lf_var1b=*earb;
                    lf_var2b=*rmrb;
                    lf_resb=lf_var1b & lf_var2b;
                    *earb=lf_resb;
                    lflags.type=t_ANDb;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1b=host_readb(eaa);
                    lf_var2b=*rmrb;
                    lf_resb=lf_var1b & lf_var2b;
                    host_writeb(eaa,lf_resb);
                    lflags.type=t_ANDb;
                }
            }
            break;
        case (PREFIX_SIZE+0x21):									/* AND r/m16, r16 */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    lf_var1w=*earw;
                    lf_var2w=*rmrw;
                    lf_resw=lf_var1w & lf_var2w;
                    *earw=lf_resw;
                    lflags.type=t_ANDw;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1w=host_readw(eaa);
                    lf_var2w=*rmrw;
                    lf_resw=lf_var1w & lf_var2w;
                    host_writew(eaa,lf_resw);
                    lflags.type=t_ANDw;
                }
            }
            break;	
        case (PREFIX_SIZE+0x22):
        case (PREFIX_NONE+0x22):									/* AND r8, r/m8 */
            {
                Bit8u rm=Fetchb();
                Bit8u * rmrb;
                rmrb=lookupRMregb[rm];
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    lf_var1b=*rmrb;
                    lf_var2b=*earb;
                    lf_resb=lf_var1b & lf_var2b;
                    *rmrb=lf_resb;
                    lflags.type=t_ANDb;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1b=*rmrb;
                    lf_var2b=host_readb(eaa);
                    lf_resb=lf_var1b & lf_var2b;
                    *rmrb=lf_resb;
                    lflags.type=t_ANDb;
                }
            }
            break;
        case (PREFIX_SIZE+0x23):									/* AND r16, r/m16 */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    lf_var1w=*rmrw;
                    lf_var2w=*earw;
                    lf_resw=lf_var1w & lf_var2w;
                    *rmrw=lf_resw;
                    lflags.type=t_ANDw;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1w=*rmrw;
                    lf_var2w=host_readw(eaa);
                    lf_resw=lf_var1w & lf_var2w;
                    *rmrw=lf_resw;
                    lflags.type=t_ANDw;
                }
            }
            break;
        case (PREFIX_SIZE+0x24):
        case (PREFIX_NONE+0x24):									/* AND AL, imm8 */
            {
                lf_var1b=reg_al;
                lf_var2b=Fetchb();
                lf_resb=lf_var1b & lf_var2b;
                reg_al=lf_resb;
                lflags.type=t_ANDb;
            }
            break;
        case (PREFIX_SIZE+0x25):									/* AND AX, imm16 */
            {
                lf_var1w=reg_ax;
                lf_var2w=Fetchw();
                lf_resw=lf_var1w & lf_var2w;
                reg_ax=lf_resw;
                lflags.type=t_ANDw;
            }
            break;
        case (PREFIX_SIZE+0x26):
        case (PREFIX_NONE+0x26):									/* SEG ES: */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0x27):
        case (PREFIX_NONE+0x27):									/* DAA */
            if (((reg_al & 0x0F)>0x09) || get_AF()) {
                if ((reg_al > 0x99) || get_CF()) {
                    reg_al+=0x60;
                    reg_flags|=FLAG_CF;
                } else {
                    reg_flags&=~FLAG_CF;
                } 
                reg_al+=0x06;
                reg_flags|=FLAG_AF;
            } else {
                if ((reg_al > 0x99) || get_CF()) {
                    reg_al+=0x60;
                    reg_flags|=FLAG_CF;
                } else {
                    reg_flags&=~FLAG_CF;
                } 
                reg_flags&=~FLAG_AF;
            } 
            if ((reg_al&0x80)) reg_flags|=FLAG_SF; else reg_flags&=~FLAG_SF;
            if ((reg_al==0)) reg_flags|=FLAG_ZF; else reg_flags&=~FLAG_ZF;
            if (parity_lookup[reg_al]) reg_flags|=FLAG_PF; else reg_flags&=~FLAG_PF;
            lflags.type=t_UNKNOWN;
            break;
        case (PREFIX_SIZE+0x28):
        case (PREFIX_NONE+0x28):									/* SUB r/m8, r8 */
            {
                Bit8u rm=Fetchb();
                Bit8u * rmrb;
                rmrb=lookupRMregb[rm];
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    lf_var1b=*earb;
                    lf_var2b=*rmrb;
                    lf_resb=lf_var1b-lf_var2b;
                    *earb=lf_resb;
                    lflags.type=t_SUBb;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1b=host_readb(eaa);
                    lf_var2b=*rmrb;
                    lf_resb=lf_var1b-lf_var2b;
                    host_writeb(eaa,lf_resb);
                    lflags.type=t_SUBb;
                }
            }
            break;
        case (PREFIX_SIZE+0x29):									/* SUB r/m16, r16 */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    lf_var1w=*earw;
                    lf_var2w=*rmrw;
                    lf_resw=lf_var1w-lf_var2w;
                    *earw=lf_resw;
                    lflags.type=t_SUBw;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1w=host_readw(eaa);
                    lf_var2w=*rmrw;
                    lf_resw=lf_var1w-lf_var2w;
                    host_writew(eaa,lf_resw);
                    lflags.type=t_SUBw;
                }
            }
            break;
        case (PREFIX_SIZE+0x2a):
        case (PREFIX_NONE+0x2a):									/* SUB r8, r/m8 */
            {
                Bit8u rm=Fetchb();
                Bit8u * rmrb;
                rmrb=lookupRMregb[rm];
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    lf_var1b=*rmrb;
                    lf_var2b=*earb;
                    lf_resb=lf_var1b-lf_var2b;
                    *rmrb=lf_resb;
                    lflags.type=t_SUBb;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1b=*rmrb;
                    lf_var2b=host_readb(eaa);
                    lf_resb=lf_var1b-lf_var2b;
                    *rmrb=lf_resb;
                    lflags.type=t_SUBb;
                }
            }
            break;
        case (PREFIX_SIZE+0x2b):									/* SUB r16, r/m16 */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    lf_var1w=*rmrw;
                    lf_var2w=*earw;
                    lf_resw=lf_var1w-lf_var2w;
                    *rmrw=lf_resw;
                    lflags.type=t_SUBw;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1w=*rmrw;
                    lf_var2w=host_readw(eaa);
                    lf_resw=lf_var1w-lf_var2w;
                    *rmrw=lf_resw;
                    lflags.type=t_SUBw;
                }
            }
            break;
        case (PREFIX_SIZE+0x2c):
        case (PREFIX_NONE+0x2c):									/* SUB AL, imm8 */
            {
                lf_var1b=reg_al;
                lf_var2b=Fetchb();
                lf_resb=lf_var1b-lf_var2b;
                reg_al=lf_resb;
                lflags.type=t_SUBb;
            }
            break;
        case (PREFIX_SIZE+0x2d):									/* SUB AX, imm16 */
            {
                lf_var1w=reg_ax;
                lf_var2w=Fetchw();
                lf_resw=lf_var1w-lf_var2w;
                reg_ax=lf_resw;
                lflags.type=t_SUBw;
            }
            break;
        case (PREFIX_SIZE+0x2e):
        case (PREFIX_NONE+0x2e):									/* SEG CS: */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0x2f):
        case (PREFIX_NONE+0x2f):									/* DAS */
            {
                Bit8u osigned=reg_al & 0x80;
                if (((reg_al & 0x0f) > 9) || get_AF()) {
                    if ((reg_al>0x99) || get_CF()) {
                        reg_al-=0x60;
                        reg_flags|=FLAG_CF;
                    } else {
                        if ((reg_al<=0x05)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                    } 
                    reg_al-=6;
                    reg_flags|=FLAG_AF;
                } else {
                    if ((reg_al>0x99) || get_CF()) {
                        reg_al-=0x60;
                        reg_flags|=FLAG_CF;
                    } else {
                        reg_flags&=~FLAG_CF;
                    } 
                    reg_flags&=~FLAG_AF;
                } 
                if (osigned && ((reg_al&0x80)==0)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                if ((reg_al&0x80)) reg_flags|=FLAG_SF; else reg_flags&=~FLAG_SF;
                if ((reg_al==0)) reg_flags|=FLAG_ZF; else reg_flags&=~FLAG_ZF;
                if (parity_lookup[reg_al]) reg_flags|=FLAG_PF; else reg_flags&=~FLAG_PF;
                lflags.type=t_UNKNOWN;
            }
            break;  
        case (PREFIX_SIZE+0x30):
        case (PREFIX_NONE+0x30):									/* XOR r/m8, r8 */
            {
                Bit8u rm=Fetchb();
                Bit8u * rmrb;
                rmrb=lookupRMregb[rm];
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    lf_var1b=*earb;
                    lf_var2b=*rmrb;
                    lf_resb=lf_var1b ^ lf_var2b;
                    *earb=lf_resb;
                    lflags.type=t_XORb;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1b=host_readb(eaa);
                    lf_var2b=*rmrb;
                    lf_resb=lf_var1b ^ lf_var2b;
                    host_writeb(eaa,lf_resb);
                    lflags.type=t_XORb;
                }
            }
            break;
        case (PREFIX_SIZE+0x31):									/* XOR r/m16, r16 */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    lf_var1w=*earw;
                    lf_var2w=*rmrw;
                    lf_resw=lf_var1w ^ lf_var2w;
                    *earw=lf_resw;
                    lflags.type=t_XORw;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1w=host_readw(eaa);
                    lf_var2w=*rmrw;
                    lf_resw=lf_var1w ^ lf_var2w;
                    host_writew(eaa,lf_resw);
                    lflags.type=t_XORw;
                }
            }
            break;	
        case (PREFIX_SIZE+0x32):
        case (PREFIX_NONE+0x32):									/* XOR r8, r/m8 */
            {
                Bit8u rm=Fetchb();
                Bit8u * rmrb;
                rmrb=lookupRMregb[rm];
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    lf_var1b=*rmrb;
                    lf_var2b=*earb;
                    lf_resb=lf_var1b ^ lf_var2b;
                    *rmrb=lf_resb;
                    lflags.type=t_XORb;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1b=*rmrb;
                    lf_var2b=host_readb(eaa);
                    lf_resb=lf_var1b ^ lf_var2b;
                    *rmrb=lf_resb;
                    lflags.type=t_XORb;
                }
            }
            break;
        case (PREFIX_SIZE+0x33):									/* XOR r16, r/m16 */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    lf_var1w=*rmrw;
                    lf_var2w=*earw;
                    lf_resw=lf_var1w ^ lf_var2w;
                    *rmrw=lf_resw;
                    lflags.type=t_XORw;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1w=*rmrw;
                    lf_var2w=host_readw(eaa);
                    lf_resw=lf_var1w ^ lf_var2w;
                    *rmrw=lf_resw;
                    lflags.type=t_XORw;
                }
            }
            break;
        case (PREFIX_SIZE+0x34):
        case (PREFIX_NONE+0x34):									/* XOR AL, imm8 */
            {
                lf_var1b=reg_al;
                lf_var2b=Fetchb();
                lf_resb=lf_var1b ^ lf_var2b;
                reg_al=lf_resb;
                lflags.type=t_XORb;
            }
            break;
        case (PREFIX_SIZE+0x35):									/* XOR AX, imm16 */
            {
                lf_var1w=reg_ax;
                lf_var2w=Fetchw();
                lf_resw=lf_var1w ^ lf_var2w;
                reg_ax=lf_resw;
                lflags.type=t_XORw;
            }
            break;
        case (PREFIX_SIZE+0x36):
        case (PREFIX_NONE+0x36):									/* SEG SS: */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0x37):
        case (PREFIX_NONE+0x37):									/* AAA */
            if (((reg_al>=0x7a) && (reg_al<=0xf9))) reg_flags|=FLAG_SF; else reg_flags&=~FLAG_SF;
            if ((reg_al & 0xf) > 9) {
                if ((reg_al&0xf0)==0x70) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                reg_ax += 0x106;
                reg_flags|=FLAG_CF;
                if ((reg_al == 0)) reg_flags|=FLAG_ZF; else reg_flags&=~FLAG_ZF;
                reg_flags|=FLAG_AF;
            } else if (get_AF()) {
                reg_ax += 0x106;
                reg_flags&=~FLAG_OF;
                reg_flags|=FLAG_CF;
                reg_flags&=~FLAG_ZF;
                reg_flags|=FLAG_AF;
            } else {
                reg_flags&=~FLAG_OF;
                reg_flags&=~FLAG_CF;
                if ((reg_al == 0)) reg_flags|=FLAG_ZF; else reg_flags&=~FLAG_ZF;
                reg_flags&=~FLAG_AF;
            } 
            if (parity_lookup[reg_al]) reg_flags|=FLAG_PF; else reg_flags&=~FLAG_PF;
            reg_al &= 0x0F;
            lflags.type=t_UNKNOWN;
            break;  
        case (PREFIX_SIZE+0x38):
        case (PREFIX_NONE+0x38):									/* CMP r/m8, r8 */
            {
                Bit8u rm=Fetchb();
                Bit8u * rmrb;
                rmrb=lookupRMregb[rm];
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    lf_var1b=*earb;
                    lf_var2b=*rmrb;
                    lf_resb=lf_var1b-lf_var2b;
                    lflags.type=t_CMPb;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1b=host_readb(eaa);
                    lf_var2b=*rmrb;
                    lf_resb=lf_var1b-lf_var2b;
                    lflags.type=t_CMPb;
                }
            }
            break;
        case (PREFIX_SIZE+0x39):									/* CMP r/m16, r16 */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    lf_var1w=*earw;
                    lf_var2w=*rmrw;
                    lf_resw=lf_var1w-lf_var2w;
                    lflags.type=t_CMPw;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1w=host_readw(eaa);
                    lf_var2w=*rmrw;
                    lf_resw=lf_var1w-lf_var2w;
                    lflags.type=t_CMPw;
                }
            }
            break;
        case (PREFIX_SIZE+0x3a):
        case (PREFIX_NONE+0x3a):									/* CMP r8, r/m8 */
            {
                Bit8u rm=Fetchb();
                Bit8u * rmrb;
                rmrb=lookupRMregb[rm];
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    lf_var1b=*rmrb;
                    lf_var2b=*earb;
                    lf_resb=lf_var1b-lf_var2b;
                    lflags.type=t_CMPb;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1b=*rmrb;
                    lf_var2b=host_readb(eaa);
                    lf_resb=lf_var1b-lf_var2b;
                    lflags.type=t_CMPb;
                }
            }
            break;
        case (PREFIX_SIZE+0x3b):									/* CMP r16, r/m16 */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    lf_var1w=*rmrw;
                    lf_var2w=*earw;
                    lf_resw=lf_var1w-lf_var2w;
                    lflags.type=t_CMPw;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1w=*rmrw;
                    lf_var2w=host_readw(eaa);
                    lf_resw=lf_var1w-lf_var2w;
                    lflags.type=t_CMPw;
                }
            }
            break;
        case (PREFIX_SIZE+0x3c):
        case (PREFIX_NONE+0x3c):									/* CMP AL, imm8 */
            {
                lf_var1b=reg_al;
                lf_var2b=Fetchb();
                lf_resb=lf_var1b-lf_var2b;
                lflags.type=t_CMPb;
            }
            break;
        case (PREFIX_SIZE+0x3d):									/* CMP AX, imm16 */
            {
                lf_var1w=reg_ax;
                lf_var2w=Fetchw();
                lf_resw=lf_var1w-lf_var2w;
                lflags.type=t_CMPw;
            }
            break;
        case (PREFIX_SIZE+0x3e):
        case (PREFIX_NONE+0x3e):									/* SEG DS: */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0x3f):
        case (PREFIX_NONE+0x3f):									/* AAS */
            if ((reg_al & 0x0f)>9) {
                if ((reg_al>0x85)) reg_flags|=FLAG_SF; else reg_flags&=~FLAG_SF;
                reg_ax -= 0x106;
                reg_flags&=~FLAG_OF;
                reg_flags|=FLAG_CF;
                reg_flags|=FLAG_AF;
            } else if (get_AF()) {
                if (((reg_al>=0x80) && (reg_al<=0x85))) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                if ((reg_al<0x06) || (reg_al>0x85)) reg_flags|=FLAG_SF; else reg_flags&=~FLAG_SF;
                reg_ax -= 0x106;
                reg_flags|=FLAG_CF;
                reg_flags|=FLAG_AF;
            } else {
                if ((reg_al>=0x80)) reg_flags|=FLAG_SF; else reg_flags&=~FLAG_SF;
                reg_flags&=~FLAG_OF;
                reg_flags&=~FLAG_CF;
                reg_flags&=~FLAG_AF;
            } 
            if ((reg_al == 0)) reg_flags|=FLAG_ZF; else reg_flags&=~FLAG_ZF;
            if (parity_lookup[reg_al]) reg_flags|=FLAG_PF; else reg_flags&=~FLAG_PF;
            reg_al &= 0x0F;
            lflags.type=t_UNKNOWN;
            break;
        case (PREFIX_SIZE+0x40):									/* INC AX */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1w=reg_ax;
            lf_resw=lf_var1w+1;
            reg_ax=lf_resw;
            lflags.type=t_INCw;
            break;
        case (PREFIX_SIZE+0x41):									/* INC CX */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1w=reg_cx;
            lf_resw=lf_var1w+1;
            reg_cx=lf_resw;
            lflags.type=t_INCw;
            break;
        case (PREFIX_SIZE+0x42):									/* INC DX */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1w=reg_dx;
            lf_resw=lf_var1w+1;
            reg_dx=lf_resw;
            lflags.type=t_INCw;
            break;
        case (PREFIX_SIZE+0x43):									/* INC BX */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1w=reg_bx;
            lf_resw=lf_var1w+1;
            reg_bx=lf_resw;
            lflags.type=t_INCw;
            break;
        case (PREFIX_SIZE+0x44):									/* INC SP */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1w=reg_sp;
            lf_resw=lf_var1w+1;
            reg_sp=lf_resw;
            lflags.type=t_INCw;
            break;
        case (PREFIX_SIZE+0x45):									/* INC BP */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1w=reg_bp;
            lf_resw=lf_var1w+1;
            reg_bp=lf_resw;
            lflags.type=t_INCw;
            break;
        case (PREFIX_SIZE+0x46):									/* INC SI */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1w=reg_si;
            lf_resw=lf_var1w+1;
            reg_si=lf_resw;
            lflags.type=t_INCw;
            break;
        case (PREFIX_SIZE+0x47):									/* INC DI */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1w=reg_di;
            lf_resw=lf_var1w+1;
            reg_di=lf_resw;
            lflags.type=t_INCw;
            break;
        case (PREFIX_SIZE+0x48):									/* DEC AX */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1w=reg_ax;
            lf_resw=lf_var1w-1;
            reg_ax=lf_resw;
            lflags.type=t_DECw;
            break;
        case (PREFIX_SIZE+0x49):									/* DEC CX */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1w=reg_cx;
            lf_resw=lf_var1w-1;
            reg_cx=lf_resw;
            lflags.type=t_DECw;
            break;
        case (PREFIX_SIZE+0x4a):									/* DEC DX */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1w=reg_dx;
            lf_resw=lf_var1w-1;
            reg_dx=lf_resw;
            lflags.type=t_DECw;
            break;
        case (PREFIX_SIZE+0x4b):									/* DEC BX */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1w=reg_bx;
            lf_resw=lf_var1w-1;
            reg_bx=lf_resw;
            lflags.type=t_DECw;
            break;
        case (PREFIX_SIZE+0x4c):									/* DEC SP */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1w=reg_sp;
            lf_resw=lf_var1w-1;
            reg_sp=lf_resw;
            lflags.type=t_DECw;
            break;
        case (PREFIX_SIZE+0x4d):									/* DEC BP */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1w=reg_bp;
            lf_resw=lf_var1w-1;
            reg_bp=lf_resw;
            lflags.type=t_DECw;
            break;
        case (PREFIX_SIZE+0x4e):									/* DEC SI */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1w=reg_si;
            lf_resw=lf_var1w-1;
            reg_si=lf_resw;
            lflags.type=t_DECw;
            break;
        case (PREFIX_SIZE+0x4f):									/* DEC DI */
            if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            lf_var1w=reg_di;
            lf_resw=lf_var1w-1;
            reg_di=lf_resw;
            lflags.type=t_DECw;
            break;
        case (PREFIX_SIZE+0x50):									/* PUSH AX */
            CPU_Push16(reg_ax);
            break;
        case (PREFIX_SIZE+0x51):									/* PUSH CX */
            CPU_Push16(reg_cx);
            break;
        case (PREFIX_SIZE+0x52):									/* PUSH DX */
            CPU_Push16(reg_dx);
            break;
        case (PREFIX_SIZE+0x53):									/* PUSH BX */
            CPU_Push16(reg_bx);
            break;
        case (PREFIX_SIZE+0x54):									/* PUSH SP */
            CPU_Push16(reg_sp);
            break;
        case (PREFIX_SIZE+0x55):									/* PUSH BP */
            CPU_Push16(reg_bp);
            break;
        case (PREFIX_SIZE+0x56):									/* PUSH SI */
            CPU_Push16(reg_si);
            break;
        case (PREFIX_SIZE+0x57):									/* PUSH DI */
            CPU_Push16(reg_di);
            break;
        case (PREFIX_SIZE+0x58):									/* POP AX */
            reg_ax=CPU_Pop16();
            break;
        case (PREFIX_SIZE+0x59):									/* POP CX */
            reg_cx=CPU_Pop16();
            break;
        case (PREFIX_SIZE+0x5a):									/* POP DX */
            reg_dx=CPU_Pop16();
            break;
        case (PREFIX_SIZE+0x5b):									/* POP BX */
            reg_bx=CPU_Pop16();
            break;
        case (PREFIX_SIZE+0x5c):									/* POP SP */
            reg_sp=CPU_Pop16();
            break;
        case (PREFIX_SIZE+0x5d):									/* POP BP */
            reg_bp=CPU_Pop16();
            break;
        case (PREFIX_SIZE+0x5e):									/* POP SI */
            reg_si=CPU_Pop16();
            break;
        case (PREFIX_SIZE+0x5f):									/* POP DI */
            reg_di=CPU_Pop16();
            break;
        case (PREFIX_SIZE+0x60):									/* PUSHA */
            {
                Bit16u old_sp=reg_sp;
                CPU_Push16(reg_ax);CPU_Push16(reg_cx);CPU_Push16(reg_dx);CPU_Push16(reg_bx);
                CPU_Push16(old_sp);CPU_Push16(reg_bp);CPU_Push16(reg_si);CPU_Push16(reg_di);
            }
            break;
        case (PREFIX_SIZE+0x61):									/* POPA */
            reg_di=CPU_Pop16();
            reg_si=CPU_Pop16();
            reg_bp=CPU_Pop16();CPU_Pop16();
            reg_bx=CPU_Pop16();
            reg_dx=CPU_Pop16();
            reg_cx=CPU_Pop16();
            reg_ax=CPU_Pop16();
            break;
        case (PREFIX_SIZE+0x62):									/* BOUND */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0x63):									/* ARPL r/m16, r16 */
            goto illegal_opcode; // not implemented 

        case (PREFIX_SIZE+0x64):
        case (PREFIX_NONE+0x64):									/* SEG FS: */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0x65):
        case (PREFIX_NONE+0x65):									/* SEG GS: */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0x66):
        case (PREFIX_NONE+0x66):									/* Operand Size Prefix */
            core.prefix=PREFIX_SIZE;
            goto restart_opcode;
        case (PREFIX_SIZE+0x67):
        case (PREFIX_NONE+0x67):									/* Address Size Prefix */
            core.addr32=(core.addr32 & ~PREFIX_ADDR) |		
                (1 ^ PREFIX_ADDR);						
            core.ea_table=&EATable[(core.addr32&1) * 256];	
            goto restart_opcode;
        case (PREFIX_SIZE+0x68):									/* PUSH imm16 */
            CPU_Push16(Fetchw());
            break;
        case (PREFIX_SIZE+0x69):									/* IMUL r16, r/m16, imm16 */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];{
                        Bits res=((Bit16s)*earw) * ((Bit16s)Fetchw());
                        *rmrw=res & 0xffff; FillFlagsNoCFOF();
                        if ((res> -32768) && (res<32767)) {
                            reg_flags&=~FLAG_CF;reg_flags&=~FLAG_OF;
                        } else {
                            reg_flags|=FLAG_CF;reg_flags|=FLAG_OF;
                        }
                    }
                } else {
                    HostPt eaa=core.ea_table[rm]();{
                        Bits res=((Bit16s)host_readw(eaa)) * ((Bit16s)Fetchw());
                        *rmrw=res & 0xffff; FillFlagsNoCFOF();
                        if ((res> -32768) && (res<32767)) {
                            reg_flags&=~FLAG_CF;reg_flags&=~FLAG_OF;
                        } else {
                            reg_flags|=FLAG_CF;reg_flags|=FLAG_OF;
                        }
                    }
                }
            }
            break;
        case (PREFIX_SIZE+0x6a):									/* PUSH imm8 */
            CPU_Push16(Fetchb());
            break;
        case (PREFIX_SIZE+0x6b):									/* IMUL r16, r/m16, imm8 */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];{
                        Bits res=((Bit16s)*earw) * ((Bit16s)Fetchb());
                        *rmrw=res & 0xffff; FillFlagsNoCFOF();
                        if ((res> -32768) && (res<32767)) {
                            reg_flags&=~FLAG_CF;reg_flags&=~FLAG_OF;
                        } else {
                            reg_flags|=FLAG_CF;reg_flags|=FLAG_OF;
                        }
                    }
                } else {
                    HostPt eaa=core.ea_table[rm]();{
                        Bits res=((Bit16s)host_readw(eaa)) * ((Bit16s)Fetchb());
                        *rmrw=res & 0xffff; FillFlagsNoCFOF();
                        if ((res> -32768) && (res<32767)) {
                            reg_flags&=~FLAG_CF;reg_flags&=~FLAG_OF;
                        } else {
                            reg_flags|=FLAG_CF;reg_flags|=FLAG_OF;
                        }
                    }
                }
            }
            break;
        case (PREFIX_SIZE+0x6c):
        case (PREFIX_NONE+0x6c):									/* INSB */


            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0x6d):									/* INSW */


            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0x6e):
        case (PREFIX_NONE+0x6e):									/* OUTSB */


            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0x6f):									/* OUTSW */


            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0x70):									/* JO */
            {
                reg_eip=(core.cseip-MemBase);
                if ((get_OF())) reg_ip+=Fetchb();
                reg_ip+=1; continue;
            }
            break;
        case (PREFIX_SIZE+0x71):									/* JNO */
            {
                reg_eip=(core.cseip-MemBase);
                if ((!get_OF())) reg_ip+=Fetchb();
                reg_ip+=1; continue;
            }
            break;
        case (PREFIX_SIZE+0x72):									/* JB */
            {
                reg_eip=(core.cseip-MemBase);
                if ((get_CF())) reg_ip+=Fetchb();
                reg_ip+=1; continue;
            }
            break;
        case (PREFIX_SIZE+0x73):									/* JNB */
            {
                reg_eip=(core.cseip-MemBase);
                if ((!get_CF())) reg_ip+=Fetchb();
                reg_ip+=1; continue;
            }
            break;
        case (PREFIX_SIZE+0x74):									/* JZ */
            {
                reg_eip=(core.cseip-MemBase);
                if ((get_ZF())) reg_ip+=Fetchb();
                reg_ip+=1; continue;
            }
            break;
        case (PREFIX_SIZE+0x75):									/* JNZ */
            {
                reg_eip=(core.cseip-MemBase);
                if ((!get_ZF())) reg_ip+=Fetchb();
                reg_ip+=1; continue;
            }
            break;
        case (PREFIX_SIZE+0x76):									/* JBE */
            {
                reg_eip=(core.cseip-MemBase);
                if ((get_CF() || get_ZF())) reg_ip+=Fetchb();
                reg_ip+=1; continue;
            }
            break;
        case (PREFIX_SIZE+0x77):									/* JNBE */
            {
                reg_eip=(core.cseip-MemBase);
                if ((!get_CF() && !get_ZF())) reg_ip+=Fetchb();
                reg_ip+=1; continue;
            }
            break;
        case (PREFIX_SIZE+0x78):									/* JS */
            {
                reg_eip=(core.cseip-MemBase);
                if ((get_SF())) reg_ip+=Fetchb();
                reg_ip+=1; continue;
            }
            break;
        case (PREFIX_SIZE+0x79):									/* JNS */
            {
                reg_eip=(core.cseip-MemBase);
                if ((!get_SF())) reg_ip+=Fetchb();
                reg_ip+=1; continue;
            }
            break;
        case (PREFIX_SIZE+0x7a):									/* JP */
            {
                reg_eip=(core.cseip-MemBase);
                if ((get_PF())) reg_ip+=Fetchb();
                reg_ip+=1; continue;
            }
            break;
        case (PREFIX_SIZE+0x7b):									/* JNP */
            {
                reg_eip=(core.cseip-MemBase);
                if ((!get_PF())) reg_ip+=Fetchb();
                reg_ip+=1; continue;
            }
            break;
        case (PREFIX_SIZE+0x7c):									/* JL */
            {
                reg_eip=(core.cseip-MemBase);
                if (((get_SF()!=0) != (get_OF()!=0))) reg_ip+=Fetchb();
                reg_ip+=1; continue;
            }
            break;
        case (PREFIX_SIZE+0x7d):									/* JNL */
            {
                reg_eip=(core.cseip-MemBase);
                if (((get_SF()!=0) == (get_OF()!=0))) reg_ip+=Fetchb();
                reg_ip+=1; continue;
            }
            break;
        case (PREFIX_SIZE+0x7e):									/* JLE */
            {
                reg_eip=(core.cseip-MemBase);
                if ((get_ZF() || ((get_SF()!=0) != (get_OF()!=0)))) reg_ip+=Fetchb();
                reg_ip+=1; continue;
            }
            break;
        case (PREFIX_SIZE+0x7f):									/* JNLE */
            {
                reg_eip=(core.cseip-MemBase);
                if ((!get_ZF() && ((get_SF()!=0) == (get_OF()!=0)))) reg_ip+=Fetchb();
                reg_ip+=1; continue;
            }
            break;
        case (PREFIX_SIZE+0x80):
        case (PREFIX_NONE+0x80):									/* Grpl r/m8, imm8 */
        case (PREFIX_SIZE+0x82):
        case (PREFIX_NONE+0x82):									/* Grpl r/m8, imm8 Mirror instruction*/
            {
                Bit8u rm=Fetchb();
                Bitu which=(rm>>3)&7;
                if (rm>= 0xc0) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    Bit8u ib=Fetchb();
                    switch (which) {
                    case 0x00:
                        lf_var1b=*earb;
                        lf_var2b=ib;
                        lf_resb=lf_var1b+lf_var2b;
                        *earb=lf_resb;
                        lflags.type=t_ADDb;
                        break;
                    case 0x01: lf_var1b=*earb;
                        lf_var2b=ib;
                        lf_resb=lf_var1b | lf_var2b;
                        *earb=lf_resb;
                        lflags.type=t_ORb;
                        break;
                    case 0x02:lflags.oldcf=get_CF()!=0;
                        lf_var1b=*earb;
                        lf_var2b=ib;
                        lf_resb=lf_var1b+lf_var2b+lflags.oldcf;
                        *earb=lf_resb;
                        lflags.type=t_ADCb;
                        break;
                    case 0x03:lflags.oldcf=get_CF()!=0;
                        lf_var1b=*earb;
                        lf_var2b=ib;
                        lf_resb=lf_var1b-(lf_var2b+lflags.oldcf);
                        *earb=lf_resb;
                        lflags.type=t_SBBb;
                        break;
                    case 0x04:
                        lf_var1b=*earb;
                        lf_var2b=ib;
                        lf_resb=lf_var1b & lf_var2b;
                        *earb=lf_resb;
                        lflags.type=t_ANDb;
                        break;
                    case 0x05:
                        lf_var1b=*earb;
                        lf_var2b=ib;
                        lf_resb=lf_var1b-lf_var2b;
                        *earb=lf_resb;
                        lflags.type=t_SUBb;
                        break;
                    case 0x06:
                        lf_var1b=*earb;
                        lf_var2b=ib;
                        lf_resb=lf_var1b ^ lf_var2b;
                        *earb=lf_resb;
                        lflags.type=t_XORb;
                        break;
                    case 0x07:
                        lf_var1b=*earb;
                        lf_var2b=ib;
                        lf_resb=lf_var1b-lf_var2b;
                        lflags.type=t_CMPb;
                        break;
                    }
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    Bit8u ib=Fetchb();
                    switch (which) {
                    case 0x00:
                        lf_var1b=host_readb(eaa);
                        lf_var2b=ib;
                        lf_resb=lf_var1b+lf_var2b;
                        host_writeb(eaa,lf_resb);
                        lflags.type=t_ADDb;
                        break;
                    case 0x01: lf_var1b=host_readb(eaa);
                        lf_var2b=ib;
                        lf_resb=lf_var1b | lf_var2b;
                        host_writeb(eaa,lf_resb);
                        lflags.type=t_ORb;
                        break;
                    case 0x02:lflags.oldcf=get_CF()!=0;
                        lf_var1b=host_readb(eaa);
                        lf_var2b=ib;
                        lf_resb=lf_var1b+lf_var2b+lflags.oldcf;
                        host_writeb(eaa,lf_resb);
                        lflags.type=t_ADCb;
                        break;
                    case 0x03:lflags.oldcf=get_CF()!=0;
                        lf_var1b=host_readb(eaa);
                        lf_var2b=ib;
                        lf_resb=lf_var1b-(lf_var2b+lflags.oldcf);
                        host_writeb(eaa,lf_resb);
                        lflags.type=t_SBBb;
                        break;
                    case 0x04:
                        lf_var1b=host_readb(eaa);
                        lf_var2b=ib;
                        lf_resb=lf_var1b & lf_var2b;
                        host_writeb(eaa,lf_resb);
                        lflags.type=t_ANDb;
                        break;
                    case 0x05:
                        lf_var1b=host_readb(eaa);
                        lf_var2b=ib;
                        lf_resb=lf_var1b-lf_var2b;
                        host_writeb(eaa,lf_resb);
                        lflags.type=t_SUBb;
                        break;
                    case 0x06:
                        lf_var1b=host_readb(eaa);
                        lf_var2b=ib;
                        lf_resb=lf_var1b ^ lf_var2b;
                        host_writeb(eaa,lf_resb);
                        lflags.type=t_XORb;
                        break;
                    case 0x07:
                        lf_var1b=host_readb(eaa);
                        lf_var2b=ib;
                        lf_resb=lf_var1b-lf_var2b;
                        lflags.type=t_CMPb;
                        break;
                    }
                }
                break;
            }
        case (PREFIX_SIZE+0x81):									/* Grpl r/m16, imm16 */
            {
                Bit8u rm=Fetchb();
                Bitu which=(rm>>3)&7;
                if (rm>= 0xc0) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    Bit16u iw=Fetchw();
                    switch (which) {
                    case 0x00:
                        lf_var1w=*earw;
                        lf_var2w=iw;
                        lf_resw=lf_var1w+lf_var2w;
                        *earw=lf_resw;
                        lflags.type=t_ADDw;
                        break;
                    case 0x01: lf_var1w=*earw;
                        lf_var2w=iw;
                        lf_resw=lf_var1w | lf_var2w;
                        *earw=lf_resw;
                        lflags.type=t_ORw;
                        break;
                    case 0x02:lflags.oldcf=get_CF()!=0;
                        lf_var1w=*earw;
                        lf_var2w=iw;
                        lf_resw=lf_var1w+lf_var2w+lflags.oldcf;
                        *earw=lf_resw;
                        lflags.type=t_ADCw;
                        break;
                    case 0x03:lflags.oldcf=get_CF()!=0;
                        lf_var1w=*earw;
                        lf_var2w=iw;
                        lf_resw=lf_var1w-(lf_var2w+lflags.oldcf);
                        *earw=lf_resw;
                        lflags.type=t_SBBw;
                        break;
                    case 0x04:
                        lf_var1w=*earw;
                        lf_var2w=iw;
                        lf_resw=lf_var1w & lf_var2w;
                        *earw=lf_resw;
                        lflags.type=t_ANDw;
                        break;
                    case 0x05:
                        lf_var1w=*earw;
                        lf_var2w=iw;
                        lf_resw=lf_var1w-lf_var2w;
                        *earw=lf_resw;
                        lflags.type=t_SUBw;
                        break;
                    case 0x06:
                        lf_var1w=*earw;
                        lf_var2w=iw;
                        lf_resw=lf_var1w ^ lf_var2w;
                        *earw=lf_resw;
                        lflags.type=t_XORw;
                        break;
                    case 0x07:
                        lf_var1w=*earw;
                        lf_var2w=iw;
                        lf_resw=lf_var1w-lf_var2w;
                        lflags.type=t_CMPw;
                        break;
                    }
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    Bit16u iw=Fetchw();
                    switch (which) {
                    case 0x00:
                        lf_var1w=host_readw(eaa);
                        lf_var2w=iw;
                        lf_resw=lf_var1w+lf_var2w;
                        host_writew(eaa,lf_resw);
                        lflags.type=t_ADDw;
                        break;
                    case 0x01: lf_var1w=host_readw(eaa);
                        lf_var2w=iw;
                        lf_resw=lf_var1w | lf_var2w;
                        host_writew(eaa,lf_resw);
                        lflags.type=t_ORw;
                        break;
                    case 0x02:lflags.oldcf=get_CF()!=0;
                        lf_var1w=host_readw(eaa);
                        lf_var2w=iw;
                        lf_resw=lf_var1w+lf_var2w+lflags.oldcf;
                        host_writew(eaa,lf_resw);
                        lflags.type=t_ADCw;
                        break;
                    case 0x03:lflags.oldcf=get_CF()!=0;
                        lf_var1w=host_readw(eaa);
                        lf_var2w=iw;
                        lf_resw=lf_var1w-(lf_var2w+lflags.oldcf);
                        host_writew(eaa,lf_resw);
                        lflags.type=t_SBBw;
                        break;
                    case 0x04:
                        lf_var1w=host_readw(eaa);
                        lf_var2w=iw;
                        lf_resw=lf_var1w & lf_var2w;
                        host_writew(eaa,lf_resw);
                        lflags.type=t_ANDw;
                        break;
                    case 0x05:
                        lf_var1w=host_readw(eaa);
                        lf_var2w=iw;
                        lf_resw=lf_var1w-lf_var2w;
                        host_writew(eaa,lf_resw);
                        lflags.type=t_SUBw;
                        break;
                    case 0x06:
                        lf_var1w=host_readw(eaa);
                        lf_var2w=iw;
                        lf_resw=lf_var1w ^ lf_var2w;
                        host_writew(eaa,lf_resw);
                        lflags.type=t_XORw;
                        break;
                    case 0x07:
                        lf_var1w=host_readw(eaa);
                        lf_var2w=iw;
                        lf_resw=lf_var1w-lf_var2w;
                        lflags.type=t_CMPw;
                        break;
                    }
                }
                break;
            }
        case (PREFIX_SIZE+0x83):									/* Grpl r/m16, imm8 */
            {
                Bit8u rm=Fetchb();
                Bitu which=(rm>>3)&7;
                if (rm>= 0xc0) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    Bit16u iw=(Bit16s)Fetchb();
                    switch (which) {
                    case 0x00:
                        lf_var1w=*earw;
                        lf_var2w=iw;
                        lf_resw=lf_var1w+lf_var2w;
                        *earw=lf_resw;
                        lflags.type=t_ADDw;
                        break;
                    case 0x01: lf_var1w=*earw;
                        lf_var2w=iw;
                        lf_resw=lf_var1w | lf_var2w;
                        *earw=lf_resw;
                        lflags.type=t_ORw;
                        break;
                    case 0x02:lflags.oldcf=get_CF()!=0;
                        lf_var1w=*earw;
                        lf_var2w=iw;
                        lf_resw=lf_var1w+lf_var2w+lflags.oldcf;
                        *earw=lf_resw;
                        lflags.type=t_ADCw;
                        break;
                    case 0x03:lflags.oldcf=get_CF()!=0;
                        lf_var1w=*earw;
                        lf_var2w=iw;
                        lf_resw=lf_var1w-(lf_var2w+lflags.oldcf);
                        *earw=lf_resw;
                        lflags.type=t_SBBw;
                        break;
                    case 0x04:
                        lf_var1w=*earw;
                        lf_var2w=iw;
                        lf_resw=lf_var1w & lf_var2w;
                        *earw=lf_resw;
                        lflags.type=t_ANDw;
                        break;
                    case 0x05:
                        lf_var1w=*earw;
                        lf_var2w=iw;
                        lf_resw=lf_var1w-lf_var2w;
                        *earw=lf_resw;
                        lflags.type=t_SUBw;
                        break;
                    case 0x06:
                        lf_var1w=*earw;
                        lf_var2w=iw;
                        lf_resw=lf_var1w ^ lf_var2w;
                        *earw=lf_resw;
                        lflags.type=t_XORw;
                        break;
                    case 0x07:
                        lf_var1w=*earw;
                        lf_var2w=iw;
                        lf_resw=lf_var1w-lf_var2w;
                        lflags.type=t_CMPw;
                        break;
                    }
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    Bit16u iw=(Bit16s)Fetchb();
                    switch (which) {
                    case 0x00:
                        lf_var1w=host_readw(eaa);
                        lf_var2w=iw;
                        lf_resw=lf_var1w+lf_var2w;
                        host_writew(eaa,lf_resw);
                        lflags.type=t_ADDw;
                        break;
                    case 0x01: lf_var1w=host_readw(eaa);
                        lf_var2w=iw;
                        lf_resw=lf_var1w | lf_var2w;
                        host_writew(eaa,lf_resw);
                        lflags.type=t_ORw;
                        break;
                    case 0x02:lflags.oldcf=get_CF()!=0;
                        lf_var1w=host_readw(eaa);
                        lf_var2w=iw;
                        lf_resw=lf_var1w+lf_var2w+lflags.oldcf;
                        host_writew(eaa,lf_resw);
                        lflags.type=t_ADCw;
                        break;
                    case 0x03:lflags.oldcf=get_CF()!=0;
                        lf_var1w=host_readw(eaa);
                        lf_var2w=iw;
                        lf_resw=lf_var1w-(lf_var2w+lflags.oldcf);
                        host_writew(eaa,lf_resw);
                        lflags.type=t_SBBw;
                        break;
                    case 0x04:
                        lf_var1w=host_readw(eaa);
                        lf_var2w=iw;
                        lf_resw=lf_var1w & lf_var2w;
                        host_writew(eaa,lf_resw);
                        lflags.type=t_ANDw;
                        break;
                    case 0x05:
                        lf_var1w=host_readw(eaa);
                        lf_var2w=iw;
                        lf_resw=lf_var1w-lf_var2w;
                        host_writew(eaa,lf_resw);
                        lflags.type=t_SUBw;
                        break;
                    case 0x06:
                        lf_var1w=host_readw(eaa);
                        lf_var2w=iw;
                        lf_resw=lf_var1w ^ lf_var2w;
                        host_writew(eaa,lf_resw);
                        lflags.type=t_XORw;
                        break;
                    case 0x07:
                        lf_var1w=host_readw(eaa);
                        lf_var2w=iw;
                        lf_resw=lf_var1w-lf_var2w;
                        lflags.type=t_CMPw;
                        break;
                    }
                }
                break;
            }
        case (PREFIX_SIZE+0x84):
        case (PREFIX_NONE+0x84):									/* TEST r/m8, r8 */
            {
                Bit8u rm=Fetchb();
                Bit8u * rmrb;
                rmrb=lookupRMregb[rm];
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    lf_var1b=*earb;
                    lf_var2b=*rmrb;
                    lf_resb=lf_var1b & lf_var2b;
                    lflags.type=t_TESTb;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1b=host_readb(eaa);
                    lf_var2b=*rmrb;
                    lf_resb=lf_var1b & lf_var2b;
                    lflags.type=t_TESTb;
                }
            }
            break;
        case (PREFIX_SIZE+0x85):									/* TEST r/m16, r16 */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    lf_var1w=*earw;
                    lf_var2w=*rmrw;
                    lf_resw=lf_var1w & lf_var2w;
                    lflags.type=t_TESTw;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    lf_var1w=host_readw(eaa);
                    lf_var2w=*rmrw;
                    lf_resw=lf_var1w & lf_var2w;
                    lflags.type=t_TESTw;
                }
            }
            break;
        case (PREFIX_SIZE+0x86):
        case (PREFIX_NONE+0x86):									/* XCHG r/m8, r8 */
            {	
                Bit8u rm=Fetchb();
                Bit8u * rmrb;
                rmrb=lookupRMregb[rm];
                Bit8u oldrmrb=*rmrb;
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    *rmrb=*earb;
                    *earb=oldrmrb;
                }
                else {
                    HostPt eaa=core.ea_table[rm]();
                    *rmrb=host_readb(eaa);
                    host_writeb(eaa,oldrmrb);
                }
                break;
            }
        case (PREFIX_SIZE+0x87):									/* XCHG r/m16, r16 */
            {	
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                Bit16u oldrmrw=*rmrw;
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    *rmrw=*earw;
                    *earw=oldrmrw;
                }
                else {
                    HostPt eaa=core.ea_table[rm]();
                    *rmrw=host_readw(eaa);
                    host_writew(eaa,oldrmrw);
                }
                break;
            }
        case (PREFIX_SIZE+0x88):
        case (PREFIX_NONE+0x88):									/* MOV r/m8, r8 */
            {	
                Bit8u rm=Fetchb();
                Bit8u * rmrb;
                rmrb=lookupRMregb[rm];
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    *earb=*rmrb;
                }
                else {
                    HostPt eaa=core.ea_table[rm]();
                    host_writeb(eaa,*rmrb);
                }
                break;
            }
        case (PREFIX_SIZE+0x89):									/* MOV r/m16, r16 */
            {	
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    *earw=*rmrw;
                }
                else {
                    HostPt eaa=core.ea_table[rm]();
                    host_writew(eaa,*rmrw);
                }
                break;
            }
        case (PREFIX_SIZE+0x8a):
        case (PREFIX_NONE+0x8a):									/* MOV r8, r/m8 */
            {	
                Bit8u rm=Fetchb();
                Bit8u * rmrb;
                rmrb=lookupRMregb[rm];
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    *rmrb=*earb;
                }
                else {
                    HostPt eaa=core.ea_table[rm]();
                    *rmrb=host_readb(eaa);
                }
                break;
            }
        case (PREFIX_SIZE+0x8b):									/* MOV r16, r/m16 */
            {	
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    *rmrw=*earw;
                }
                else {
                    HostPt eaa=core.ea_table[rm]();
                    *rmrw=host_readw(eaa);
                }
                break;
            }
        case (PREFIX_SIZE+0x8c):									/* Mov r/m16,Sreg */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0x8d):									/* LEA r16 */
            {

                core.base_ds=core.base_ss=0;
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                if (core.addr32 & PREFIX_ADDR) {
                    *rmrw=(Bit16u)(*EATable[256+rm])();
                } else {
                    *rmrw=(Bit16u)(*EATable[rm])();
                }
                break;
            }
        case (PREFIX_SIZE+0x8e):
        case (PREFIX_NONE+0x8e):									/* MOV Sreg, r/m16 */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0x8f):									/* POP r/m16 */
            {
                Bit16u val=CPU_Pop16();
                Bit8u rm=Fetchb();
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    *earw=val;
                }
                else {
                    HostPt eaa=core.ea_table[rm]();
                    host_writew(eaa,val);
                }
                break;
            }
        case (PREFIX_SIZE+0x90):
        case (PREFIX_NONE+0x90):									/* NOP */
            break;
        case (PREFIX_SIZE+0x91):									/* XCHG CX,AX */
            {
                Bit16u temp=reg_ax;
                reg_ax=reg_cx;
                reg_cx=temp;
            }
            break;
        case (PREFIX_SIZE+0x92):									/* XCHG DX,AX */
            {
                Bit16u temp=reg_ax;
                reg_ax=reg_dx;
                reg_dx=temp;
            }
            break;
        case (PREFIX_SIZE+0x93):									/* XCHG BX,AX */
            {
                Bit16u temp=reg_ax;
                reg_ax=reg_bx;
                reg_bx=temp;
            }
            break;
        case (PREFIX_SIZE+0x94):									/* XCHG SP,AX */
            {
                Bit16u temp=reg_ax;
                reg_ax=reg_sp;
                reg_sp=temp;
            }
            break;
        case (PREFIX_SIZE+0x95):									/* XCHG BP,AX */
            {
                Bit16u temp=reg_ax;
                reg_ax=reg_bp;
                reg_bp=temp;
            }
            break;
        case (PREFIX_SIZE+0x96):									/* XCHG SI,AX */
            {
                Bit16u temp=reg_ax;
                reg_ax=reg_si;
                reg_si=temp;
            }
            break;
        case (PREFIX_SIZE+0x97):									/* XCHG DI,AX */
            {
                Bit16u temp=reg_ax;
                reg_ax=reg_di;
                reg_di=temp;
            }
            break;
        case (PREFIX_SIZE+0x98):									/* CBW */
            reg_ax=(Bit8s)reg_al;
            break;
        case (PREFIX_SIZE+0x99):									/* CWD */
            if (reg_ax & 0x8000) reg_dx=0xffff;else reg_dx=0;
            break;
        case (PREFIX_SIZE+0x9a):									/* CALL ptr16:32 */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0x9b):
        case (PREFIX_NONE+0x9b):									/* WAIT */
            break; /* No waiting here */
        case (PREFIX_SIZE+0x9c):									/* PUSHF */
            CPU_PUSHF(false);
            break;
        case (PREFIX_SIZE+0x9d):									/* POPF */
            CPU_POPF(false);
            break;
        case (PREFIX_SIZE+0x9e):
        case (PREFIX_NONE+0x9e):									/* SAHF */
            {
                if (get_OF()) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                lflags.type=t_UNKNOWN; CPU_SetFlags(reg_ah,FMASK_NORMAL & 0xff);
            }
            break;
        case (PREFIX_SIZE+0x9f):
        case (PREFIX_NONE+0x9f):									/* LAHF */
            FillFlags();
            reg_ah=reg_flags&0xff;
            break;
        case (PREFIX_SIZE+0xa0):
        case (PREFIX_NONE+0xa0):									/* MOV AL, moffs8 */
            {
                HostPt eaa;
                if (core.addr32 & PREFIX_ADDR) {
                    eaa=core.base_ds+Fetchd();
                } else {
                    eaa=core.base_ds+Fetchw();
                }
                reg_al=host_readb(eaa);
            }
            break;
        case (PREFIX_SIZE+0xa1):									/* MOV AX, moffs16 */
            {
                HostPt eaa;
                if (core.addr32 & PREFIX_ADDR) {
                    eaa=core.base_ds+Fetchd();
                } else {
                    eaa=core.base_ds+Fetchw();
                }
                reg_ax=host_readw(eaa);
            }
            break;
        case (PREFIX_SIZE+0xa2):
        case (PREFIX_NONE+0xa2):									/* MOV moffs8,AL */
            {
                HostPt eaa;
                if (core.addr32 & PREFIX_ADDR) {
                    eaa=core.base_ds+Fetchd();
                } else {
                    eaa=core.base_ds+Fetchw();
                }
                host_writeb(eaa,reg_al);
            }
            break;
        case (PREFIX_SIZE+0xa3):									/* MOV moffs16,AX */
            {
                HostPt eaa;
                if (core.addr32 & PREFIX_ADDR) {
                    eaa=core.base_ds+Fetchd();
                } else {
                    eaa=core.base_ds+Fetchw();
                }
                host_writew(eaa,reg_ax);
            }
            break;
        case (PREFIX_SIZE+0xa4):
        case (PREFIX_NONE+0xa4):									/* MOVSB */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0xa5):									/* MOVSW */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0xa6):
        case (PREFIX_NONE+0xa6):									/* CMPSB */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0xa7):									/* CMPSW */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0xa8):
        case (PREFIX_NONE+0xa8):									/* TEST AL, imm8 */
            {
                lf_var1b=reg_al;
                lf_var2b=Fetchb();
                lf_resb=lf_var1b & lf_var2b;
                lflags.type=t_TESTb;
            }
            break;
        case (PREFIX_SIZE+0xa9):									/* TEST AX, imm16 */
            {
                lf_var1w=reg_ax;
                lf_var2w=Fetchw();
                lf_resw=lf_var1w & lf_var2w;
                lflags.type=t_TESTw;
            }
            break;
        case (PREFIX_SIZE+0xaa):
        case (PREFIX_NONE+0xaa):									/* STOSB */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0xab):									/* STOSW */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0xac):
        case (PREFIX_NONE+0xac):									/* LODSB */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0xad):									/* LODSW */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0xae):
        case (PREFIX_NONE+0xae):									/* SCASB */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0xaf):									/* SCASW */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0xb0):
        case (PREFIX_NONE+0xb0):									/* MOV AL, imm8 */
            reg_al=Fetchb();
            break;
        case (PREFIX_SIZE+0xb1):
        case (PREFIX_NONE+0xb1):									/* MOV CL, imm8 */
            reg_cl=Fetchb();
            break;
        case (PREFIX_SIZE+0xb2):
        case (PREFIX_NONE+0xb2):									/* MOV DL, imm8 */
            reg_dl=Fetchb();
            break;
        case (PREFIX_SIZE+0xb3):
        case (PREFIX_NONE+0xb3):									/* MOV BL, imm8 */
            reg_bl=Fetchb();
            break;
        case (PREFIX_SIZE+0xb4):
        case (PREFIX_NONE+0xb4):									/* MOV AH, imm8 */
            reg_ah=Fetchb();
            break;
        case (PREFIX_SIZE+0xb5):
        case (PREFIX_NONE+0xb5):									/* MOV CH, imm8 */
            reg_ch=Fetchb();
            break;
        case (PREFIX_SIZE+0xb6):
        case (PREFIX_NONE+0xb6):									/* MOV DH, imm8 */
            reg_dh=Fetchb();
            break;
        case (PREFIX_SIZE+0xb7):
        case (PREFIX_NONE+0xb7):									/* MOV BH, imm8 */
            reg_bh=Fetchb();
            break;
        case (PREFIX_SIZE+0xb8):									/* MOV AX, imm16 */
            reg_ax=Fetchw();
            break;
        case (PREFIX_SIZE+0xb9):									/* MOV CX, imm16 */
            reg_cx=Fetchw();
            break;
        case (PREFIX_SIZE+0xba):									/* MOV DX, imm16 */
            reg_dx=Fetchw();
            break;
        case (PREFIX_SIZE+0xbb):									/* MOV BX, imm16 */
            reg_bx=Fetchw();
            break;
        case (PREFIX_SIZE+0xbc):									/* MOV SP, imm16 */
            reg_sp=Fetchw();
            break;
        case (PREFIX_SIZE+0xbd):									/* MOV BP, imm16 */
            reg_bp=Fetchw();
            break;
        case (PREFIX_SIZE+0xbe):									/* MOV SI, imm16 */
            reg_si=Fetchw();
            break;
        case (PREFIX_SIZE+0xbf):									/* MOV DI, imm16 */
            reg_di=Fetchw();
            break;
        case (PREFIX_SIZE+0xc0):
        case (PREFIX_NONE+0xc0):									/* GRP2 r/m8, imm8 */
            {
                Bit8u rm=Fetchb();
                Bitu which=(rm>>3)&7;
                if (rm >= 0xc0) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    Bit8u val=Fetchb() & 0x1f; 
                    switch (which) {
                    case 0x00:
                        FillFlagsNoCFOF();
                        lf_var1b=*earb;
                        lf_var2b=val&0x07;
                        lf_resb=(lf_var1b << lf_var2b) | (lf_var1b >> (8-lf_var2b));
                        *earb=lf_resb;
                        if (lf_resb & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resb & 1) ^ (lf_resb >> 7)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x01:
                        FillFlagsNoCFOF();
                        lf_var1b=*earb;
                        lf_var2b=val&0x07;
                        lf_resb=(lf_var1b >> lf_var2b) | (lf_var1b << (8-lf_var2b));
                        *earb=lf_resb;
                        if (lf_resb & 0x80) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resb ^ (lf_resb<<1)) & 0x80) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x02:
                        if (!(val%9)) break; {
                            Bit8u cf=(Bit8u)FillFlags()&0x1;
                            lf_var1b=*earb;
                            lf_var2b=val%9;
                            lf_resb=(lf_var1b << lf_var2b) | (cf << (lf_var2b-1)) | (lf_var1b >> (9-lf_var2b));
                            *earb=lf_resb;
                            if (((lf_var1b >> (8-lf_var2b)) & 1)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((reg_flags & 1) ^ (lf_resb >> 7)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x03:
                        if (val%9) {
                            Bit8u cf=(Bit8u)FillFlags()&0x1;
                            lf_var1b=*earb;
                            lf_var2b=val%9;
                            lf_resb=(lf_var1b >> lf_var2b) | (cf << (8-lf_var2b)) | (lf_var1b << (9-lf_var2b));
                            *earb=lf_resb;
                            if ((lf_var1b >> (lf_var2b - 1)) & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((lf_resb ^ (lf_resb<<1)) & 0x80) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x04:
                    case 0x06:
                        if (!val) break;
                        lf_var1b=*earb;
                        lf_var2b=val;
                        lf_resb=lf_var1b << lf_var2b;
                        *earb=lf_resb;
                        lflags.type=t_SHLb;
                        break; case 0x05:if (!val) break;
                        lf_var1b=*earb;
                        lf_var2b=val;
                        lf_resb=lf_var1b >> lf_var2b;
                        *earb=lf_resb;
                        lflags.type=t_SHRb;
                        break; 
                    case 0x07:
                        if (!val) break;
                        lf_var1b=*earb;
                        lf_var2b=val;
                        if (lf_var2b>8) lf_var2b=8;
                        if (lf_var1b & 0x80) {
                            lf_resb=(lf_var1b >> lf_var2b)| (0xff << (8 - lf_var2b));
                        } else {
                            lf_resb=lf_var1b >> lf_var2b;
                        } 
                        *earb=lf_resb;
                        lflags.type=t_SARb;
                        break;
                    }
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    Bit8u val=Fetchb() & 0x1f; 
                    switch (which) {
                    case 0x00:
                        FillFlagsNoCFOF();
                        lf_var1b=host_readb(eaa);
                        lf_var2b=val&0x07;
                        lf_resb=(lf_var1b << lf_var2b) | (lf_var1b >> (8-lf_var2b));
                        host_writeb(eaa,lf_resb);
                        if (lf_resb & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resb & 1) ^ (lf_resb >> 7)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x01:
                        FillFlagsNoCFOF();
                        lf_var1b=host_readb(eaa);
                        lf_var2b=val&0x07;
                        lf_resb=(lf_var1b >> lf_var2b) | (lf_var1b << (8-lf_var2b));
                        host_writeb(eaa,lf_resb);
                        if (lf_resb & 0x80) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resb ^ (lf_resb<<1)) & 0x80) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x02:
                        if (!(val%9)) break; {
                            Bit8u cf=(Bit8u)FillFlags()&0x1;
                            lf_var1b=host_readb(eaa);
                            lf_var2b=val%9;
                            lf_resb=(lf_var1b << lf_var2b) | (cf << (lf_var2b-1)) | (lf_var1b >> (9-lf_var2b));
                            host_writeb(eaa,lf_resb);
                            if (((lf_var1b >> (8-lf_var2b)) & 1)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((reg_flags & 1) ^ (lf_resb >> 7)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x03:
                        if (val%9) {
                            Bit8u cf=(Bit8u)FillFlags()&0x1;
                            lf_var1b=host_readb(eaa);
                            lf_var2b=val%9;
                            lf_resb=(lf_var1b >> lf_var2b) | (cf << (8-lf_var2b)) | (lf_var1b << (9-lf_var2b));
                            host_writeb(eaa,lf_resb);
                            if ((lf_var1b >> (lf_var2b - 1)) & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((lf_resb ^ (lf_resb<<1)) & 0x80) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x04:
                    case 0x06:
                        if (!val) break;
                        lf_var1b=host_readb(eaa);
                        lf_var2b=val;
                        lf_resb=lf_var1b << lf_var2b;
                        host_writeb(eaa,lf_resb);
                        lflags.type=t_SHLb;
                        break; 
                    case 0x05:
                        if (!val) break;
                        lf_var1b=host_readb(eaa);
                        lf_var2b=val;
                        lf_resb=lf_var1b >> lf_var2b;
                        host_writeb(eaa,lf_resb);
                        lflags.type=t_SHRb;
                        break; 
                    case 0x07:
                        if (!val) break;
                        lf_var1b=host_readb(eaa);
                        lf_var2b=val;
                        if (lf_var2b>8) lf_var2b=8;
                        if (lf_var1b & 0x80) {
                            lf_resb=(lf_var1b >> lf_var2b)| (0xff << (8 - lf_var2b));
                        } else {
                            lf_resb=lf_var1b >> lf_var2b;
                        } 
                        host_writeb(eaa,lf_resb);
                        lflags.type=t_SARb;
                        break;
                    }
                } 
            }
            break;
        case (PREFIX_SIZE+0xc1):									/* GRP2 r/m16, imm8 */
            {
                Bit8u rm=Fetchb();
                Bitu which=(rm>>3)&7;
                if (rm >= 0xc0) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    Bit8u val=Fetchb() & 0x1f; 
                    switch (which) {
                    case 0x00:
                        FillFlagsNoCFOF();
                        lf_var1w=*earw;
                        lf_var2b=val&0xf;
                        lf_resw=(lf_var1w << lf_var2b) | (lf_var1w >> (16-lf_var2b));
                        *earw=lf_resw;
                        if (lf_resw & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resw & 1) ^ (lf_resw >> 15)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x01:
                        FillFlagsNoCFOF();
                        lf_var1w=*earw;
                        lf_var2b=val&0xf;
                        lf_resw=(lf_var1w >> lf_var2b) | (lf_var1w << (16-lf_var2b));
                        *earw=lf_resw;
                        if (lf_resw & 0x8000) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resw ^ (lf_resw<<1)) & 0x8000) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x02:
                        if (!(val%17)) break; 
                        {
                            Bit16u cf=(Bit16u)FillFlags()&0x1;
                            lf_var1w=*earw;
                            lf_var2b=val%17;
                            lf_resw=(lf_var1w << lf_var2b) | (cf << (lf_var2b-1)) | (lf_var1w >> (17-lf_var2b));
                            *earw=lf_resw;
                            if (((lf_var1w >> (16-lf_var2b)) & 1)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((reg_flags & 1) ^ (lf_resw >> 15)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x03:
                        if (val%17) {
                            Bit16u cf=(Bit16u)FillFlags()&0x1;
                            lf_var1w=*earw;
                            lf_var2b=val%17;
                            lf_resw=(lf_var1w >> lf_var2b) | (cf << (16-lf_var2b)) | (lf_var1w << (17-lf_var2b));
                            *earw=lf_resw;
                            if ((lf_var1w >> (lf_var2b - 1)) & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((lf_resw ^ (lf_resw<<1)) & 0x8000) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x04:
                    case 0x06:
                        if (!val) break;
                        lf_var1w=*earw;
                        lf_var2b=val ;
                        lf_resw=lf_var1w << lf_var2b;
                        *earw=lf_resw;
                        lflags.type=t_SHLw;
                        break; 
                    case 0x05:
                        if (!val) break;
                        lf_var1w=*earw;
                        lf_var2b=val;
                        lf_resw=lf_var1w >> lf_var2b;
                        *earw=lf_resw;
                        lflags.type=t_SHRw;
                        break; 
                    case 0x07:
                        if (!val) break;
                        lf_var1w=*earw;
                        lf_var2b=val;
                        if (lf_var2b>16) lf_var2b=16;
                        if (lf_var1w & 0x8000) {
                            lf_resw=(lf_var1w >> lf_var2b)| (0xffff << (16 - lf_var2b));
                        } else {
                            lf_resw=lf_var1w >> lf_var2b;
                        } 
                        *earw=lf_resw;
                        lflags.type=t_SARw;
                        break;
                    }
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    Bit8u val=Fetchb() & 0x1f; 
                    switch (which) {
                    case 0x00:
                        FillFlagsNoCFOF();
                        lf_var1w=host_readw(eaa);
                        lf_var2b=val&0xf;
                        lf_resw=(lf_var1w << lf_var2b) | (lf_var1w >> (16-lf_var2b));
                        host_writew(eaa,lf_resw);
                        if (lf_resw & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resw & 1) ^ (lf_resw >> 15)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x01:
                        FillFlagsNoCFOF();
                        lf_var1w=host_readw(eaa);
                        lf_var2b=val&0xf;
                        lf_resw=(lf_var1w >> lf_var2b) | (lf_var1w << (16-lf_var2b));
                        host_writew(eaa,lf_resw);
                        if (lf_resw & 0x8000) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resw ^ (lf_resw<<1)) & 0x8000) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x02:
                        if (!(val%17)) break; {
                            Bit16u cf=(Bit16u)FillFlags()&0x1;
                            lf_var1w=host_readw(eaa);
                            lf_var2b=val%17;
                            lf_resw=(lf_var1w << lf_var2b) | (cf << (lf_var2b-1)) | (lf_var1w >> (17-lf_var2b));
                            host_writew(eaa,lf_resw);
                            if (((lf_var1w >> (16-lf_var2b)) & 1)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((reg_flags & 1) ^ (lf_resw >> 15)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x03:
                        if (val%17) {
                            Bit16u cf=(Bit16u)FillFlags()&0x1;
                            lf_var1w=host_readw(eaa);
                            lf_var2b=val%17;
                            lf_resw=(lf_var1w >> lf_var2b) | (cf << (16-lf_var2b)) | (lf_var1w << (17-lf_var2b));
                            host_writew(eaa,lf_resw);
                            if ((lf_var1w >> (lf_var2b - 1)) & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((lf_resw ^ (lf_resw<<1)) & 0x8000) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x04:
                    case 0x06:
                        if (!val) break;
                        lf_var1w=host_readw(eaa);
                        lf_var2b=val ;
                        lf_resw=lf_var1w << lf_var2b;
                        host_writew(eaa,lf_resw);
                        lflags.type=t_SHLw;
                        break; 
                    case 0x05:
                        if (!val) break;
                        lf_var1w=host_readw(eaa);
                        lf_var2b=val;
                        lf_resw=lf_var1w >> lf_var2b;
                        host_writew(eaa,lf_resw);
                        lflags.type=t_SHRw;
                        break; 
                    case 0x07:
                        if (!val) break;
                        lf_var1w=host_readw(eaa);
                        lf_var2b=val;
                        if (lf_var2b>16) lf_var2b=16;
                        if (lf_var1w & 0x8000) {
                            lf_resw=(lf_var1w >> lf_var2b)| (0xffff << (16 - lf_var2b));
                        } else {
                            lf_resw=lf_var1w >> lf_var2b;
                        } 
                        host_writew(eaa,lf_resw);
                        lflags.type=t_SARw;
                        break;
                    }
                } 
            }
            break;
        case (PREFIX_SIZE+0xc2):									/* RETN imm16 */
            reg_eip=CPU_Pop16();
            reg_esp+=Fetchw();
            continue;
        case (PREFIX_SIZE+0xc3):									/* RETN */
            reg_eip=CPU_Pop16();
            continue;
        case (PREFIX_SIZE+0xc4):									/* LES */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0xc5):									/* LDS */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0xc6):
        case (PREFIX_NONE+0xc6):									/* MOV r/m8, imm8 */
            {
                Bit8u rm=Fetchb();
                if (rm >= 0xc0) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    *earb=Fetchb();
                }
                else {
                    HostPt eaa=core.ea_table[rm]();
                    host_writeb(eaa,Fetchb());
                }
                break;
            }
        case (PREFIX_SIZE+0xc7):									/* MOV EW, imm16 */
            {
                Bit8u rm=Fetchb();
                if (rm >= 0xc0) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    *earw=Fetchw();
                }
                else {
                    HostPt eaa=core.ea_table[rm]();
                    host_writew(eaa,Fetchw());
                }
                break;
            }
        case (PREFIX_SIZE+0xc8):									/* ENTER imm16, imm8 */
            {
                Bitu bytes=Fetchw();
                Bitu level=Fetchb();
                CPU_ENTER(bytes,level);
            }
            break;
        case (PREFIX_SIZE+0xc9):									/* LEAVE */
            reg_esp=reg_ebp;
            reg_bp=CPU_Pop16();
            break;
        case (PREFIX_SIZE+0xca):									/* RETF imm16 */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0xcb):									/* RETF */			
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0xcc):
        case (PREFIX_NONE+0xcc):									/* INT3 */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0xcd):
        case (PREFIX_NONE+0xcd):									/* INT imm8 */	
            {
                Bit8u num=Fetchb();
                continue;
            }
        case (PREFIX_SIZE+0xce):
        case (PREFIX_NONE+0xce):									/* INTO */
            if (get_OF()) {
                continue;
            }
            break;
        case (PREFIX_SIZE+0xcf):									/* IRET */
            {
                goto illegal_opcode; // not implemented 
            }
        case (PREFIX_SIZE+0xd0):
        case (PREFIX_NONE+0xd0):									/* GRP2 r/m8, 1 */
            {
                Bit8u rm=Fetchb();
                Bitu which=(rm>>3)&7;
                if (rm >= 0xc0) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    Bit8u val=1 & 0x1f; 
                    switch (which) {
                    case 0x00:
                        FillFlagsNoCFOF();
                        lf_var1b=*earb;
                        lf_var2b=val&0x07;
                        lf_resb=(lf_var1b << lf_var2b) | (lf_var1b >> (8-lf_var2b));
                        *earb=lf_resb;
                        if (lf_resb & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resb & 1) ^ (lf_resb >> 7)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x01:
                        FillFlagsNoCFOF();
                        lf_var1b=*earb;
                        lf_var2b=val&0x07;
                        lf_resb=(lf_var1b >> lf_var2b) | (lf_var1b << (8-lf_var2b));
                        *earb=lf_resb;
                        if (lf_resb & 0x80) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resb ^ (lf_resb<<1)) & 0x80) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x02:
                        if (!(val%9)) break; {
                            Bit8u cf=(Bit8u)FillFlags()&0x1;
                            lf_var1b=*earb;
                            lf_var2b=val%9;
                            lf_resb=(lf_var1b << lf_var2b) | (cf << (lf_var2b-1)) | (lf_var1b >> (9-lf_var2b));
                            *earb=lf_resb;
                            if (((lf_var1b >> (8-lf_var2b)) & 1)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((reg_flags & 1) ^ (lf_resb >> 7)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x03:
                        if (val%9) {
                            Bit8u cf=(Bit8u)FillFlags()&0x1;
                            lf_var1b=*earb;
                            lf_var2b=val%9;
                            lf_resb=(lf_var1b >> lf_var2b) | (cf << (8-lf_var2b)) | (lf_var1b << (9-lf_var2b));
                            *earb=lf_resb;
                            if ((lf_var1b >> (lf_var2b - 1)) & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((lf_resb ^ (lf_resb<<1)) & 0x80) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x04:
                    case 0x06:
                        if (!val) break;
                        lf_var1b=*earb;
                        lf_var2b=val;
                        lf_resb=lf_var1b << lf_var2b;
                        *earb=lf_resb;
                        lflags.type=t_SHLb;
                        break; 
                    case 0x05:
                        if (!val) break;
                        lf_var1b=*earb;
                        lf_var2b=val;
                        lf_resb=lf_var1b >> lf_var2b;
                        *earb=lf_resb;
                        lflags.type=t_SHRb;
                        break; 
                    case 0x07:
                        if (!val) break;
                        lf_var1b=*earb;
                        lf_var2b=val;
                        if (lf_var2b>8) lf_var2b=8;
                        if (lf_var1b & 0x80) {
                            lf_resb=(lf_var1b >> lf_var2b)| (0xff << (8 - lf_var2b));
                        } else {
                            lf_resb=lf_var1b >> lf_var2b;
                        } 
                        *earb=lf_resb;
                        lflags.type=t_SARb;
                        break;
                    }
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    Bit8u val=1 & 0x1f; 
                    switch (which) {
                    case 0x00:
                        FillFlagsNoCFOF();
                        lf_var1b=host_readb(eaa);
                        lf_var2b=val&0x07;
                        lf_resb=(lf_var1b << lf_var2b) | (lf_var1b >> (8-lf_var2b));
                        host_writeb(eaa,lf_resb);
                        if (lf_resb & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resb & 1) ^ (lf_resb >> 7)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x01:
                        FillFlagsNoCFOF();
                        lf_var1b=host_readb(eaa);
                        lf_var2b=val&0x07;
                        lf_resb=(lf_var1b >> lf_var2b) | (lf_var1b << (8-lf_var2b));
                        host_writeb(eaa,lf_resb);
                        if (lf_resb & 0x80) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resb ^ (lf_resb<<1)) & 0x80) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x02:
                        if (!(val%9)) break; 
                        {
                            Bit8u cf=(Bit8u)FillFlags()&0x1;
                            lf_var1b=host_readb(eaa);
                            lf_var2b=val%9;
                            lf_resb=(lf_var1b << lf_var2b) | (cf << (lf_var2b-1)) | (lf_var1b >> (9-lf_var2b));
                            host_writeb(eaa,lf_resb);
                            if (((lf_var1b >> (8-lf_var2b)) & 1)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((reg_flags & 1) ^ (lf_resb >> 7)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x03:
                        if (val%9) {
                            Bit8u cf=(Bit8u)FillFlags()&0x1;
                            lf_var1b=host_readb(eaa);
                            lf_var2b=val%9;
                            lf_resb=(lf_var1b >> lf_var2b) | (cf << (8-lf_var2b)) | (lf_var1b << (9-lf_var2b));
                            host_writeb(eaa,lf_resb);
                            if ((lf_var1b >> (lf_var2b - 1)) & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((lf_resb ^ (lf_resb<<1)) & 0x80) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x04:
                    case 0x06:
                        if (!val) break;
                        lf_var1b=host_readb(eaa);
                        lf_var2b=val;
                        lf_resb=lf_var1b << lf_var2b;
                        host_writeb(eaa,lf_resb);
                        lflags.type=t_SHLb;
                        break; 
                    case 0x05:
                        if (!val) break;
                        lf_var1b=host_readb(eaa);
                        lf_var2b=val;
                        lf_resb=lf_var1b >> lf_var2b;
                        host_writeb(eaa,lf_resb);
                        lflags.type=t_SHRb;
                        break; 
                    case 0x07:
                        if (!val) break;
                        lf_var1b=host_readb(eaa);
                        lf_var2b=val;
                        if (lf_var2b>8) lf_var2b=8;
                        if (lf_var1b & 0x80) {
                            lf_resb=(lf_var1b >> lf_var2b)| (0xff << (8 - lf_var2b));
                        } else {
                            lf_resb=lf_var1b >> lf_var2b;
                        } 
                        host_writeb(eaa,lf_resb);
                        lflags.type=t_SARb;
                        break;
                    }
                } 
            }
            break;
        case (PREFIX_SIZE+0xd1):									/* GRP2 r/m16, 1 */
            {
                Bit8u rm=Fetchb();
                Bitu which=(rm>>3)&7;
                if (rm >= 0xc0) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    Bit8u val=1 & 0x1f; 
                    switch (which) {
                    case 0x00:
                        FillFlagsNoCFOF();
                        lf_var1w=*earw;
                        lf_var2b=val&0xf;
                        lf_resw=(lf_var1w << lf_var2b) | (lf_var1w >> (16-lf_var2b));
                        *earw=lf_resw;
                        if (lf_resw & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resw & 1) ^ (lf_resw >> 15)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x01:
                        FillFlagsNoCFOF();
                        lf_var1w=*earw;
                        lf_var2b=val&0xf;
                        lf_resw=(lf_var1w >> lf_var2b) | (lf_var1w << (16-lf_var2b));
                        *earw=lf_resw;
                        if (lf_resw & 0x8000) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resw ^ (lf_resw<<1)) & 0x8000) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x02:
                        if (!(val%17)) break; 
                        {
                            Bit16u cf=(Bit16u)FillFlags()&0x1;
                            lf_var1w=*earw;
                            lf_var2b=val%17;
                            lf_resw=(lf_var1w << lf_var2b) | (cf << (lf_var2b-1)) | (lf_var1w >> (17-lf_var2b));
                            *earw=lf_resw;
                            if (((lf_var1w >> (16-lf_var2b)) & 1)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((reg_flags & 1) ^ (lf_resw >> 15)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x03:
                        if (val%17) {
                            Bit16u cf=(Bit16u)FillFlags()&0x1;
                            lf_var1w=*earw;
                            lf_var2b=val%17;
                            lf_resw=(lf_var1w >> lf_var2b) | (cf << (16-lf_var2b)) | (lf_var1w << (17-lf_var2b));
                            *earw=lf_resw;
                            if ((lf_var1w >> (lf_var2b - 1)) & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((lf_resw ^ (lf_resw<<1)) & 0x8000) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x04:
                    case 0x06:
                        if (!val) break;
                        lf_var1w=*earw;
                        lf_var2b=val ;
                        lf_resw=lf_var1w << lf_var2b;
                        *earw=lf_resw;
                        lflags.type=t_SHLw;
                        break; 
                    case 0x05:
                        if (!val) break;
                        lf_var1w=*earw;
                        lf_var2b=val;
                        lf_resw=lf_var1w >> lf_var2b;
                        *earw=lf_resw;
                        lflags.type=t_SHRw;
                        break; 
                    case 0x07:
                        if (!val) break;
                        lf_var1w=*earw;
                        lf_var2b=val;
                        if (lf_var2b>16) lf_var2b=16;
                        if (lf_var1w & 0x8000) {
                            lf_resw=(lf_var1w >> lf_var2b)| (0xffff << (16 - lf_var2b));
                        } else {
                            lf_resw=lf_var1w >> lf_var2b;
                        } 
                        *earw=lf_resw;
                        lflags.type=t_SARw;
                        break;
                    }
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    Bit8u val=1 & 0x1f; 
                    switch (which) {
                    case 0x00:
                        FillFlagsNoCFOF();
                        lf_var1w=host_readw(eaa);
                        lf_var2b=val&0xf;
                        lf_resw=(lf_var1w << lf_var2b) | (lf_var1w >> (16-lf_var2b));
                        host_writew(eaa,lf_resw);
                        if (lf_resw & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resw & 1) ^ (lf_resw >> 15)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x01:
                        FillFlagsNoCFOF();
                        lf_var1w=host_readw(eaa);
                        lf_var2b=val&0xf;
                        lf_resw=(lf_var1w >> lf_var2b) | (lf_var1w << (16-lf_var2b));
                        host_writew(eaa,lf_resw);
                        if (lf_resw & 0x8000) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resw ^ (lf_resw<<1)) & 0x8000) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x02:
                        if (!(val%17)) break; 
                        {
                            Bit16u cf=(Bit16u)FillFlags()&0x1;
                            lf_var1w=host_readw(eaa);
                            lf_var2b=val%17;
                            lf_resw=(lf_var1w << lf_var2b) | (cf << (lf_var2b-1)) | (lf_var1w >> (17-lf_var2b));
                            host_writew(eaa,lf_resw);
                            if (((lf_var1w >> (16-lf_var2b)) & 1)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((reg_flags & 1) ^ (lf_resw >> 15)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x03:
                        if (val%17) {
                            Bit16u cf=(Bit16u)FillFlags()&0x1;
                            lf_var1w=host_readw(eaa);
                            lf_var2b=val%17;
                            lf_resw=(lf_var1w >> lf_var2b) | (cf << (16-lf_var2b)) | (lf_var1w << (17-lf_var2b));
                            host_writew(eaa,lf_resw);
                            if ((lf_var1w >> (lf_var2b - 1)) & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((lf_resw ^ (lf_resw<<1)) & 0x8000) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x04:
                    case 0x06:
                        if (!val) break;
                        lf_var1w=host_readw(eaa);
                        lf_var2b=val ;
                        lf_resw=lf_var1w << lf_var2b;
                        host_writew(eaa,lf_resw);
                        lflags.type=t_SHLw;
                        break; 
                    case 0x05:
                        if (!val) break;
                        lf_var1w=host_readw(eaa);
                        lf_var2b=val;
                        lf_resw=lf_var1w >> lf_var2b;
                        host_writew(eaa,lf_resw);
                        lflags.type=t_SHRw;
                        break; 
                    case 0x07:
                        if (!val) break;
                        lf_var1w=host_readw(eaa);
                        lf_var2b=val;
                        if (lf_var2b>16) lf_var2b=16;
                        if (lf_var1w & 0x8000) {
                            lf_resw=(lf_var1w >> lf_var2b)| (0xffff << (16 - lf_var2b));
                        } else {
                            lf_resw=lf_var1w >> lf_var2b;
                        } 
                        host_writew(eaa,lf_resw);
                        lflags.type=t_SARw;
                        break;
                    }
                } 
            }
            break;
        case (PREFIX_SIZE+0xd2):
        case (PREFIX_NONE+0xd2):									/* GRP2 r/m8, CL */
            {
                Bit8u rm=Fetchb();
                Bitu which=(rm>>3)&7;
                if (rm >= 0xc0) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    Bit8u val=reg_cl & 0x1f; 
                    switch (which) {
                    case 0x00:
                        FillFlagsNoCFOF();
                        lf_var1b=*earb;
                        lf_var2b=val&0x07;
                        lf_resb=(lf_var1b << lf_var2b) | (lf_var1b >> (8-lf_var2b));
                        *earb=lf_resb;
                        if (lf_resb & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resb & 1) ^ (lf_resb >> 7)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x01:
                        FillFlagsNoCFOF();
                        lf_var1b=*earb;
                        lf_var2b=val&0x07;
                        lf_resb=(lf_var1b >> lf_var2b) | (lf_var1b << (8-lf_var2b));
                        *earb=lf_resb;
                        if (lf_resb & 0x80) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resb ^ (lf_resb<<1)) & 0x80) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x02:
                        if (!(val%9)) break; 
                        {
                            Bit8u cf=(Bit8u)FillFlags()&0x1;
                            lf_var1b=*earb;
                            lf_var2b=val%9;
                            lf_resb=(lf_var1b << lf_var2b) | (cf << (lf_var2b-1)) | (lf_var1b >> (9-lf_var2b));
                            *earb=lf_resb;
                            if (((lf_var1b >> (8-lf_var2b)) & 1)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((reg_flags & 1) ^ (lf_resb >> 7)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x03:
                        if (val%9) {
                            Bit8u cf=(Bit8u)FillFlags()&0x1;
                            lf_var1b=*earb;
                            lf_var2b=val%9;
                            lf_resb=(lf_var1b >> lf_var2b) | (cf << (8-lf_var2b)) | (lf_var1b << (9-lf_var2b));
                            *earb=lf_resb;
                            if ((lf_var1b >> (lf_var2b - 1)) & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((lf_resb ^ (lf_resb<<1)) & 0x80) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x04:
                    case 0x06:
                        if (!val) break;
                        lf_var1b=*earb;
                        lf_var2b=val;
                        lf_resb=lf_var1b << lf_var2b;
                        *earb=lf_resb;
                        lflags.type=t_SHLb;
                        break; 
                    case 0x05:
                        if (!val) break;
                        lf_var1b=*earb;
                        lf_var2b=val;
                        lf_resb=lf_var1b >> lf_var2b;
                        *earb=lf_resb;
                        lflags.type=t_SHRb;
                        break; 
                    case 0x07:
                        if (!val) break;
                        lf_var1b=*earb;
                        lf_var2b=val;
                        if (lf_var2b>8) lf_var2b=8;
                        if (lf_var1b & 0x80) {
                            lf_resb=(lf_var1b >> lf_var2b)| (0xff << (8 - lf_var2b));
                        } else {
                            lf_resb=lf_var1b >> lf_var2b;
                        } 
                        *earb=lf_resb;
                        lflags.type=t_SARb;
                        break;
                    }
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    Bit8u val=reg_cl & 0x1f; 
                    switch (which) {
                    case 0x00:
                        FillFlagsNoCFOF();
                        lf_var1b=host_readb(eaa);
                        lf_var2b=val&0x07;
                        lf_resb=(lf_var1b << lf_var2b) | (lf_var1b >> (8-lf_var2b));
                        host_writeb(eaa,lf_resb);
                        if (lf_resb & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resb & 1) ^ (lf_resb >> 7)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x01:
                        FillFlagsNoCFOF();
                        lf_var1b=host_readb(eaa);
                        lf_var2b=val&0x07;
                        lf_resb=(lf_var1b >> lf_var2b) | (lf_var1b << (8-lf_var2b));
                        host_writeb(eaa,lf_resb);
                        if (lf_resb & 0x80) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resb ^ (lf_resb<<1)) & 0x80) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x02:
                        if (!(val%9)) break; 
                        {
                            Bit8u cf=(Bit8u)FillFlags()&0x1;
                            lf_var1b=host_readb(eaa);
                            lf_var2b=val%9;
                            lf_resb=(lf_var1b << lf_var2b) | (cf << (lf_var2b-1)) | (lf_var1b >> (9-lf_var2b));
                            host_writeb(eaa,lf_resb);
                            if (((lf_var1b >> (8-lf_var2b)) & 1)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((reg_flags & 1) ^ (lf_resb >> 7)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x03:
                        if (val%9) {
                            Bit8u cf=(Bit8u)FillFlags()&0x1;
                            lf_var1b=host_readb(eaa);
                            lf_var2b=val%9;
                            lf_resb=(lf_var1b >> lf_var2b) | (cf << (8-lf_var2b)) | (lf_var1b << (9-lf_var2b));
                            host_writeb(eaa,lf_resb);
                            if ((lf_var1b >> (lf_var2b - 1)) & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((lf_resb ^ (lf_resb<<1)) & 0x80) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x04:
                    case 0x06:
                        if (!val) break;
                        lf_var1b=host_readb(eaa);
                        lf_var2b=val;
                        lf_resb=lf_var1b << lf_var2b;
                        host_writeb(eaa,lf_resb);
                        lflags.type=t_SHLb;
                        break; 
                    case 0x05:
                        if (!val) break;
                        lf_var1b=host_readb(eaa);
                        lf_var2b=val;
                        lf_resb=lf_var1b >> lf_var2b;
                        host_writeb(eaa,lf_resb);
                        lflags.type=t_SHRb;
                        break; 
                    case 0x07:
                        if (!val) break;
                        lf_var1b=host_readb(eaa);
                        lf_var2b=val;
                        if (lf_var2b>8) lf_var2b=8;
                        if (lf_var1b & 0x80) {
                            lf_resb=(lf_var1b >> lf_var2b)| (0xff << (8 - lf_var2b));
                        } else {
                            lf_resb=lf_var1b >> lf_var2b;
                        } 
                        host_writeb(eaa,lf_resb);
                        lflags.type=t_SARb;
                        break;
                    }
                } 
            }
            break;
        case (PREFIX_SIZE+0xd3):									/* GRP2 r/m16, CL */
            {
                Bit8u rm=Fetchb();
                Bitu which=(rm>>3)&7;
                if (rm >= 0xc0) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    Bit8u val=reg_cl & 0x1f; 
                    switch (which) {
                    case 0x00:
                        FillFlagsNoCFOF();
                        lf_var1w=*earw;
                        lf_var2b=val&0xf;
                        lf_resw=(lf_var1w << lf_var2b) | (lf_var1w >> (16-lf_var2b));
                        *earw=lf_resw;
                        if (lf_resw & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resw & 1) ^ (lf_resw >> 15)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x01:
                        FillFlagsNoCFOF();
                        lf_var1w=*earw;
                        lf_var2b=val&0xf;
                        lf_resw=(lf_var1w >> lf_var2b) | (lf_var1w << (16-lf_var2b));
                        *earw=lf_resw;
                        if (lf_resw & 0x8000) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resw ^ (lf_resw<<1)) & 0x8000) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x02:
                        if (!(val%17)) break; 
                        {
                            Bit16u cf=(Bit16u)FillFlags()&0x1;
                            lf_var1w=*earw;
                            lf_var2b=val%17;
                            lf_resw=(lf_var1w << lf_var2b) | (cf << (lf_var2b-1)) | (lf_var1w >> (17-lf_var2b));
                            *earw=lf_resw;
                            if (((lf_var1w >> (16-lf_var2b)) & 1)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((reg_flags & 1) ^ (lf_resw >> 15)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x03:
                        if (val%17) {
                            Bit16u cf=(Bit16u)FillFlags()&0x1;
                            lf_var1w=*earw;
                            lf_var2b=val%17;
                            lf_resw=(lf_var1w >> lf_var2b) | (cf << (16-lf_var2b)) | (lf_var1w << (17-lf_var2b));
                            *earw=lf_resw;
                            if ((lf_var1w >> (lf_var2b - 1)) & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((lf_resw ^ (lf_resw<<1)) & 0x8000) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x04:
                    case 0x06:
                        if (!val) break;
                        lf_var1w=*earw;
                        lf_var2b=val ;
                        lf_resw=lf_var1w << lf_var2b;
                        *earw=lf_resw;
                        lflags.type=t_SHLw;
                        break; 
                    case 0x05:
                        if (!val) break;
                        lf_var1w=*earw;
                        lf_var2b=val;
                        lf_resw=lf_var1w >> lf_var2b;
                        *earw=lf_resw;
                        lflags.type=t_SHRw;
                        break; 
                    case 0x07:
                        if (!val) break;
                        lf_var1w=*earw;
                        lf_var2b=val;
                        if (lf_var2b>16) lf_var2b=16;
                        if (lf_var1w & 0x8000) {
                            lf_resw=(lf_var1w >> lf_var2b)| (0xffff << (16 - lf_var2b));
                        } else {
                            lf_resw=lf_var1w >> lf_var2b;
                        } 
                        *earw=lf_resw;
                        lflags.type=t_SARw;
                        break;
                    }
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    Bit8u val=reg_cl & 0x1f; 
                    switch (which) {
                    case 0x00:
                        FillFlagsNoCFOF();
                        lf_var1w=host_readw(eaa);
                        lf_var2b=val&0xf;
                        lf_resw=(lf_var1w << lf_var2b) | (lf_var1w >> (16-lf_var2b));
                        host_writew(eaa,lf_resw);
                        if (lf_resw & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resw & 1) ^ (lf_resw >> 15)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x01:
                        FillFlagsNoCFOF();
                        lf_var1w=host_readw(eaa);
                        lf_var2b=val&0xf;
                        lf_resw=(lf_var1w >> lf_var2b) | (lf_var1w << (16-lf_var2b));
                        host_writew(eaa,lf_resw);
                        if (lf_resw & 0x8000) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                        if ((lf_resw ^ (lf_resw<<1)) & 0x8000) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        break; 
                    case 0x02:
                        if (!(val%17)) break; 
                        {
                            Bit16u cf=(Bit16u)FillFlags()&0x1;
                            lf_var1w=host_readw(eaa);
                            lf_var2b=val%17;
                            lf_resw=(lf_var1w << lf_var2b) | (cf << (lf_var2b-1)) | (lf_var1w >> (17-lf_var2b));
                            host_writew(eaa,lf_resw);
                            if (((lf_var1w >> (16-lf_var2b)) & 1)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((reg_flags & 1) ^ (lf_resw >> 15)) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x03:
                        if (val%17) {
                            Bit16u cf=(Bit16u)FillFlags()&0x1;
                            lf_var1w=host_readw(eaa);
                            lf_var2b=val%17;
                            lf_resw=(lf_var1w >> lf_var2b) | (cf << (16-lf_var2b)) | (lf_var1w << (17-lf_var2b));
                            host_writew(eaa,lf_resw);
                            if ((lf_var1w >> (lf_var2b - 1)) & 1) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            if ((lf_resw ^ (lf_resw<<1)) & 0x8000) reg_flags|=FLAG_OF; else reg_flags&=~FLAG_OF;
                        }
                        break; 
                    case 0x04:
                    case 0x06:
                        if (!val) break;
                        lf_var1w=host_readw(eaa);
                        lf_var2b=val ;
                        lf_resw=lf_var1w << lf_var2b;
                        host_writew(eaa,lf_resw);
                        lflags.type=t_SHLw;
                        break; 
                    case 0x05:
                        if (!val) break;
                        lf_var1w=host_readw(eaa);
                        lf_var2b=val;
                        lf_resw=lf_var1w >> lf_var2b;
                        host_writew(eaa,lf_resw);
                        lflags.type=t_SHRw;
                        break; 
                    case 0x07:
                        if (!val) break;
                        lf_var1w=host_readw(eaa);
                        lf_var2b=val;
                        if (lf_var2b>16) lf_var2b=16;
                        if (lf_var1w & 0x8000) {
                            lf_resw=(lf_var1w >> lf_var2b)| (0xffff << (16 - lf_var2b));
                        } else {
                            lf_resw=lf_var1w >> lf_var2b;
                        } 
                        host_writew(eaa,lf_resw);
                        lflags.type=t_SARw;
                        break;
                    }
                } 
            }
            break;
        case (PREFIX_SIZE+0xd4):
        case (PREFIX_NONE+0xd4):									/* AAM imm8 */
            {
                Bit8u dv=Fetchb();
                if (dv!=0) {
                    reg_ah=reg_al / dv;
                    reg_al=reg_al % dv;
                    if ((reg_al & 0x80)) reg_flags|=FLAG_SF; else reg_flags&=~FLAG_SF;
                    if ((reg_al == 0)) reg_flags|=FLAG_ZF; else reg_flags&=~FLAG_ZF;
                    if (parity_lookup[reg_al]) reg_flags|=FLAG_PF; else reg_flags&=~FLAG_PF;
                    reg_flags&=~FLAG_CF;
                    reg_flags&=~FLAG_OF;
                    reg_flags&=~FLAG_AF;
                    lflags.type=t_UNKNOWN;
                }
            }
            break;
        case (PREFIX_SIZE+0xd5):
        case (PREFIX_NONE+0xd5):									/* AAD imm8 */
            {
                Bit16u ax1 = reg_ah * Fetchb();
                Bit16u ax2 = ax1 + reg_al;
                reg_al = (Bit8u) ax2;
                reg_ah = 0;
                reg_flags&=~FLAG_CF;
                reg_flags&=~FLAG_OF;
                reg_flags&=~FLAG_AF;
                if (reg_al >= 0x80) reg_flags|=FLAG_SF; else reg_flags&=~FLAG_SF;
                if (reg_al == 0) reg_flags|=FLAG_ZF; else reg_flags&=~FLAG_ZF;
                if (parity_lookup[reg_al]) reg_flags|=FLAG_PF; else reg_flags&=~FLAG_PF;
                lflags.type=t_UNKNOWN;
            }
            break;
        case (PREFIX_SIZE+0xd6):
        case (PREFIX_NONE+0xd6):									/* SALC */
            reg_al = get_CF() ? 0xFF : 0;
            break;
        case (PREFIX_SIZE+0xd7):
        case (PREFIX_NONE+0xd7):									/* XLAT */
            if (core.addr32 & PREFIX_ADDR) {
                reg_al=host_readb(core.base_ds+(Bit32u)(reg_ebx+reg_al));
            } else {
                reg_al=host_readb(core.base_ds+(Bit16u)(reg_bx+reg_al));
            }
            break;
        case (PREFIX_SIZE+0xd8):
        case (PREFIX_NONE+0xd8):									/* FPU ESC 0 */
        case (PREFIX_SIZE+0xd9):
        case (PREFIX_NONE+0xd9):									/* FPU ESC 1 */
        case (PREFIX_SIZE+0xda):
        case (PREFIX_NONE+0xda):									/* FPU ESC 2 */
        case (PREFIX_SIZE+0xdb):
        case (PREFIX_NONE+0xdb):									/* FPU ESC 3 */
        case (PREFIX_SIZE+0xdc):
        case (PREFIX_NONE+0xdc):									/* FPU ESC 4 */
        case (PREFIX_SIZE+0xdd):
        case (PREFIX_NONE+0xdd):									/* FPU ESC 5 */
        case (PREFIX_SIZE+0xde):
        case (PREFIX_NONE+0xde):									/* FPU ESC 6 */
        case (PREFIX_SIZE+0xdf):
        case (PREFIX_NONE+0xdf):									/* FPU ESC 7 */
            {
                Bit8u rm=Fetchb();
                if (rm<0xc0) HostPt eaa=core.ea_table[rm]();
            }
            break;
        case (PREFIX_SIZE+0xe0):									/* LOOPNZ */
            if (core.addr32 & PREFIX_ADDR) {
                {
                    reg_eip=(core.cseip-MemBase);
                    if (--reg_ecx && !get_ZF()) reg_ip+=Fetchb();
                    reg_ip+=1; continue;
                }
            } else {
                {
                    reg_eip=(core.cseip-MemBase);
                    if (--reg_cx && !get_ZF()) reg_ip+=Fetchb();
                    reg_ip+=1; continue;
                }
            }
            break;
        case (PREFIX_SIZE+0xe1):									/* LOOPZ */
            if (core.addr32 & PREFIX_ADDR) {
                {
                    reg_eip=(core.cseip-MemBase);
                    if (--reg_ecx && get_ZF()) reg_ip+=Fetchb();
                    reg_ip+=1; continue;
                }
            } else {
                {
                    reg_eip=(core.cseip-MemBase);
                    if (--reg_cx && get_ZF()) reg_ip+=Fetchb();
                    reg_ip+=1; continue;
                }
            }
            break;
        case (PREFIX_SIZE+0xe2):									/* LOOP */
            if (core.addr32 & PREFIX_ADDR) {	
                {
                    reg_eip=(core.cseip-MemBase);
                    if (--reg_ecx) reg_ip+=Fetchb();
                    reg_ip+=1; continue;
                }
            } else {
                {
                    reg_eip=(core.cseip-MemBase);
                    if (--reg_cx) reg_ip+=Fetchb();
                    reg_ip+=1; continue;
                }
            }
            break;
        case (PREFIX_SIZE+0xe3):									/* JCXZ */
            {
                reg_eip=(core.cseip-MemBase);
                if (!(reg_ecx & AddrMaskTable[core.addr32& PREFIX_ADDR])) reg_ip+=Fetchb();
                reg_ip+=1; continue;
            }
            break;
        case (PREFIX_SIZE+0xe4):
        case (PREFIX_NONE+0xe4):									/* IN AL, imm8 */
            {	
                Bitu port=Fetchb();
                reg_al=IO_ReadB(port);
                break;
            }
        case (PREFIX_SIZE+0xe5):									/* IN AX, imm8 */
            {	
                Bitu port=Fetchb();
                reg_al=IO_ReadW(port);
                break;
            }
        case (PREFIX_SIZE+0xe6):
        case (PREFIX_NONE+0xe6):									/* OUT imm8,AL */
            {
                Bitu port=Fetchb();
                IO_WriteB(port,reg_al);
                break;
            }		
        case (PREFIX_SIZE+0xe7):									/* OUT imm8, AX */
            {
                Bitu port=Fetchb();
                IO_WriteW(port,reg_ax);
                break;
            }
        case (PREFIX_SIZE+0xe8):									/* CALL rel16 */
            { 
                Bit16u addip=Fetchw();
                reg_eip=(core.cseip-MemBase);
                CPU_Push16(reg_eip);
                reg_eip=(Bit16u)(reg_eip+addip);
                continue;
            }
        case (PREFIX_SIZE+0xe9):									/* JMP rel16 */
            { 
                Bit16u addip=Fetchw();
                reg_eip=(core.cseip-MemBase);
                reg_eip=(Bit16u)(reg_eip+addip);
                continue;
            }
        case (PREFIX_SIZE+0xea):									/* JMP ptr16:32 */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0xeb):									/* JMP rel8 */
            { 
                Bit16s addip=Fetchb();
                reg_eip=(core.cseip-MemBase);
                reg_eip=(Bit16u)(reg_eip+addip);
                continue;
            }
        case (PREFIX_SIZE+0xec):
        case (PREFIX_NONE+0xec):									/* IN AL,DX */
            reg_al=IO_ReadB(reg_dx);
            break;
        case (PREFIX_SIZE+0xed):									/* IN AX,DX */
            reg_ax=IO_ReadW(reg_dx);
            break;
        case (PREFIX_SIZE+0xee):
        case (PREFIX_NONE+0xee):									/* OUT DX,AL */
            IO_WriteB(reg_dx,reg_al);
            break;
        case (PREFIX_SIZE+0xef):									/* OUT DX,AX */
            IO_WriteW(reg_dx,reg_ax);
            break;
        case (PREFIX_SIZE+0xf0):
        case (PREFIX_NONE+0xf0):									/* LOCK */
            break;
        case (PREFIX_SIZE+0xf1):
        case (PREFIX_NONE+0xf1):									/* ICEBP */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0xf2):
        case (PREFIX_NONE+0xf2):									/* REPNZ */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0xf3):
        case (PREFIX_NONE+0xf3):									/* REPZ */
            goto illegal_opcode; // not implemented 
        case (PREFIX_SIZE+0xf4):
        case (PREFIX_NONE+0xf4):									/* HLT */
            running = false;
            break;		
        case (PREFIX_SIZE+0xf5):
        case (PREFIX_NONE+0xf5):									/* CMC */
            FillFlags();
            if (!(reg_flags & FLAG_CF)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
            break;
        case (PREFIX_SIZE+0xf6):
        case (PREFIX_NONE+0xf6):									/* GRP3 r/m8(, imm8) */
            {	
                Bit8u rm=Fetchb();
                Bitu which=(rm>>3)&7;
                switch (which) {
                case 0x00:								/* TEST r/m8, imm8 */
                case 0x01:								/* TEST r/m8, imm8 Undocumented*/
                    {
                        if (rm >= 0xc0 ) {
                            Bit8u * earb=lookupRMEAregb[rm];
                            lf_var1b=*earb;
                            lf_var2b=Fetchb();
                            lf_resb=lf_var1b & lf_var2b;
                            lflags.type=t_TESTb;
                        }
                        else {
                            HostPt eaa=core.ea_table[rm]();
                            lf_var1b=host_readb(eaa);
                            lf_var2b=Fetchb();
                            lf_resb=lf_var1b & lf_var2b;
                            lflags.type=t_TESTb;
                        }
                        break;
                    }
                case 0x02:								/* NOT r/m8 */
                    {
                        if (rm >= 0xc0 ) {
                            Bit8u * earb=lookupRMEAregb[rm];
                            *earb=~*earb;
                        }
                        else {
                            HostPt eaa=core.ea_table[rm]();
                            host_writeb(eaa,~host_readb(eaa));
                        }
                        break;
                    }
                case 0x03:								/* NEG r/m8 */
                    {
                        lflags.type=t_NEGb;
                        if (rm >= 0xc0 ) {
                            Bit8u * earb=lookupRMEAregb[rm];
                            lf_var1b=*earb;
                            lf_resb=0-lf_var1b;
                            *earb=lf_resb;
                        } else {
                            HostPt eaa=core.ea_table[rm]();
                            lf_var1b=host_readb(eaa);
                            lf_resb=0-lf_var1b;
                            host_writeb(eaa,lf_resb);
                        }
                        break;
                    }
                case 0x04:								/* MUL AL, r/m8 */
                    {
                        if (rm >= 0xc0 ) {
                            Bit8u * earb=lookupRMEAregb[rm];
                            reg_ax=reg_al**earb; FillFlagsNoCFOF();
                            if (reg_al == 0) reg_flags|=FLAG_ZF; else reg_flags&=~FLAG_ZF;
                            if (reg_ax & 0xff00) {
                                reg_flags|=FLAG_CF;reg_flags|=FLAG_OF;
                            } else {
                                reg_flags&=~FLAG_CF;reg_flags&=~FLAG_OF;
                            }
                        } else {
                            HostPt eaa=core.ea_table[rm]();
                            reg_ax=reg_al*host_readb(eaa); FillFlagsNoCFOF();
                            if (reg_al == 0) reg_flags|=FLAG_ZF; else reg_flags&=~FLAG_ZF;
                            if (reg_ax & 0xff00) {
                                reg_flags|=FLAG_CF;reg_flags|=FLAG_OF;
                            } else {
                                reg_flags&=~FLAG_CF;reg_flags&=~FLAG_OF;
                            }
                        }
                    }
                    break;
                case 0x05:								/* IMUL AL, r/m8 */
                    {
                        if (rm >= 0xc0 ) {
                            Bit8u * earb=lookupRMEAregb[rm];{
                                reg_ax=((Bit8s)reg_al) * ((Bit8s)(*earb)); FillFlagsNoCFOF();
                                if ((reg_ax & 0xff80)==0xff80 || (reg_ax & 0xff80)==0x0000) {
                                    reg_flags&=~FLAG_CF;reg_flags&=~FLAG_OF;
                                } else {
                                    reg_flags|=FLAG_CF;reg_flags|=FLAG_OF;
                                }
                            }
                        } else {
                            HostPt eaa=core.ea_table[rm]();{
                                reg_ax=((Bit8s)reg_al) * ((Bit8s)(host_readb(eaa))); FillFlagsNoCFOF();
                                if ((reg_ax & 0xff80)==0xff80 || (reg_ax & 0xff80)==0x0000) {
                                    reg_flags&=~FLAG_CF;reg_flags&=~FLAG_OF;
                                } else {
                                    reg_flags|=FLAG_CF;reg_flags|=FLAG_OF;
                                }
                            }
                        }
                    }
                    break;
                case 0x06:								/* DIV r/m8 */
                    {
                        if (rm >= 0xc0 ) {
                            Bit8u * earb=lookupRMEAregb[rm];{
                                Bitu val=*earb;
                                if (val==0) val=1;
                                Bitu quo=reg_ax / val;
                                Bit8u rem=(Bit8u)(reg_ax % val);
                                Bit8u quo8=(Bit8u)(quo&0xff);
                                reg_ah=rem;
                                reg_al=quo8;
                            }
                        } else {
                            HostPt eaa=core.ea_table[rm]();{
                                Bitu val=host_readb(eaa);
                                if (val==0) val=1;
                                Bitu quo=reg_ax / val;
                                Bit8u rem=(Bit8u)(reg_ax % val);
                                Bit8u quo8=(Bit8u)(quo&0xff);
                                reg_ah=rem;
                                reg_al=quo8;
                            }
                        }
                    }
                    break;
                case 0x07:								/* IDIV r/m8 */
                    {
                        if (rm >= 0xc0 ) {
                            Bit8u * earb=lookupRMEAregb[rm];{
                                Bits val=(Bit8s)(*earb);
                                if (val==0) val=1 ;
                                Bits quo=((Bit16s)reg_ax) / val;
                                Bit8s rem=(Bit8s)((Bit16s)reg_ax % val);
                                Bit8s quo8s=(Bit8s)(quo&0xff);
                                reg_ah=rem;
                                reg_al=quo8s;
                            }
                        } else {
                            HostPt eaa=core.ea_table[rm]();{
                                Bits val=(Bit8s)(host_readb(eaa));
                                if (val==0) val=1 ;
                                Bits quo=((Bit16s)reg_ax) / val;
                                Bit8s rem=(Bit8s)((Bit16s)reg_ax % val);
                                Bit8s quo8s=(Bit8s)(quo&0xff);
                                reg_ah=rem;
                                reg_al=quo8s;
                            }
                        }
                    }
                    break;
                }
                break;
            }
        case (PREFIX_SIZE+0xf7):									/* GRP3 r/m16(, imm16) */
            { 
                Bit8u rm=Fetchb();
                Bitu which=(rm>>3)&7;
                switch (which) {
                case 0x00:								/* TEST r/m16, imm16 */
                case 0x01:								/* TEST r/m16, imm16 Undocumented*/
                    {
                        if (rm >= 0xc0 ) {
                            Bit16u * earw=lookupRMEAregw[rm];
                            lf_var1w=*earw;
                            lf_var2w=Fetchw();
                            lf_resw=lf_var1w & lf_var2w;
                            lflags.type=t_TESTw;
                        }
                        else {
                            HostPt eaa=core.ea_table[rm]();
                            lf_var1w=host_readw(eaa);
                            lf_var2w=Fetchw();
                            lf_resw=lf_var1w & lf_var2w;
                            lflags.type=t_TESTw;
                        }
                        break;
                    }
                case 0x02:								/* NOT r/m16 */
                    {
                        if (rm >= 0xc0 ) {
                            Bit16u * earw=lookupRMEAregw[rm];
                            *earw=~*earw;
                        }
                        else {
                            HostPt eaa=core.ea_table[rm]();
                            host_writew(eaa,~host_readw(eaa));
                        }
                        break;
                    }
                case 0x03:								/* NEG r/m16 */
                    {
                        lflags.type=t_NEGw;
                        if (rm >= 0xc0 ) {
                            Bit16u * earw=lookupRMEAregw[rm];
                            lf_var1w=*earw;
                            lf_resw=0-lf_var1w;
                            *earw=lf_resw;
                        } else {
                            HostPt eaa=core.ea_table[rm]();
                            lf_var1w=host_readw(eaa);
                            lf_resw=0-lf_var1w;
                            host_writew(eaa,lf_resw);
                        }
                        break;
                    }
                case 0x04:								/* MUL AX, r/m16 */
                    {
                        if (rm >= 0xc0 ) {
                            Bit16u * earw=lookupRMEAregw[rm];{
                                Bitu tempu=(Bitu)reg_ax*(Bitu)(*earw);
                                reg_ax=(Bit16u)(tempu);
                                reg_dx=(Bit16u)(tempu >> 16); FillFlagsNoCFOF();
                                if (reg_ax == 0) reg_flags|=FLAG_ZF; else reg_flags&=~FLAG_ZF;
                                if (reg_dx) {
                                    reg_flags|=FLAG_CF;reg_flags|=FLAG_OF;
                                } else {
                                    reg_flags&=~FLAG_CF;reg_flags&=~FLAG_OF;
                                }
                            }
                        } else {
                            HostPt eaa=core.ea_table[rm]();{
                                Bitu tempu=(Bitu)reg_ax*(Bitu)(host_readw(eaa));
                                reg_ax=(Bit16u)(tempu);
                                reg_dx=(Bit16u)(tempu >> 16); FillFlagsNoCFOF();
                                if (reg_ax == 0) reg_flags|=FLAG_ZF; else reg_flags&=~FLAG_ZF;
                                if (reg_dx) {
                                    reg_flags|=FLAG_CF;reg_flags|=FLAG_OF;
                                } else {
                                    reg_flags&=~FLAG_CF;reg_flags&=~FLAG_OF;
                                }
                            }
                        }
                    }
                    break;
                case 0x05:								/* IMUL AX, r/m16 */
                    {
                        if (rm >= 0xc0 ) {
                            Bit16u * earw=lookupRMEAregw[rm];{
                                Bits temps=((Bit16s)reg_ax)*((Bit16s)(*earw));
                                reg_ax=(Bit16s)(temps);
                                reg_dx=(Bit16s)(temps >> 16); FillFlagsNoCFOF();
                                if (((temps & 0xffff8000)==0xffff8000 || (temps & 0xffff8000)==0x0000)) {
                                    reg_flags&=~FLAG_CF;reg_flags&=~FLAG_OF;
                                } else {
                                    reg_flags|=FLAG_CF;reg_flags|=FLAG_OF;
                                }
                            }
                        } else {
                            HostPt eaa=core.ea_table[rm]();{
                                Bits temps=((Bit16s)reg_ax)*((Bit16s)(host_readw(eaa)));
                                reg_ax=(Bit16s)(temps);
                                reg_dx=(Bit16s)(temps >> 16); FillFlagsNoCFOF();
                                if (((temps & 0xffff8000)==0xffff8000 || (temps & 0xffff8000)==0x0000)) {
                                    reg_flags&=~FLAG_CF;reg_flags&=~FLAG_OF;
                                } else {
                                    reg_flags|=FLAG_CF;reg_flags|=FLAG_OF;
                                }
                            }
                        }
                    }
                    break;
                case 0x06:								/* DIV r/m16 */
                    {
                        if (rm >= 0xc0 ) {
                            Bit16u * earw=lookupRMEAregw[rm];{
                                Bitu val=*earw;
                                if (val==0) val=1 ;
                                Bitu num=((Bit32u)reg_dx<<16)|reg_ax;
                                Bitu quo=num/val;
                                Bit16u rem=(Bit16u)(num % val);
                                Bit16u quo16=(Bit16u)(quo&0xffff);
                                reg_dx=rem;
                                reg_ax=quo16;
                            }
                        } else {
                            HostPt eaa=core.ea_table[rm]();{
                                Bitu val=host_readw(eaa);
                                if (val==0) val=1 ;
                                Bitu num=((Bit32u)reg_dx<<16)|reg_ax;
                                Bitu quo=num/val;
                                Bit16u rem=(Bit16u)(num % val);
                                Bit16u quo16=(Bit16u)(quo&0xffff);
                                reg_dx=rem;
                                reg_ax=quo16;
                            }
                        }
                    }
                    break;
                case 0x07:								/* IDIV r/m16 */
                    {
                        if (rm >= 0xc0 ) {
                            Bit16u * earw=lookupRMEAregw[rm];{
                                Bits val=(Bit16s)(*earw);
                                if (val==0) val=1;
                                Bits num=(Bit32s)((reg_dx<<16)|reg_ax);
                                Bits quo=num/val;
                                Bit16s rem=(Bit16s)(num % val);
                                Bit16s quo16s=(Bit16s)quo;
                                reg_dx=rem;
                                reg_ax=quo16s;
                            }
                        } else {
                            HostPt eaa=core.ea_table[rm]();{
                                Bits val=(Bit16s)(host_readw(eaa));
                                if (val==0) val=1;
                                Bits num=(Bit32s)((reg_dx<<16)|reg_ax);
                                Bits quo=num/val;
                                Bit16s rem=(Bit16s)(num % val);
                                Bit16s quo16s=(Bit16s)quo;
                                reg_dx=rem;
                                reg_ax=quo16s;
                            }
                        }
                    }
                    break;
                }
                break;
            }
        case (PREFIX_SIZE+0xf8):
        case (PREFIX_NONE+0xf8):									/* CLC */
            FillFlags();
            reg_flags&=~FLAG_CF;
            break;
        case (PREFIX_SIZE+0xf9):
        case (PREFIX_NONE+0xf9):									/* STC */
            FillFlags();
            reg_flags|=FLAG_CF;
            break;
        case (PREFIX_SIZE+0xfa):
        case (PREFIX_NONE+0xfa):									/* CLI */
            CPU_CLI();
            break;
        case (PREFIX_SIZE+0xfb):
        case (PREFIX_NONE+0xfb):									/* STI */
            CPU_STI();
            break;
        case (PREFIX_SIZE+0xfc):
        case (PREFIX_NONE+0xfc):									/* CLD */
            reg_flags&=~FLAG_DF;

            break;
        case (PREFIX_SIZE+0xfd):
        case (PREFIX_NONE+0xfd):									/* STD */
            reg_flags|=FLAG_DF;

            break;
        case (PREFIX_SIZE+0xfe):
        case (PREFIX_NONE+0xfe):									/* GRP4 r/m8 */
            {
                Bit8u rm=Fetchb();
                Bitu which=(rm>>3)&7;
                switch (which) {
                case 0x00:							/* INC r/m8 */
                    {
                        if (rm >= 0xc0 ) {
                            Bit8u * earb=lookupRMEAregb[rm];if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            lf_var1b=*earb;
                            lf_resb=lf_var1b+1;
                            *earb=lf_resb;
                            lflags.type=t_INCb;
                        } else {
                            HostPt eaa=core.ea_table[rm]();if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            lf_var1b=host_readb(eaa);
                            lf_resb=lf_var1b+1;
                            host_writeb(eaa,lf_resb);
                            lflags.type=t_INCb;
                        }
                    }
                    break;		
                case 0x01:							/* DEC r/m8 */
                    {
                        if (rm >= 0xc0 ) {
                            Bit8u * earb=lookupRMEAregb[rm];if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            lf_var1b=*earb;
                            lf_resb=lf_var1b-1;
                            *earb=lf_resb;
                            lflags.type=t_DECb;
                        } else {
                            HostPt eaa=core.ea_table[rm]();if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            lf_var1b=host_readb(eaa);
                            lf_resb=lf_var1b-1;
                            host_writeb(eaa,lf_resb);
                            lflags.type=t_DECb;
                        }
                    }
                    break;
                case 0x07:							/* CallBack */
                    {
                        Bitu cb=Fetchw();
                        FillFlags();
                        reg_eip=(core.cseip-MemBase);
                        return;
                    }
                default:
                    break;
                }
                break;
            }
        case (PREFIX_SIZE+0xff):									/* GRP5 r/m16 */
            {
                Bit8u rm=Fetchb();
                Bitu which=(rm>>3)&7;
                switch (which) {
                case 0x00:							/* INC r/m16 */
                    {
                        if (rm >= 0xc0 ) {
                            Bit16u * earw=lookupRMEAregw[rm];if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            lf_var1w=*earw;
                            lf_resw=lf_var1w+1;
                            *earw=lf_resw;
                            lflags.type=t_INCw;
                        } else {
                            HostPt eaa=core.ea_table[rm]();if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            lf_var1w=host_readw(eaa);
                            lf_resw=lf_var1w+1;
                            host_writew(eaa,lf_resw);
                            lflags.type=t_INCw;
                        }
                    }
                    break;		
                case 0x01:							/* DEC r/m16 */
                    {
                        if (rm >= 0xc0 ) {
                            Bit16u * earw=lookupRMEAregw[rm];if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            lf_var1w=*earw;
                            lf_resw=lf_var1w-1;
                            *earw=lf_resw;
                            lflags.type=t_DECw;
                        } else {
                            HostPt eaa=core.ea_table[rm]();if (get_CF()) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                            lf_var1w=host_readw(eaa);
                            lf_resw=lf_var1w-1;
                            host_writew(eaa,lf_resw);
                            lflags.type=t_DECw;
                        }
                    }
                    break;		
                case 0x02:							/* CALL r/m16 */
                    if (rm >= 0xc0 ) {
                        Bit16u * earw=lookupRMEAregw[rm];
                        reg_eip=*earw;
                    }
                    else {
                        HostPt eaa=core.ea_table[rm]();
                        reg_eip=host_readw(eaa);
                    }
                    CPU_Push16((core.cseip-MemBase));
                    continue;
                case 0x03:							/* CALL m16:32 */
                    goto illegal_opcode; // not implemented 
                case 0x04:							/* JMP r/m16 */	
                    if (rm >= 0xc0 ) {
                        Bit16u * earw=lookupRMEAregw[rm];
                        reg_eip=*earw;
                    }
                    else {
                        HostPt eaa=core.ea_table[rm]();
                        reg_eip=host_readw(eaa);
                    }
                    continue;
                case 0x05:							/* JMP m16:32 */	
                    goto illegal_opcode; // not implemented 
                case 0x06:							/* PUSH r/m16 */
                    if (rm >= 0xc0 ) {
                        Bit16u * earw=lookupRMEAregw[rm];CPU_Push16(*earw);
                    }
                    else {
                        HostPt eaa=core.ea_table[rm]();CPU_Push16(host_readw(eaa));
                    }
                    break;
                default:
                    break;
                }
                break;
            }




        case ((OPCODE_0F|PREFIX_NONE)+0x80):									/* JO */
            {
                reg_eip=(core.cseip-MemBase);
                if ((get_OF())) reg_eip+=Fetchd();
                reg_eip+=4; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_NONE)+0x81):									/* JNO */
            {
                reg_eip=(core.cseip-MemBase);
                if ((!get_OF())) reg_eip+=Fetchd();
                reg_eip+=4; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_NONE)+0x82):									/* JB */
            {
                reg_eip=(core.cseip-MemBase);
                if ((get_CF())) reg_eip+=Fetchd();
                reg_eip+=4; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_NONE)+0x83):									/* JNB */
            {
                reg_eip=(core.cseip-MemBase);
                if ((!get_CF())) reg_eip+=Fetchd();
                reg_eip+=4; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_NONE)+0x84):									/* JZ */
            {
                reg_eip=(core.cseip-MemBase);
                if ((get_ZF())) reg_eip+=Fetchd();
                reg_eip+=4; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_NONE)+0x85):									/* JNZ */
            {
                reg_eip=(core.cseip-MemBase);
                if ((!get_ZF())) reg_eip+=Fetchd();
                reg_eip+=4; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_NONE)+0x86):									/* JBE */
            {
                reg_eip=(core.cseip-MemBase);
                if ((get_CF() || get_ZF())) reg_eip+=Fetchd();
                reg_eip+=4; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_NONE)+0x87):									/* JNBE */
            {
                reg_eip=(core.cseip-MemBase);
                if ((!get_CF() && !get_ZF())) reg_eip+=Fetchd();
                reg_eip+=4; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_NONE)+0x88):									/* JS */
            {
                reg_eip=(core.cseip-MemBase);
                if ((get_SF())) reg_eip+=Fetchd();
                reg_eip+=4; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_NONE)+0x89):									/* JNS */
            {
                reg_eip=(core.cseip-MemBase);
                if ((!get_SF())) reg_eip+=Fetchd();
                reg_eip+=4; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_NONE)+0x8a):									/* JP */
            {
                reg_eip=(core.cseip-MemBase);
                if ((get_PF())) reg_eip+=Fetchd();
                reg_eip+=4; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_NONE)+0x8b):									/* JNP */
            {
                reg_eip=(core.cseip-MemBase);
                if ((!get_PF())) reg_eip+=Fetchd();
                reg_eip+=4; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_NONE)+0x8c):									/* JL */
            {
                reg_eip=(core.cseip-MemBase);
                if (((get_SF()!=0) != (get_OF()!=0))) reg_eip+=Fetchd();
                reg_eip+=4; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_NONE)+0x8d):									/* JNL */
            {
                reg_eip=(core.cseip-MemBase);
                if (((get_SF()!=0) == (get_OF()!=0))) reg_eip+=Fetchd();
                reg_eip+=4; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_NONE)+0x8e):									/* JLE */
            {
                reg_eip=(core.cseip-MemBase);
                if ((get_ZF() || ((get_SF()!=0) != (get_OF()!=0)))) reg_eip+=Fetchd();
                reg_eip+=4; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_NONE)+0x8f):									/* JNLE */
            {
                reg_eip=(core.cseip-MemBase);
                if ((!get_ZF() && ((get_SF()!=0) == (get_OF()!=0)))) reg_eip+=Fetchd();
                reg_eip+=4; continue;
            }
            break;

        case ((OPCODE_0F|PREFIX_NONE)+0xa3):									/* BT r/m32, r32 */
            {
                FillFlags();
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                Bit32u mask=1 << (*rmrd & 31);
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    if ((*eard & mask)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                } else {
                    HostPt eaa=core.ea_table[rm]();eaa+=(((Bit32s)*rmrd)>>5)*4;
                    Bit32u old=host_readd(eaa);
                    if ((old & mask)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                }
                break;
            }
        case ((OPCODE_0F|PREFIX_NONE)+0xa4):									/* SHLD r/m32, r32, imm8 */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    Bit8u val=Fetchb() & 0x1F;
                    if (!val) break;
                    lf_var2b=val;
                    lf_var1d=*eard;
                    lf_resd=(lf_var1d << lf_var2b) | (*rmrd >> (32-lf_var2b));
                    *eard=lf_resd;
                    lflags.type=t_DSHLd;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    Bit8u val=Fetchb() & 0x1F;
                    if (!val) break;
                    lf_var2b=val;
                    lf_var1d=host_readd(eaa);
                    lf_resd=(lf_var1d << lf_var2b) | (*rmrd >> (32-lf_var2b));
                    host_writed(eaa,lf_resd);
                    lflags.type=t_DSHLd;
                }
            }
            break;
        case ((OPCODE_0F|PREFIX_NONE)+0xa5):									/* SHLD r/m32, r32,CL */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    Bit8u val=reg_cl & 0x1F;
                    if (!val) break;
                    lf_var2b=val;
                    lf_var1d=*eard;
                    lf_resd=(lf_var1d << lf_var2b) | (*rmrd >> (32-lf_var2b));
                    *eard=lf_resd;
                    lflags.type=t_DSHLd;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    Bit8u val=reg_cl & 0x1F;
                    if (!val) break;
                    lf_var2b=val;
                    lf_var1d=host_readd(eaa);
                    lf_resd=(lf_var1d << lf_var2b) | (*rmrd >> (32-lf_var2b));
                    host_writed(eaa,lf_resd);
                    lflags.type=t_DSHLd;
                }
            }
            break;

        case ((OPCODE_0F|PREFIX_NONE)+0xab):									/* BTS r/m32, r32 */
            {
                FillFlags();
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                Bit32u mask=1 << (*rmrd & 31);
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    if ((*eard & mask)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                    *eard|=mask;
                } else {
                    HostPt eaa=core.ea_table[rm]();eaa+=(((Bit32s)*rmrd)>>5)*4;
                    Bit32u old=host_readd(eaa);
                    if ((old & mask)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                    host_writed(eaa,old | mask);
                }
                break;
            }

        case ((OPCODE_0F|PREFIX_NONE)+0xac):									/* SHRD r/m32, r32, imm8 */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    Bit8u val=Fetchb() & 0x1F;
                    if (!val) break;
                    lf_var2b=val;
                    lf_var1d=*eard;
                    lf_resd=(lf_var1d >> lf_var2b) | (*rmrd << (32-lf_var2b));
                    *eard=lf_resd;
                    lflags.type=t_DSHRd;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    Bit8u val=Fetchb() & 0x1F;
                    if (!val) break;
                    lf_var2b=val;
                    lf_var1d=host_readd(eaa);
                    lf_resd=(lf_var1d >> lf_var2b) | (*rmrd << (32-lf_var2b));
                    host_writed(eaa,lf_resd);
                    lflags.type=t_DSHRd;
                }
            }
            break;
        case ((OPCODE_0F|PREFIX_NONE)+0xad):									/* SHRD r/m32, r32,CL */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    Bit8u val=reg_cl & 0x1F;
                    if (!val) break;
                    lf_var2b=val;
                    lf_var1d=*eard;
                    lf_resd=(lf_var1d >> lf_var2b) | (*rmrd << (32-lf_var2b));
                    *eard=lf_resd;
                    lflags.type=t_DSHRd;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    Bit8u val=reg_cl & 0x1F;
                    if (!val) break;
                    lf_var2b=val;
                    lf_var1d=host_readd(eaa);
                    lf_resd=(lf_var1d >> lf_var2b) | (*rmrd << (32-lf_var2b));
                    host_writed(eaa,lf_resd);
                    lflags.type=t_DSHRd;
                }
            }
            break;
        case ((OPCODE_0F|PREFIX_NONE)+0xaf):									/* IMUL r32, r/m32 */
            {
                {
                    Bit8u rm=Fetchb();
                    Bit32u * rmrd;
                    rmrd=lookupRMregd[rm];
                    if (rm >= 0xc0 ) {
                        Bit32u * eard=lookupRMEAregd[rm];{
                            Bit64s res=((Bit64s)((Bit32s)*eard))*((Bit64s)((Bit32s)*rmrd));
                            *rmrd=(Bit32s)res; FillFlagsNoCFOF();
                            if ((res>-((Bit64s)(2147483647)+1)) && (res<(Bit64s)2147483647)) {
                                reg_flags&=~FLAG_CF;reg_flags&=~FLAG_OF;
                            } else {
                                reg_flags|=FLAG_CF;reg_flags|=FLAG_OF;
                            }
                        }
                    } else {
                        HostPt eaa=core.ea_table[rm]();{
                            Bit64s res=((Bit64s)((Bit32s)host_readd(eaa)))*((Bit64s)((Bit32s)*rmrd));
                            *rmrd=(Bit32s)res; FillFlagsNoCFOF();
                            if ((res>-((Bit64s)(2147483647)+1)) && (res<(Bit64s)2147483647)) {
                                reg_flags&=~FLAG_CF;reg_flags&=~FLAG_OF;
                            } else {
                                reg_flags|=FLAG_CF;reg_flags|=FLAG_OF;
                            }
                        }
                    }
                }
                break;
            }

        case ((OPCODE_0F|PREFIX_NONE)+0xb3):									/* BTR r/m32, r32 */
            {
                FillFlags();
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                Bit32u mask=1 << (*rmrd & 31);
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    if ((*eard & mask)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                    *eard&= ~mask;
                } else {
                    HostPt eaa=core.ea_table[rm]();eaa+=(((Bit32s)*rmrd)>>5)*4;
                    Bit32u old=host_readd(eaa);
                    if ((old & mask)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                    host_writed(eaa,old & ~mask);
                }
                break;
            }

        case ((OPCODE_0F|PREFIX_NONE)+0xb6):									/* MOVZX r32, r/m8 */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];															
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    *rmrd=*earb;
                }
                else {
                    HostPt eaa=core.ea_table[rm]();
                    *rmrd=host_readb(eaa);
                }
                break;
            }
        case ((OPCODE_0F|PREFIX_NONE)+0xb7):									/* MOVXZ r32, r/m16 */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    *rmrd=*earw;
                }
                else {
                    HostPt eaa=core.ea_table[rm]();
                    *rmrd=host_readw(eaa);
                }
                break;
            }
        case ((OPCODE_0F|PREFIX_NONE)+0xba):									/* GRP8 r/m32, imm8 */
            {
                FillFlags();
                Bit8u rm=Fetchb();
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    Bit32u mask=1 << (Fetchb() & 31);
                    if ((*eard & mask)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                    switch (rm & 0x38) {
                    case 0x20:								/* BT */
                        break;
                    case 0x28:								/* BTS */
                        *eard|=mask;
                        break;
                    case 0x30:								/* BTR */
                        *eard&=~mask;
                        break;
                    case 0x38:								/* BTC */
                        if ((reg_flags & FLAG_CF)) *eard&=~mask;
                        else *eard|=mask;
                        break;
                    default:
                        printf("CPU:66:0F:BA:Illegal subfunction %X\n",rm & 0x38);
                    }
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    Bit32u old=host_readd(eaa);
                    Bit32u mask=1 << (Fetchb() & 31);
                    if ((old & mask)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                    switch (rm & 0x38) {
                    case 0x20:								/* BT */
                        break;
                    case 0x28:								/* BTS */
                        host_writed(eaa,old|mask);
                        break;
                    case 0x30:								/* BTR */
                        host_writed(eaa,old & ~mask);
                        break;
                    case 0x38:								/* BTC */
                        if ((reg_flags & FLAG_CF)) old&=~mask;
                        else old|=mask;
                        host_writed(eaa,old);
                        break;
                    default:
                        printf("CPU:66:0F:BA:Illegal subfunction %X\n",rm & 0x38);
                    }
                }
                break;
            }
        case ((OPCODE_0F|PREFIX_NONE)+0xbb):									/* BTC r/m32, r32 */
            {
                FillFlags();
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                Bit32u mask=1 << (*rmrd & 31);
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    if ((*eard & mask)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                    *eard^=mask;
                } else {
                    HostPt eaa=core.ea_table[rm]();eaa+=(((Bit32s)*rmrd)>>5)*4;
                    Bit32u old=host_readd(eaa);
                    if ((old & mask)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                    host_writed(eaa,old ^ mask);
                }
                break;
            }
        case ((OPCODE_0F|PREFIX_NONE)+0xbc):									/* BSF r32, r/m32 */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                Bit32u result,value;
                if (rm >= 0xc0) {
                    Bit32u * eard=lookupRMEAregd[rm]; value=*eard;
                } else			{
                    HostPt eaa=core.ea_table[rm](); value=host_readd(eaa);
                }
                if (value==0) {
                    reg_flags|=FLAG_ZF;
                } else {
                    result = 0;
                    while ((value & 0x01)==0) {
                        result++; value>>=1;
                    }
                    reg_flags&=~FLAG_ZF;
                    *rmrd = result;
                }
                lflags.type=t_UNKNOWN;
                break;
            }
        case ((OPCODE_0F|PREFIX_NONE)+0xbd):									/*  BSR r32, r/m32 */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                Bit32u result,value;
                if (rm >= 0xc0) {
                    Bit32u * eard=lookupRMEAregd[rm]; value=*eard;
                } else			{
                    HostPt eaa=core.ea_table[rm](); value=host_readd(eaa);
                }
                if (value==0) {
                    reg_flags|=FLAG_ZF;
                } else {
                    result = 31;	
                    while ((value & 0x80000000)==0) {
                        result--; value<<=1;
                    }
                    reg_flags&=~FLAG_ZF;
                    *rmrd = result;
                }
                lflags.type=t_UNKNOWN;
                break;
            }
        case ((OPCODE_0F|PREFIX_NONE)+0xbe):									/* MOVSX r32, r/m8 */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];															
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    *rmrd=*(Bit8s *)earb;
                }
                else {
                    HostPt eaa=core.ea_table[rm]();
                    *rmrd=host_readb(eaa);
                }
                break;
            }
        case ((OPCODE_0F|PREFIX_NONE)+0xbf):									/* MOVSX r32, r/m16 */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];															
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    *rmrd=*(Bit16s *)earw;
                }
                else {
                    HostPt eaa=core.ea_table[rm]();
                    *rmrd=host_readw(eaa);
                }
                break;
            }
        case ((OPCODE_0F|PREFIX_NONE)+0xc1):									/* XADD r32, r/m32 */
            {
                Bit8u rm=Fetchb();
                Bit32u * rmrd;
                rmrd=lookupRMregd[rm];
                Bit32u oldrmrd=*rmrd;
                if (rm >= 0xc0 ) {
                    Bit32u * eard=lookupRMEAregd[rm];
                    *rmrd=*eard;
                    *eard+=oldrmrd;
                }
                else {
                    HostPt eaa=core.ea_table[rm]();
                    *rmrd=host_readd(eaa);
                    host_writed(eaa,host_readd(eaa)+oldrmrd);
                }
                break;
            }



        case ((OPCODE_0F|PREFIX_SIZE)+0x80):									/* JO */
            {
                reg_eip=(core.cseip-MemBase);
                if ((get_OF())) reg_ip+=Fetchw();
                reg_ip+=2; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x81):									/* JNO */
            {
                reg_eip=(core.cseip-MemBase);
                if ((!get_OF())) reg_ip+=Fetchw();
                reg_ip+=2; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x82):									/* JB */
            {
                reg_eip=(core.cseip-MemBase);
                if ((get_CF())) reg_ip+=Fetchw();
                reg_ip+=2; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x83):									/* JNB */
            {
                reg_eip=(core.cseip-MemBase);
                if ((!get_CF())) reg_ip+=Fetchw();
                reg_ip+=2; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x84):									/* JZ */
            {
                reg_eip=(core.cseip-MemBase);
                if ((get_ZF())) reg_ip+=Fetchw();
                reg_ip+=2; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x85):									/* JNZ */
            {
                reg_eip=(core.cseip-MemBase);
                if ((!get_ZF())) reg_ip+=Fetchw();
                reg_ip+=2; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x86):									/* JBE */
            {
                reg_eip=(core.cseip-MemBase);
                if ((get_CF() || get_ZF())) reg_ip+=Fetchw();
                reg_ip+=2; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x87):									/* JNBE */
            {
                reg_eip=(core.cseip-MemBase);
                if ((!get_CF() && !get_ZF())) reg_ip+=Fetchw();
                reg_ip+=2; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x88):									/* JS */
            {
                reg_eip=(core.cseip-MemBase);
                if ((get_SF())) reg_ip+=Fetchw();
                reg_ip+=2; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x89):									/* JNS */
            {
                reg_eip=(core.cseip-MemBase);
                if ((!get_SF())) reg_ip+=Fetchw();
                reg_ip+=2; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x8a):									/* JP */
            {
                reg_eip=(core.cseip-MemBase);
                if ((get_PF())) reg_ip+=Fetchw();
                reg_ip+=2; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x8b):									/* JNP */
            {
                reg_eip=(core.cseip-MemBase);
                if ((!get_PF())) reg_ip+=Fetchw();
                reg_ip+=2; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x8c):									/* JL */
            {
                reg_eip=(core.cseip-MemBase);
                if (((get_SF()!=0) != (get_OF()!=0))) reg_ip+=Fetchw();
                reg_ip+=2; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x8d):									/* JNL */
            {
                reg_eip=(core.cseip-MemBase);
                if (((get_SF()!=0) == (get_OF()!=0))) reg_ip+=Fetchw();
                reg_ip+=2; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x8e):									/* JLE */
            {
                reg_eip=(core.cseip-MemBase);
                if ((get_ZF() || ((get_SF()!=0) != (get_OF()!=0)))) reg_ip+=Fetchw();
                reg_ip+=2; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x8f):									/* JNLE */
            {
                reg_eip=(core.cseip-MemBase);
                if ((!get_ZF() && ((get_SF()!=0) == (get_OF()!=0)))) reg_ip+=Fetchw();
                reg_ip+=2; continue;
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x90):
        case ((OPCODE_0F|PREFIX_NONE)+0x90):									/* SETO */
            {
                Bit8u rm=Fetchb();
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    *earb=((get_OF())) ? 1 : 0;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    host_writeb(eaa,((get_OF())) ? 1 : 0);
                }
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x91):
        case ((OPCODE_0F|PREFIX_NONE)+0x91):									/* SETNO */
            {
                Bit8u rm=Fetchb();
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    *earb=((!get_OF())) ? 1 : 0;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    host_writeb(eaa,((!get_OF())) ? 1 : 0);
                }
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x92):
        case ((OPCODE_0F|PREFIX_NONE)+0x92):									/* SETB */
            {
                Bit8u rm=Fetchb();
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    *earb=((get_CF())) ? 1 : 0;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    host_writeb(eaa,((get_CF())) ? 1 : 0);
                }
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x93):
        case ((OPCODE_0F|PREFIX_NONE)+0x93):									/* SETNB */
            {
                Bit8u rm=Fetchb();
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    *earb=((!get_CF())) ? 1 : 0;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    host_writeb(eaa,((!get_CF())) ? 1 : 0);
                }
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x94):
        case ((OPCODE_0F|PREFIX_NONE)+0x94):									/* SETZ */
            {
                Bit8u rm=Fetchb();
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    *earb=((get_ZF())) ? 1 : 0;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    host_writeb(eaa,((get_ZF())) ? 1 : 0);
                }
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x95):
        case ((OPCODE_0F|PREFIX_NONE)+0x95):									/* SETNZ */
            {
                Bit8u rm=Fetchb();
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    *earb=((!get_ZF())) ? 1 : 0;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    host_writeb(eaa,((!get_ZF())) ? 1 : 0);
                }
            }	break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x96):
        case ((OPCODE_0F|PREFIX_NONE)+0x96):									/* SETBE */
            {
                Bit8u rm=Fetchb();
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    *earb=((get_CF() || get_ZF())) ? 1 : 0;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    host_writeb(eaa,((get_CF() || get_ZF())) ? 1 : 0);
                }
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x97):
        case ((OPCODE_0F|PREFIX_NONE)+0x97):									/* SETNBE */
            {
                Bit8u rm=Fetchb();
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    *earb=((!get_CF() && !get_ZF())) ? 1 : 0;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    host_writeb(eaa,((!get_CF() && !get_ZF())) ? 1 : 0);
                }
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x98):
        case ((OPCODE_0F|PREFIX_NONE)+0x98):									/* SETS */
            {
                Bit8u rm=Fetchb();
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    *earb=((get_SF())) ? 1 : 0;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    host_writeb(eaa,((get_SF())) ? 1 : 0);
                }
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x99):
        case ((OPCODE_0F|PREFIX_NONE)+0x99):									/* SETNS */
            {
                Bit8u rm=Fetchb();
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    *earb=((!get_SF())) ? 1 : 0;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    host_writeb(eaa,((!get_SF())) ? 1 : 0);
                }
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x9a):
        case ((OPCODE_0F|PREFIX_NONE)+0x9a):									/* SETP */
            {
                Bit8u rm=Fetchb();
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    *earb=((get_PF())) ? 1 : 0;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    host_writeb(eaa,((get_PF())) ? 1 : 0);
                }
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x9b):
        case ((OPCODE_0F|PREFIX_NONE)+0x9b):									/* SETNP */
            {
                Bit8u rm=Fetchb();
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    *earb=((!get_PF())) ? 1 : 0;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    host_writeb(eaa,((!get_PF())) ? 1 : 0);
                }
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x9c):
        case ((OPCODE_0F|PREFIX_NONE)+0x9c):									/* SETL */
            {
                Bit8u rm=Fetchb();
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    *earb=(((get_SF()!=0) != (get_OF()!=0))) ? 1 : 0;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    host_writeb(eaa,(((get_SF()!=0) != (get_OF()!=0))) ? 1 : 0);
                }
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x9d):
        case ((OPCODE_0F|PREFIX_NONE)+0x9d):									/* SETNL */
            {
                Bit8u rm=Fetchb();
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    *earb=(((get_SF()!=0) == (get_OF()!=0))) ? 1 : 0;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    host_writeb(eaa,(((get_SF()!=0) == (get_OF()!=0))) ? 1 : 0);
                }
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x9e):
        case ((OPCODE_0F|PREFIX_NONE)+0x9e):									/* SETLE */
            {
                Bit8u rm=Fetchb();
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    *earb=((get_ZF() || ((get_SF()!=0) != (get_OF()!=0)))) ? 1 : 0;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    host_writeb(eaa,((get_ZF() || ((get_SF()!=0) != (get_OF()!=0)))) ? 1 : 0);
                }
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0x9f):
        case ((OPCODE_0F|PREFIX_NONE)+0x9f):									/* SETNLE */
            {
                Bit8u rm=Fetchb();
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    *earb=((!get_ZF() && ((get_SF()!=0) == (get_OF()!=0)))) ? 1 : 0;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    host_writeb(eaa,((!get_ZF() && ((get_SF()!=0) == (get_OF()!=0)))) ? 1 : 0);
                }
            }
            break;

        case ((OPCODE_0F|PREFIX_SIZE)+0xa2):
        case ((OPCODE_0F|PREFIX_NONE)+0xa2):									/* CPUID */
            reg_eax=0;
            reg_ebx=0;
            reg_ecx=0;
            reg_edx=0;
            break;

        case ((OPCODE_0F|PREFIX_SIZE)+0xa3):									/* BT r/m16, r16 */
            {
                FillFlags();
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                Bit16u mask=1 << (*rmrw & 15);
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    if ((*earw & mask)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                } else {
                    HostPt eaa=core.ea_table[rm]();eaa+=(((Bit16s)*rmrw)>>4)*2;
                    Bit16u old=host_readw(eaa);
                    if ((old & mask)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                }
                break;
            }
        case ((OPCODE_0F|PREFIX_SIZE)+0xa4):									/* SHLD r/m16, r16, imm8 */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    Bit8u val=Fetchb() & 0x1F;
                    if (!val) break;
                    lf_var2b=val;
                    lf_var1d=(*earw<<16)|*rmrw;
                    Bit32u tempd=lf_var1d << lf_var2b;
                    if (lf_var2b>16) tempd |= (*rmrw << (lf_var2b - 16));
                    lf_resw=(Bit16u)(tempd >> 16);
                    *earw=lf_resw;
                    lflags.type=t_DSHLw;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    Bit8u val=Fetchb() & 0x1F;
                    if (!val) break;
                    lf_var2b=val;
                    lf_var1d=(host_readw(eaa)<<16)|*rmrw;
                    Bit32u tempd=lf_var1d << lf_var2b;
                    if (lf_var2b>16) tempd |= (*rmrw << (lf_var2b - 16));
                    lf_resw=(Bit16u)(tempd >> 16);
                    host_writew(eaa,lf_resw);
                    lflags.type=t_DSHLw;
                }
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0xa5):									/* SHLD r/m16, r16,CL */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    Bit8u val=reg_cl & 0x1F;
                    if (!val) break;
                    lf_var2b=val;
                    lf_var1d=(*earw<<16)|*rmrw;
                    Bit32u tempd=lf_var1d << lf_var2b;
                    if (lf_var2b>16) tempd |= (*rmrw << (lf_var2b - 16));
                    lf_resw=(Bit16u)(tempd >> 16);
                    *earw=lf_resw;
                    lflags.type=t_DSHLw;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    Bit8u val=reg_cl & 0x1F;
                    if (!val) break;
                    lf_var2b=val;
                    lf_var1d=(host_readw(eaa)<<16)|*rmrw;
                    Bit32u tempd=lf_var1d << lf_var2b;
                    if (lf_var2b>16) tempd |= (*rmrw << (lf_var2b - 16));
                    lf_resw=(Bit16u)(tempd >> 16);
                    host_writew(eaa,lf_resw);
                    lflags.type=t_DSHLw;
                }
            }
            break;

        case ((OPCODE_0F|PREFIX_SIZE)+0xab):									/* BTS r/m16, r16 */
            {
                FillFlags();
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                Bit16u mask=1 << (*rmrw & 15);
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    if ((*earw & mask)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                    *earw|=mask;
                } else {
                    HostPt eaa=core.ea_table[rm]();eaa+=(((Bit16s)*rmrw)>>4)*2;
                    Bit16u old=host_readw(eaa);
                    if ((old & mask)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                    host_writew(eaa,old | mask);
                }
                break;
            }
        case ((OPCODE_0F|PREFIX_SIZE)+0xac):									/* SHRD r/m16, r16, imm8 */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    Bit8u val=Fetchb() & 0x1F;
                    if (!val) break;
                    lf_var2b=val;
                    lf_var1d=(*rmrw<<16)|*earw;
                    Bit32u tempd=lf_var1d >> lf_var2b;
                    if (lf_var2b>16) tempd |= (*rmrw << (32-lf_var2b ));
                    lf_resw=(Bit16u)(tempd);
                    *earw=lf_resw;
                    lflags.type=t_DSHRw;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    Bit8u val=Fetchb() & 0x1F;
                    if (!val) break;
                    lf_var2b=val;
                    lf_var1d=(*rmrw<<16)|host_readw(eaa);
                    Bit32u tempd=lf_var1d >> lf_var2b;
                    if (lf_var2b>16) tempd |= (*rmrw << (32-lf_var2b ));
                    lf_resw=(Bit16u)(tempd);
                    host_writew(eaa,lf_resw);
                    lflags.type=t_DSHRw;
                }
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0xad):									/* SHRD r/m16, r16,CL */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    Bit8u val=reg_cl & 0x1F;
                    if (!val) break;
                    lf_var2b=val;
                    lf_var1d=(*rmrw<<16)|*earw;
                    Bit32u tempd=lf_var1d >> lf_var2b;
                    if (lf_var2b>16) tempd |= (*rmrw << (32-lf_var2b ));
                    lf_resw=(Bit16u)(tempd);
                    *earw=lf_resw;
                    lflags.type=t_DSHRw;
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    Bit8u val=reg_cl & 0x1F;
                    if (!val) break;
                    lf_var2b=val;
                    lf_var1d=(*rmrw<<16)|host_readw(eaa);
                    Bit32u tempd=lf_var1d >> lf_var2b;
                    if (lf_var2b>16) tempd |= (*rmrw << (32-lf_var2b ));
                    lf_resw=(Bit16u)(tempd);
                    host_writew(eaa,lf_resw);
                    lflags.type=t_DSHRw;
                }
            }
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0xaf):									/* IMUL r16, r/m16 */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];{
                        Bits res=((Bit16s)*earw) * ((Bit16s)*rmrw);
                        *rmrw=res & 0xffff; FillFlagsNoCFOF();
                        if ((res> -32768) && (res<32767)) {
                            reg_flags&=~FLAG_CF;reg_flags&=~FLAG_OF;
                        } else {
                            reg_flags|=FLAG_CF;reg_flags|=FLAG_OF;
                        }
                    }
                } else {
                    HostPt eaa=core.ea_table[rm]();{
                        Bits res=((Bit16s)host_readw(eaa)) * ((Bit16s)*rmrw);
                        *rmrw=res & 0xffff; FillFlagsNoCFOF();
                        if ((res> -32768) && (res<32767)) {
                            reg_flags&=~FLAG_CF;reg_flags&=~FLAG_OF;
                        } else {
                            reg_flags|=FLAG_CF;reg_flags|=FLAG_OF;
                        }
                    }
                }
            }
            break;

        case ((OPCODE_0F|PREFIX_SIZE)+0xb3):									/* BTR r/m16, r16 */
            {
                FillFlags();
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                Bit16u mask=1 << (*rmrw & 15);
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    if ((*earw & mask)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                    *earw&= ~mask;
                } else {
                    HostPt eaa=core.ea_table[rm]();eaa+=(((Bit16s)*rmrw)>>4)*2;
                    Bit16u old=host_readw(eaa);
                    if ((old & mask)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                    host_writew(eaa,old & ~mask);
                }
                break;
            }

        case ((OPCODE_0F|PREFIX_SIZE)+0xb6):									/* MOVZX r16, r/m8 */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];															
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    *rmrw=*earb;
                }
                else {
                    HostPt eaa=core.ea_table[rm]();
                    *rmrw=host_readb(eaa);
                }
                break;
            }
        case ((OPCODE_0F|PREFIX_SIZE)+0xb7):									/* MOVZX r16, r/m16 */
        case ((OPCODE_0F|PREFIX_SIZE)+0xbf):									/* MOVSX r16, r/m16 */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];															
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    *rmrw=*earw;
                }
                else {
                    HostPt eaa=core.ea_table[rm]();
                    *rmrw=host_readw(eaa);
                }
                break;
            }
        case ((OPCODE_0F|PREFIX_SIZE)+0xba):									/* GRP8 r/m16, imm8 */
            {
                FillFlags();
                Bit8u rm=Fetchb();
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    Bit16u mask=1 << (Fetchb() & 15);
                    if ((*earw & mask)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                    switch (rm & 0x38) {
                    case 0x20:							/* BT */
                        break;
                    case 0x28:							/* BTS */
                        *earw|=mask;
                        break;
                    case 0x30:							/* BTR */
                        *earw&= ~mask;
                        break;
                    case 0x38:							/* BTC */
                        *earw^=mask;
                        break;
                    default:
                        printf("CPU:0F:BA:Illegal subfunction %X\n",rm & 0x38);
                    }
                } else {
                    HostPt eaa=core.ea_table[rm]();
                    Bit16u old=host_readw(eaa);
                    Bit16u mask=1 << (Fetchb() & 15);
                    if ((old & mask)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                    switch (rm & 0x38) {
                    case 0x20:							/* BT */
                        break;
                    case 0x28:							/* BTS */
                        host_writew(eaa,old|mask);
                        break;
                    case 0x30:							/* BTR */
                        host_writew(eaa,old & ~mask);
                        break;
                    case 0x38:							/* BTC */
                        host_writew(eaa,old ^ mask);
                        break;
                    default:
                        printf("CPU:0F:BA:Illegal subfunction %X\n",rm & 0x38);
                    }
                }
                break;
            }
        case ((OPCODE_0F|PREFIX_SIZE)+0xbb):									/* BTC r/m16, r16 */
            {
                FillFlags();
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                Bit16u mask=1 << (*rmrw & 15);
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    if ((*earw & mask)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                    *earw^=mask;
                } else {
                    HostPt eaa=core.ea_table[rm]();eaa+=(((Bit16s)*rmrw)>>4)*2;
                    Bit16u old=host_readw(eaa);
                    if ((old & mask)) reg_flags|=FLAG_CF; else reg_flags&=~FLAG_CF;
                    host_writew(eaa,old ^ mask);
                }
                break;
            }
        case ((OPCODE_0F|PREFIX_SIZE)+0xbc):									/* BSF r16, r/m16 */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                Bit16u result,value;
                if (rm >= 0xc0) {
                    Bit16u * earw=lookupRMEAregw[rm]; value=*earw;
                } else			{
                    HostPt eaa=core.ea_table[rm](); value=host_readw(eaa);
                }
                if (value==0) {
                    reg_flags|=FLAG_ZF;
                } else {
                    result = 0;
                    while ((value & 0x01)==0) {
                        result++; value>>=1;
                    }
                    reg_flags&=~FLAG_ZF;
                    *rmrw = result;
                }
                lflags.type=t_UNKNOWN;
                break;
            }
        case ((OPCODE_0F|PREFIX_SIZE)+0xbd):									/* BSR r16, r/m16 */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                Bit16u result,value;
                if (rm >= 0xc0) {
                    Bit16u * earw=lookupRMEAregw[rm]; value=*earw;
                } else			{
                    HostPt eaa=core.ea_table[rm](); value=host_readw(eaa);
                }
                if (value==0) {
                    reg_flags|=FLAG_ZF;
                } else {
                    result = 15;	
                    while ((value & 0x8000)==0) {
                        result--; value<<=1;
                    }
                    reg_flags&=~FLAG_ZF;
                    *rmrw = result;
                }
                lflags.type=t_UNKNOWN;
                break;
            }
        case ((OPCODE_0F|PREFIX_SIZE)+0xbe):									/* MOVSX r16, r/m8 */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];															
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    *rmrw=*(Bit8s *)earb;
                }
                else {
                    HostPt eaa=core.ea_table[rm]();
                    *rmrw=host_readb(eaa);
                }
                break;
            }

        case ((OPCODE_0F|PREFIX_SIZE)+0xc0):
        case ((OPCODE_0F|PREFIX_NONE)+0xc0):									/* XADD r8, r/m8 */
            {
                Bit8u rm=Fetchb();
                Bit8u * rmrb;
                rmrb=lookupRMregb[rm];
                Bit8u oldrmrb=*rmrb;
                if (rm >= 0xc0 ) {
                    Bit8u * earb=lookupRMEAregb[rm];
                    *rmrb=*earb;
                    *earb+=oldrmrb;
                }
                else {
                    HostPt eaa=core.ea_table[rm]();
                    *rmrb=host_readb(eaa);
                    host_writeb(eaa,host_readb(eaa)+oldrmrb);
                }
                break;
            }
        case ((OPCODE_0F|PREFIX_SIZE)+0xc1):									/* XADD r16, r/m16 */
            {
                Bit8u rm=Fetchb();
                Bit16u * rmrw;
                rmrw=lookupRMregw[rm];
                Bit16u oldrmrw=*rmrw;
                if (rm >= 0xc0 ) {
                    Bit16u * earw=lookupRMEAregw[rm];
                    *rmrw=*earw;
                    *earw+=oldrmrw;
                }
                else {
                    HostPt eaa=core.ea_table[rm]();
                    *rmrw=host_readw(eaa);
                    host_writew(eaa,host_readw(eaa)+oldrmrw);
                }
                break;
            }
        case ((OPCODE_0F|PREFIX_SIZE)+0xc8):
        case ((OPCODE_0F|PREFIX_NONE)+0xc8):									/* BSWAP EAX */
            reg_eax = (reg_eax>>24)|((reg_eax>>8)&0xFF00)|((reg_eax<<8)&0xFF0000)|((reg_eax<<24)&0xFF000000);
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0xc9):
        case ((OPCODE_0F|PREFIX_NONE)+0xc9):									/* BSWAP ECX */
            reg_ecx = (reg_ecx>>24)|((reg_ecx>>8)&0xFF00)|((reg_ecx<<8)&0xFF0000)|((reg_ecx<<24)&0xFF000000);
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0xca):
        case ((OPCODE_0F|PREFIX_NONE)+0xca):									/* BSWAP EDX */
            reg_edx = (reg_edx>>24)|((reg_edx>>8)&0xFF00)|((reg_edx<<8)&0xFF0000)|((reg_edx<<24)&0xFF000000);
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0xcb):
        case ((OPCODE_0F|PREFIX_NONE)+0xcb):									/* BSWAP EBX */
            reg_ebx = (reg_ebx>>24)|((reg_ebx>>8)&0xFF00)|((reg_ebx<<8)&0xFF0000)|((reg_ebx<<24)&0xFF000000);
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0xcc):
        case ((OPCODE_0F|PREFIX_NONE)+0xcc):									/* BSWAP ESP */
            reg_esp = (reg_esp>>24)|((reg_esp>>8)&0xFF00)|((reg_esp<<8)&0xFF0000)|((reg_esp<<24)&0xFF000000);
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0xcd):
        case ((OPCODE_0F|PREFIX_NONE)+0xcd):									/* BSWAP EBP */
            reg_ebp = (reg_ebp>>24)|((reg_ebp>>8)&0xFF00)|((reg_ebp<<8)&0xFF0000)|((reg_ebp<<24)&0xFF000000);
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0xce):
        case ((OPCODE_0F|PREFIX_NONE)+0xce):									/* BSWAP ESI */
            reg_esi = (reg_esi>>24)|((reg_esi>>8)&0xFF00)|((reg_esi<<8)&0xFF0000)|((reg_esi<<24)&0xFF000000);
            break;
        case ((OPCODE_0F|PREFIX_SIZE)+0xcf):
        case ((OPCODE_0F|PREFIX_NONE)+0xcf):									/* BSWAP EDI */
            reg_edi = (reg_edi>>24)|((reg_edi>>8)&0xFF00)|((reg_edi<<8)&0xFF0000)|((reg_edi<<24)&0xFF000000);
            break;


        default:
    illegal_opcode:
            printf("*** opcode not supported ***\n");
            running = false;
            break;
        }
        reg_eip=(core.cseip-MemBase);
        if ( trap )
            print_trace();
    }
    FillFlags();
    return;
}

int main(int argc, char* argv[]) {
    if (argc < 2 ) { printf("Could not open input file\n"); exit(1);
    }
    FILE *src = fopen(argv[1], "r");
    if (src == 0) { printf("Could not open input file\n"); exit(1);
    }
    MEM_Init(1);
    int number;
    int index=0;
    while (fscanf(src, "%x", &number) == 1) {
        MemBase[index++]=number % 256;
    }

    reg_eip=0;
    core.base_ds = core.base_ss = MemBase;
    CPU_Core_Simple_Run();
    return 0;
}
